      program wdxd
c......................................................................
c  wdxd -- white dwarf experiment (with) diffusion (profiles)
c......................................................................
c Last modified on 08 19 94
c......................................................................
c
c  This is the white dwarf evolution code as written by Martin
c  Schwarzschild. Subsequent modifications made by: 
c  Version         Author(s)
c  WDEC 1.0        Kutter & Savedoff   (1969, ApJ, 157, 1021)
c  WDEC 2.0        D.Q. Lamb           (1975, ApJ, 200, 306)
c  WDEC 3.0        D.E. Winget
c  WDEC 4.0        S.D. Kawaler
c  WDEC 5.0        M.A. Wood           (aka WDXD)
c  WDEC 6.0        P.A. Bradley        (aka WDXDL, WDEC6)
c
c  This particular version evolves c/o white dwarfs, using the old
c  radiative opacities, and new conductive opacities for the liquid
c  metal (Itoh et al 1983, ApJ, 273, 774) and crystalline lattice
c  (Itoh et al. 1984, ApJ, 285, 758).  The envelopes contain
c  reasonable approximations to the expected diffusion profiles, and
c  the thermodynamic quantities are interpolated using the additive
c  volume technique.
c  Recent additions allow this code to compute smoother quantities
c  needed for pulsation calculations and it also computes the Modified
c  Ledoux term for the Brunt-Vaisala frequency.
c
c......................................................................
c  summary of changes made:
c
c  6/10/88:  expanded interior eos table.  now reads 39 isotherms spanning
c            5.1 <= log t <= 8.9.  full eos table now stored on pf4 as
c            fulleos, along with fullhed.
c
c  6/13/88:  adding check on envelope eos.  program will now echo to
c            tty if we try to extrapolate off the envelope eos.  also 
c            echos to tape6.
c
c            adding write statements so that user will now know more about
c            what is actually going on.  for example, code now tells echos
c            to tty when a new envelope is calculated, and prints a more
c            complete summary to tty than simply mod no and temp.
c            also updated the information written to tape1. 
c            in the header, i now write the stellar mass, timestep,
c            stop mass, and number of steps per pressure scale height 
c            used in the envelope integration.  in the summary lines
c            i took out fcc, bcc, and smass, and added amxold.
c
c  6/25/88:  put in fix to vca plague.  if coarse or very coarse approximation
c            occurs, then back off time step, change del(g), put
c            contents of 'stash' back into 'y', and call write1 just
c            like we're starting fresh. 
c
c  7/12/88: files modified (and how)
c      begin:   common/shells/ and common/temp/ modified (add o)
c      calc:   sim to begin, plus add o interp.
c      end:    add o, add recalc to coarse approx also
c      homo:   add o.
c      interp: add o. add xc, xo interpolation too.
c      istate: add o. interpolate in both c and o tables. interpolate 
c              to mixture using additive volume technique.
c              added different freezing prescriptions:
c                 iswch = 1   ==>  pfreeze = phasec(t)
c                 iswch = 2   ==>  pfreeze = phaseo(t)
c                 iswch = 3   ==>  pfreeze = xc*phasec(t) + xo*phaseo(t)
c                 iswch = 4   ==>  pfreeze[c,o] = phase[c,o](t)  (two fronts).
c      opac:   if d > 1.8, use itoh conductive opac
c              if 1.8 > d > 1.5 interpolate b/n itoh and hubbard-lampe
c              if d < 1.5  use hubbard lampe opacities only 
c
c  8/4/88:  specify desired c/o profile in 'input' on lines 3-n, where
c           each line contains the pair (q,x12).  for example, to
c           specify a model where there's a 50/50 mix of c/o in the
c           core, the two lines would be
c                   0. .5
c                   1. .5
c           up to 10 lines are currently allowed.  also 'interp' will 
c           zone to put these mass points on zone boundaries so that
c           the profile doesn't spread out.  in calc before the call
c           to istatco, the x12 and x16 values are now calculated with
c           from these input lines instead of interpolating in the
c           old model (this will have to change if we ever put in
c           phase separation, or when we put in convection).
c
c 12/6/88:  put in ability to read in desired gamma xtal.  it now determines
c           if it's frozen by, once it gets into istate, call getdens 
c           (which returns density), computes local gamma, and if greater
c           than gamxtal (see below), then it's crystallized.
c           when putting this in, i took out the ability for freezing 
c           at c, o, c+o, or separate c and o (ie, ixswtch doesn't do 
c           anything anymore).  (this is a special version, not this one).
c
c  2/1/89:  wdxd conversion started: version of the code which includes
c           c/o interior and (near-) diffusion profile envelopes.
c           begin conversion from wdx --> wdxd.  first subconversion will
c           still compute discontinuous envelopes, and will still interpolate
c           between the triangle envelopes as usual, but the code will be
c           *in place* which will compute mixtures by the additive volume
c           technique.  i do this by simply calling ccomp (new routine) which
c           given [amhyhe, amheca, structure variables] returns the vector
c           x(4) where:
c               x(1) = ( h  mass fraction )
c               x(2) = ( he mass fraction )
c               x(3) = ( c  mass fraction )
c               x(4) = ( o  mass fraction )
c           and where sum(x(i)) = 1.0 is obviously enforced.  given x(),
c           the (new routine) estatem will call estate for each element
c           with non-zero contribution, and add them together with the
c           additive volume method.
c
c  31 Oct 90:  Changed to write P.Brassard file format to tape50
c		(MW)
c
c  Nov 90:  Ported to Montreal.  Converted from Cyber MNF compiler
c	    to Silicon Graphics F77 compiler.  Files are openned in
c	    subroutine openem.
c
c  Nov 90:  adding fix to stop sequence NXMODS models after full interior
c  	    crystallization, where NXMODS is given on one of the first
c	    numerical input lines.  When it's there, set killit=true.
c
c  Feb 91:  (PAB) Killing out the Pierre format.
c
c  Feb 91:  (PAB) Start modifying to print out Gille's Ledoux term.
c           Reset nhp to 8 (was 10)
c  Mar 91:  (PAB) modified next statement
c           if (s1.gt.-1.0.and.s1.lt.-.02) ds=dsx
c           and changed write statement in subroutine HOMO.
c  May 91:  (PAB) Modified to recompute smooth opacity derivatives.
c           Also sorted common statements into purely real and integer
c           parts.
c  Sept 91: (PAB) Fixed He profile discontinuity when M_{He} = 100 M_{H}
c           CCOMP now treats *both* H and C as trace elements in this case.
c  Sept 91: (PAB) Modified Ledoux term now matches Num. Diff. in most cases.
c           Physical justification lacking at present.
c  Sept 91: (PAB) Inserted Munkata etal (1985, ApJ, 296, 197) neutrino rates.
c  Oct. 91: (PAB) Putting in Modified Ledoux routine for core.
c  Oct. 91: (PAB) Fixed problem in OPACDER. Opacity dervatives are ok now.
c  Nov. 91: (PAB) Now read in U, ChiT, Chirho from EEOS tables and interpolate
c           them directly. Also compute Lrad for the envelope by a more stable
c           way. Finally, I got rid of the star stuff in CALC. These
c           changes make the output models considerably smoother.
c  Feb. 93: (PAB) Added Itoh and company neutrino rate routines.
c           Allow selection of neutrino rates through NU. Also made
c           screen output more comprehensible.
c  June 93: (PAB) Fixed array problem in block data pointed out by Matt Wood.
c           Also fixed passage of n=2 in subroutine neutrino.
c 09 27 93: Putting in various fix-its that the (picky) CRAY compiler caught.
c 08 17 94: Added Itoh etal recombination neutrino subroutine.
c 01 03 99: (MHM) Changed to read in HE3 EEOS instead of H. This is useful for
c           DB's, which should have a layer of HE3 above a layer of HE4.
c	    This functionality is controlled with the the icomp switch:
c	    icomp=1 => outer layer is hydrogen (like normal code)
c	    icomp=2 => outer layer is He3 
c	    icomp=3 => outer layer is He4 
c           This 3rd value (icomp=3) is useful for test purposes, since in 
c           this case the ledoux term should still be zero. 
c 01 05 99: (MHM) Using icomp=3, I uncovered a problem with the Ledoux term
c           for the He/H transition zone (only): the partial pressures did
c           not add up to one. I fixed this, and then found array problems
c           with the common block common/tenv/rhok(3,60,35)... the third 
c           index varies all the way up to 38 for some of the EEOS lines,
c           so now everything is dimensioned as follows:
c         parameter (nd=38)
c         common/tenv/rhok(3,60,nd),eta(3,60,nd),...
c 06 07 99: (MHM) Updated changes made to converge to a specific temperature.
c           Strange, hard-to-read code inserted in subroutine end(). gnew is
c           the adjusted stepsize (in log(sec)) which is changed to converge
c           to a particular temperature.
c...........................................................................
c START
c...........................................................................
      implicit double precision(a-h,o-z)
      logical writeit
      common/contrl/ds,g,sm,wc,it,nite,ja,jb,j,k,l
      common/writeit/writeit
      common/oldval/bold,teold 
      common/ifail/ifail
      common/thresh/thresh
      common/deldg/deldg
      common/ixswch/ixswch
      common/idiffus/idiffus
      common/mixl/ aml,mls
      common/prep/iprep
      common/dprep/aa(20,500), ecv(400), ext(400), exr(400)
      common/kill/ killit, nxmods, ikill
      common/tfit/tfit(100),told,ifit,itfit,nite1
      common/icomps/icomp
      logical killit

      killit = .false.
      ikill = 0
      icomp=1

c..........................................................................
c  connect to terminal for interactive messages
c..........................................................................
      open(16,file='temperatures.con',status='old') 
      do i=1,100
        read(16,*,end=322) tfit(i)
      enddo
322   continue
      itfit=0
      ifit=1

      call openem
      write(17,321) 
      write(20,321) 
      print 321
321   format(/,20x,'white dwarf evolution code wdxd'/
     1		//22x'last modified August 94 (PAB)'//)

      read *, thresh,deldg,ixswch,iprep,idiffus
      read *, aml, fmls
      mls = INT(fmls)
      nxmods = 5
      write(17,*) ' threshold entered:',thresh
      write(20,*) ' threshold entered:',thresh
      write(17,*) '     deldg entered:',deldg
      write(20,*) '     deldg entered:',deldg
      write(17,*) '    ixswch entered:',ixswch
      write(20,*) '    ixswch entered:',ixswch
      write(17,*) '    nxmods entered:',nxmods
      write(20,*) '    nxmods entered:',nxmods
      print *,    '    nxmods entered:',nxmods
      if(iprep .eq. 1)then
        write(17,*) 'Prepped models? YES!' 
        write(20,*) 'Prepped models? YES!' 
        print *,    'Prepped models? YES!' 
       else
        write(17,*) 'Prepped models? NO!' 
        write(20,*) 'Prepped models? NO!' 
        print *,    'Prepped models? NO!' 
      endif
      if(idiffus .eq. 1)then
        write(17,*) 'Diffusionesque transition zones'
        write(20,*) 'Diffusionesque transition zones'
        print *,    'Diffusionesque transition zones'
       else
        write(17,*) 'Discontinuous transition zones'
        write(20,*) 'Discontinuous transition zones'
        print *,    'Discontinuous transition zones'
      endif

c echo to screen which the layering case we're looking at...
      if (icomp.eq.1) then
        write(*,337) 
        write(17,337) 
337      format(/' Layering of envelope is C/He4/H -- normal (DA) case'/)
      elseif (icomp.eq.2) then
        write(*,333) 
        write(17,333) 
333     format(/' Layering of envelope is C/He4/He3 -- a DB with a', 
     1  ' surface Helium3 layer'/)
      elseif (icomp.eq.3) then
        write(*,334) 
        write(17,334) 
334     format(/' Layering of envelope is C/He4/He4. This is for test',
     1  ' purposes--it',/' should be equivalent to a normal DB model',
     2  ' with just C/He layering'/)
      else
        write(*,*) 'Flag icomp must be set...Exiting'
	stop
      endif


      if(mls .eq. 0)then
      write(17,*) 'Bohm-Vitense ML Version (inefficient)'
      write(17,*) 'mixing length/pressure scale height ratio = ',aml
      write(20,*) 'Bohm-Vitense ML Version (inefficient)'
      write(20,*) 'mixing length/pressure scale height ratio = ',aml
      print *,    'Bohm-Vitense ML Version (inefficient)'
      print *,    'mixing length/pressure scale height ratio = ',aml
       else
      write(17,*) 'Bohm & Cassinelli ML Version (efficient)'
      write(17,*) 'mixing length/pressure scale height ratio = ',aml
      write(20,*) 'Bohm & Cassinelli ML Version (efficient)'
      write(20,*) 'mixing length/pressure scale height ratio = ',aml
      print *,    'Bohm & Cassinelli ML Version (efficient)'
      print *,    'mixing length/pressure scale height ratio = ',aml
      endif
c..........................................................................
c  initialize variables
c..........................................................................
      ifail = 0
      bold = 0.
      teold = 0.
c..........................................................................
c  main program
c..........................................................................
      call read1
      call read2
      if ( idiffus .eq. 1 ) call setprof
    1 call write1
    2 call begin
    3 call calc(line)
      if ( line .eq. 2 ) then 
         call cshell
      elseif ( line .eq. 1 ) then
         call gshell
      endif
      if ( nite.eq.1) call write2
      if ( it .lt. 0 ) then
         call interp
         goto 3
      else
         call end(line)
      endif
      if ( killit ) then
	 write(*,*) 'End of the line.'
	 write(*,*) 'Terminus.'
	 stop
      endif
      go to (1,2,10),line
 
10    continue
      end 
c**************************************************************************
c array linear extrapolation.  use a and b to extrapolate to c, but
c first interpolate a to the same mass mesh points as in b. 
c**************************************************************************
      subroutine arline(a,b,c,ja,jb,kc,ip8,p)
      implicit double precision(a-h,o-z)
 
      dimension a(400,1),b(400,1),c(400,1),aj(7)
c
9     format (21h a does not contain b//)
c
      ip8=1
      check=(b(1,1)-a(1,1))/a(1,1)-1.d-07
 
      if( check .le. 0. )then
        l=2
        do 3 j=1,jb
7         if ( l .le. ja ) then
            if ( b(j,1) .le. a(l,1) ) then
              sfrac=(b(j,1)-a(l,1))/(a(l,1)-a(l-1,1))
              do 13 k=1,kc
                aj(k)=a(l,k)+(a(l,k)-a(l-1,k))*sfrac
                c(j,k)=b(j,k)+(b(j,k)-aj(k))*p
 13           continue
             else 
              l=l+1
              go to 7
            endif
           else
c...................................................................
c if a does not contain b, then simply set c = b (no extrapolation)
c...................................................................
            write(17,9)
            do 10 m=j,jb
              do 11 k=1,kc
                c(m,k)=b(m,k)
11            continue
10          continue
            return
          endif
3       continue
        return
       else
        write(17,9)
        ip8=0
        return
      endif
      end 
c**************************************************************************
c  copy array y into array x
c**************************************************************************
      subroutine armove(x,y,j,k)
      implicit double precision(a-h,o-z)
 
c      dimension x(400,1),y(400,1)
      dimension x(400,*),y(400,*)
 
      do 2 l=1,k
        do 12 i=1,j
          x(i,l)=y(i,l)
12      continue
2     continue
 
      return
      end 
c**************************************************************************
c  initialize variables for central boundary condition
c**************************************************************************
      subroutine begin
      implicit double precision(a-h,o-z)
      common/shells/ sa(400),ra(400),ba(400),pa(400),ta(400),
     1 ea(400),xca(400),fca(400),s(400),r(400),b(400),
     2 p(400),t(400),xc(400),e(400),sk(400),
     3 rk(400),bk(400),pk(400),tk(400)
      common/contrl/ds,g,sm,wc,it,nite,ja,jb,j,k,l
      common/temp/s1,r1,s2,r2,b2,p2,t2,e2,xc2,xo2,fca2,f2,q2,w2,c
      common/fin/ifin
c
      j=2 
      k=0 
      l=2 
      s1=-10.
      r1=-10.
      s2=s(1)
      r2=r(1)
      b2=b(1)
      p2=p(1)
      t2=t(1)
      ifin = 0
 
      return
      end 
c**************************************************************************
c  block data:  initiallizes some physical constants, and also the
c   constants used by the conductive opacity regimes.  the numbers
c   have been taken from itoh et al (1983; 1984). 
c Also added Itoh and company Bremsstrahlung neutrino coefficients.
c See Itoh etal 1989, ApJ, 339, 354. + erratum 360, 741.
c**************************************************************************
      block data
      implicit double precision(a-h,l,o-z)
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
c...........................................................................
c the following are the fitting coefficients quoted by itoh 
c "cdats" ==> carbon, solid state
c "cdatl" ==> carbon, liquid metal state
c "odats" ==> oxygen, solid state
c "odatl" ==> oxygen, liquid metal state
c  and so on.
c...........................................................................
      common /hydatl/ alhy(4),blhy(3),clhy(3),dlhy(4),elhy(3),flhy(3) 
 
      common /hedatl/ alhe(4),blhe(3),clhe(3),dlhe(4),elhe(3),flhe(3) 
      common/hedats/ sahe(5),sbhe(3),sehe(5),sfhe(3),sihe(5),sjhe(3), 
     1          sphe(5), sqhe(3), alphahe(4), betahe(4), sche,
     2          sdhe, sghe, shhe, skhe, slhe, srhe, sshe
 
      common /cdatl/ alc(4), blc(3), clc(3), dlc(4), elc(3), flc(3)
      common/cdats/ sac(5), sbc(3), sec(5), sfc(3), sic(5), sjc(3),
     1          spc(5), sqc(3), alphac(4), betac(4), scc,
     2          sdc, sgc, shc, skc, slc, src, ssc 
 
      common /odatl/ alo(4), blo(3), clo(3), dlo(4), elo(3), flo(3)
      common/odats/ sao(5), sbo(3), seo(5), sfo(3), sio(5), sjo(3),
     1          spo(5), sqo(3), alphao(4), betao(4), sco,
     2          sdo, sgo, sho, sko, slo, sro, sso 
c......................................................................
c the following are the Bremsstrahlung neutrino rate fitting 
c coefficients quoted by Itoh. Common'd blocks are:
c Helium: soldath, phondath, liqdath, quandath
c Carbon: soldatc, phondatc, liqdatc, quandatc
c Oxygen: soldato, phondato, liqdato, quandato
c   Iron: soldatf, phondatf, liqdatf, quandatf 
c......................................................................
      common/soldath/ snah(5), snbh(3), sneh(5), snfh(3), snih(5), 
     1          snjh(3), snph(5), snqh(3), alphasnh(4), 
     2          betasnh(4), snch, sndh, sngh, snhh, snkh, 
     3          snlh, snrh, snsh
      common/phondath/ pnah(5), pnbh(3), pnih(5), pnjh(3), 
     1          alphapnh(4), betapnh(4), pnch, pndh, pnkh, pnlh
      common/liqdath/ lnah(6), lnbh(4), lneh(6), lnfh(4), lnih(6), 
     1          lnjh(4), lnph(6), lnqh(4), alphalnh(4), 
     2          betalnh(4), lnch, lndh, lngh, lnhh, lnkh, 
     3          lnlh, lnrh, lnsh
      common/quandath/ qnah(6), qnbh(4), qneh(6), qnfh(4), 
     1          alphaqnh(5), qnch, qndh, qngh, qnhh
 
      common/soldatc/ snac(5), snbc(3), snec(5), snfc(3), snic(5), 
     1          snjc(3), snpc(5), snqc(3), alphasnc(4), 
     2          betasnc(4), sncc, sndc, sngc, snhc, snkc, 
     3          snlc, snrc, snsc 
      common/phondatc/ pnac(5), pnbc(3), pnic(5), pnjc(3), 
     1          alphapnc(4), betapnc(4), pncc, pndc, pnkc, pnlc
      common/liqdatc/ lnac(6), lnbc(4), lnec(6), lnfc(4), lnic(6), 
     1          lnjc(4), lnpc(6), lnqc(4), alphalnc(4), 
     2          betalnc(4), lncc, lndc, lngc, lnhc, lnkc, 
     3          lnlc, lnrc, lnsc 
      common/quandatc/ qnac(6), qnbc(4), qnec(6), qnfc(4), 
     1          alphaqnc(5), qncc, qndc, qngc, qnhc
 
      common/soldato/ snao(5), snbo(3), sneo(5), snfo(3), snio(5), 
     1          snjo(3), snpo(5), snqo(3), alphasno(4), 
     2          betasno(4), snco, sndo, sngo, snho, snko, 
     3          snlo, snro, snso
      common/phondato/ pnao(5), pnbo(3), pnio(5), pnjo(3), 
     1          alphapno(4), betapno(4), pnco, pndo, pnko, pnlo
      common/liqdato/ lnao(6), lnbo(4), lneo(6), lnfo(4), lnio(6), 
     1          lnjo(4), lnpo(6), lnqo(4), alphalno(4), 
     2          betalno(4), lnco, lndo, lngo, lnho, lnko, 
     3          lnlo, lnro, lnso
      common/quandato/ qnao(6), qnbo(4), qneo(6), qnfo(4), 
     1          alphaqno(5), qnco, qndo, qngo, qnho

      common/sfodatf/ snaf(5), snbf(3), snef(5), snff(3), snif(5), 
     1          snjf(3), snpf(5), snqf(3), alphasnf(4), 
     2          betasnf(4), sncf, sndf, sngf, snhf, snkf, 
     3          snlf, snrf, snsf
      common/phondatf/ pnaf(5), pnbf(3), pnif(5), pnjf(3), 
     1          alphapnf(4), betapnf(4), pncf, pndf, pnkf, pnlf
      common/liqdatf/ lnaf(6), lnbf(4), lnef(6), lnff(4), lnif(6), 
     1          lnjf(4), lnpf(6), lnqf(4), alphalnf(4), 
     2          betalnf(4), lncf, lndf, lngf, lnhf, lnkf, 
     3          lnlf, lnrf, lnsf
      common/quandatf/ qnaf(6), qnbf(4), qnef(6), qnff(4), 
     1          alphaqnf(5), qncf, qndf, qngf, qnhf
 
      data an0,bk,pi/6.022169e23,1.380622e-16,3.1415926535e0/
c...........................................................................
c  variables used in condlhy
c...........................................................................
      data alhy / 0.34542, -.28157, 0.09184, -.03734 /
      data blhy / -.61919, 0.40004, -.16585 /
      data clhy / 0.35742, -.41151, 0.21552 /
      data dlhy / 0.21512, -.10843, -.00596, -.00950 /
      data elhy / -.36667, 0.14040, -.04588 /
      data flhy / 0.10493, -.09537, 0.04682 /
c...........................................................................
c  variables used in condlhe
c...........................................................................
      data alhe / 0.62199, -.16110, 0.15574, -.02893 /
      data blhe / -.65222, 0.48601, -.18266 /
      data clhe / 0.36580, -.52176, 0.26240 /
      data dlhe / 0.36090, 0.02576, 0.05061, 0.00015 /
      data elhe / -.40559, 0.15316, -.04058 /
      data flhe / 0.12140, -.11621, 0.04939 /
c...................................................................
c variables used in condshe
c NOTE: the values stored in a(1), e(1), i(1) and p(1) are 1/2 the
c  values listed in the paper.  this is because they are used in the
c  formulae in the paper as a(1)/2, etc.  (same for carbon and oxygen).
c...................................................................
      data sahe       / -.01006, 0.05975, -.04290, -.00341, -.00338 / 
      data sbhe       / 0.11689, 0.01560, -.00713 /
      data sche, sdhe / -.16902, 1.4525 /
      data sehe       / -.05144, 0.10844, -.04380, -.00663, -.00657 / 
      data sfhe       / 0.12937, 0.02888, -.00413 /
      data sghe, shhe / -.20406, 1.36576 /
      data sihe       / 0.12429, -.04498, -.06418, -.00931, -.00582 / 
      data sjhe       / 0.11189, 0.00562, -.01176 /
      data skhe, slhe / -.22355, 1.70151 /
      data sphe       / 0.09218, -.00644, -.06552, -.01190, -.00832 / 
      data sqhe       / 0.12575, 0.01598, -.00950 /
      data srhe, sshe / -.25532, 1.90219 /
      data alphahe    / 1.5611, -20.6104, 247.0830, -1003.2600 /
      data betahe     / 1.5874, -21.4772, 255.7040, -1028.9000 /
c...........................................................................
c  variables used in condlc
c...........................................................................
      data alc/  0.9896, -0.1851,  0.1019, -0.0360/
      data blc/ -0.8825,  0.6657, -0.3798/
      data clc/ -0.0915, -1.5848,  1.1882/
      data dlc/  0.4406, -0.0161, -0.0093, -0.0028/
      data elc/ -0.4821,  0.0826, -0.0557/
      data flc/ -0.5193, -0.0830,  0.0147/
c...........................................................................
c  variables used in condsc
c...........................................................................
      data sac /   0.03387,  0.01795, -0.04324, -0.00437, -0.00422/
      data sbc /   0.04865, -0.00473, -0.00865/
      data scc, sdc/-0.09378,  0.94180/ 
      data sec /  -0.00329,  0.08497, -0.06909, -0.00582, -0.00677/
      data sfc /   0.12695,  0.00854, -0.01197/
      data sgc, shc/-0.19961,  1.63031/ 
      data sic /   0.177955,-0.08588, -0.07995, -0.00957, -0.00256/
      data sjc /   0.03417, -0.03395, -0.02519/
      data skc, slc/-0.10127,  2.10721/ 
      data spc /   0.18606, -0.06547, -0.10440, -0.01172, -0.00448/
      data sqc /   0.10304, -0.02601, -0.02922/
      data src, ssc/-0.20326,  2.80479/ 
      data alphac/0.4786,  16.4194, -139.1920, 184.7580/
      data betac/ 0.6146, 11.7296, -89.8319, 32.0399/
c...........................................................................
c  variables used in condlo
c...........................................................................
      data alo/ 1.0779, -0.1838, 0.1059, -.0290/
      data blo/ -.9743,  0.6955, -.3966/
      data clo/ -.1040, -1.7692, 1.3546 /
      data dlo/ 0.4486, -0.0160, -.0014, 0.0039 / 
      data elo/ -.5193,  0.0822, -.0467 /
      data flo/ -.5403, -0.1022, 0.0416 /
c...........................................................................
c  variables used in condso
c...........................................................................
      data sao /     0.02312, 0.02087, -.03684, -.00366, -.00348 /
      data sbo /     0.04177, -.00712, -.00865 /
      data sco, sdo/ -.07178, 0.85073 / 
      data seo /     -.001185, 0.08552, -.07206, -.00554, -.00673 /
      data sfo /     0.12586, 0.00371, -.01360 /
      data sgo, sho/ -.19226, 1.66185 / 
      data sio /      .160905, -.08292, -.07377, -.00577, 0.00154/
      data sjo /     0.03816, -.04206, -.02870 /
      data sko, slo/ -.06064, 2.14532 / 
      data spo /      .19600, -.07598, -.10917, -.00942, -.00142 /
      data sqo /     0.10403, -.03774, -.03483 /
      data sro, sso/ -.17568, 2.99352 / 
      data alphao/    .4682, 18.0197, -171.0090, 313.8890 / 
      data betao/    0.6085, 12.9705, -115.3180, 136.3590 / 
c......................................................................
c variables used in bremsolh
c......................................................................
      data snah       / -.01148, 0.01601, -.00433, 0.00015, -.00034 / 
      data snbh       / 0.01558, 0.00191, -.00055 /
      data snch, sndh / -.01694, 0.10649 /
      data sneh       / -.01827, 0.02395, -.00448, -.00033, -.00088 / 
      data snfh       / 0.01730, 0.00402, -.00005 /
      data sngh, snhh / -.02222, 0.13969 /
      data snih       / -.00323, 0.00440, -.00110, 0.00001, -.00007 / 
      data snjh       / 0.00294, 0.00059, -.00018 /
      data snkh, snlh / -.00337, 0.02116 /
      data snph       / -.00469, 0.00610, -.00114, -.00010, -.00018 / 
      data snqh       / 0.00320, 0.00107, -.00008 /
      data snrh, snsh / -.00442, 0.02775 /
      data alphasnh   / 1.6449, -23.2588, 272.1670, -1074.7000 /
      data betasnh    / 1.6443, -23.2414, 272.0080, -1074.2500 /
      data pnah       / -.00687, 0.00957, -.00204, -.00005, -.00003 / 
      data pnbh       / 0.00661, 0.00135, -.00035 /
      data pnch, pndh / -.00811, 0.05098 /
      data pnih       / -.00169, 0.00231, -.00047, -.00003, 0.00000 / 
      data pnjh       / 0.00111, 0.00042, -.00010 /
      data pnkh, pnlh / -.00161, 0.01013 /
      data alphapnh   / -.1394, 7.0680, -115.5940, 619.9170 /
      data betapnh    / -.1394, 7.0664, -115.5800, 619.8790 /
c......................................................................
c variables used in bremsolc
c......................................................................
      data snac       / 0.01838, -.01066, -.00458, -.00177, -.00138 / 
      data snbc       / -.00244, -.00206, -.00037 /
      data sncc, sndc / -.01093, 0.12431 /
      data snec       / 0.02360, -.01353, -.00619, -.00211, -.00176 / 
      data snfc       / 0.00456, -.00174, -.00031 /
      data sngc, snhc / -.02259, 0.20343 /
      data snic       / 0.00053, -.00048, -.00022, 0.00019, -.00001 / 
      data snjc       / 0.00658, -.00180, 0.00036 /
      data snkc, snlc / -.00398, 0.02499 /
      data snpc       / -.00024, 0.00063, -.00064, 0.00030, -.00006 / 
      data snqc       / 0.01013, -.00247, 0.00052 /
      data snrc, snsc / -.00650, 0.04087 /
      data alphasnc   / 0.6252, 10.6819, -70.6879, -44.3349 /
      data betasnc    / 0.6307, 10.4966, -68.7973, -50.0581 /
      data pnac       / 0.01116, -.00589, -.00279, -.00073, -.00043 / 
      data pnbc       / -.00095, -.00059, 0.00002 /
      data pncc, pndc / -.00729, 0.06630 /
      data pnic       / 0.00012, 0.00018, -.00028, 0.00012, -.00004 / 
      data pnjc       / 0.00339, -.00082, 0.00015 /
      data pnkc, pnlc / -.00212, 0.01332 /
      data alphapnc   / 0.5481, -20.4731, 223.9220, -534.9400 /
      data betapnc    / 0.5413, -20.2069, 220.7060, -524.1240 /
c......................................................................
c variables used in bremsolo
c......................................................................
      data snao       / 0.01616, -.00874, -.00413, -.00190, -.00139 / 
      data snbo       / -.00344, -.00261, -.00070 /
      data snco, sndo / -.00791, 0.13980 /
      data sneo       / 0.02210, -.00883, -.00857, -.00257, -.00214 / 
      data snfo       / 0.00629, -.00210, -.00099 /
      data sngo, snho / -.02610, 0.26993 /
      data snio       / 0.00100, -.00112, -.00003, 0.00014, 0.00001 / 
      data snjo       / 0.00745, -.00209, 0.00044 /
      data snko, snlo / -.00447, 0.02811 /
      data snpo       / -.00056, 0.00110, -.00074, 0.00024, -.00004 / 
      data snqo       / 0.01286, -.00281, 0.00057 /
      data snro, snso / -.00861, 0.05414 /
      data alphasno   / 0.4889, 16.1962, -138.4860, 185.7060 /
      data betasno    / 0.5111, 15.4195, -130.1540, 159.6050 /
      data pnao       / 0.00800, -.00191, -.00330, -.00075, -.00047 / 
      data pnbo       / 0.00088, -.00098, -.00036 /
      data pnco, pndo / -.00776, 0.08995 /
      data pnio       / -.00009, 0.00055, -.00038, 0.00011, -.00003 / 
      data pnjo       / 0.00429, -.00088, 0.00014 /
      data pnko, pnlo / -.00287, 0.01803 /
      data alphapno   / 0.3173, -14.4048, 186.9100, -476.8100 /
      data betapno    / 0.3073, -13.7973, 176.9940, -438.7520 /
c......................................................................
c variables used in bremsolf
c......................................................................
      data snaf       / 0.02096, -.01768, -.00007, -.00241, -.00080 / 
      data snbf       / -.01705, -.00268, -.00141 /
      data sncf, sndf / 0.00818, 0.13629 /
      data snef       / 0.04628, -.03290, -.00523, -.00539, -.00276 / 
      data snff       / -.02574, -.00630, -.00285 /
      data sngf, snhf / 0.00022, 0.31871 /
      data snif       / 0.00489, -.00653, 0.00171, -.00024, 0.00017 / 
      data snjf       / 0.00869, -.00323, 0.00075 /
      data snkf, snlf / -.00439, 0.02766 /
      data snpf       / 0.00732, -.00957, 0.00222, -.00022, 0.00025 / 
      data snqf       / 0.01867, -.00615, 0.00133 /
      data snrf, snsf / -.01023, 0.06442 /
      data alphasnf   / 0.6798, 12.7527, -140.1800, 268.8290 /
      data betasnf    / 0.7783, 10.2315, -124.2640, 241.3060 /
      data pnaf       / 0.01292, -.00894, -.00097, -.00125, -.00045 / 
      data pnbf       / -.00739, -.00190, -.00062 /
      data pncf, pndf / 0.00167, 0.09950 /
      data pnif       / 0.00240, -.00271, 0.00047, 0.00001, -.00001 / 
      data pnjf       / 0.00604, -.00197, 0.00044 /
      data pnkf, pnlf / -.00320, 0.02012 /
      data alphapnf   / 0.2847, -13.0828, 192.5030, -543.1480 /
      data betapnf    / 0.3221, -13.8640, 201.0700, -573.1160 /
c......................................................................
c variables used in bremliqh
c......................................................................
      data lnah       / 0.04518, -.03009, -.00564, -.00544, -.00290,
     1                  -.00224 /
      data lnbh       / -.02148, -.00817, -.00300, -.00170 /
      data lnch, lndh / 0.00671, 0.28130 /
      data lneh       / -.01003, 0.01790, -.00783, -.00021, 0.00024,
     1                  -.00014 / 
      data lnfh       / 0.00538, -.00175, -.00346, -.00031 /
      data lngh, lnhh / -.02199, 0.17300 /
      data lnih       / 0.00096, -.00301, -.00073, 0.00182, 0.00037,
     1                  0.00116 / 
      data lnjh       / 0.01706, -.00753, 0.00066, -.00060 /
      data lnkh, lnlh / -.01021, 0.06417 /
      data lnph       / -.00556, 0.00603, -.00149, 0.00047, 0.00040,
     1                  0.00028 / 
      data lnqh       / 0.00422, -.00009, -.00066, -.00003 /
      data lnrh, lnsh / -.00561, 0.03522 /
      data alphalnh   / -.07980, 0.17057, 1.51980, -.61058 /
      data betalnh    / -.05881, 0.00165, 1.82700, -.76993 /
      data qnah       / -.02260, 0.02634, -.00424, 0.00055, 0.00017, 
     1                  -.00042 / 
      data qnbh       / 0.00550, 0.00105, -.00179, 0.00028 /
      data qnch, qndh / -.02408, 0.28763 /
      data qneh       / -.00539, 0.00617, -.00205, 0.00089, 0.00022,
     1                  0.00032 / 
      data qnfh       / 0.00615, -.00149, -.00055, -.00004 /
      data qngh, qnhh / -.00764, 0.53346 /
      data alphaqnh   / 2.5344, -7.7188, -9.5605, -5.6433, 1.2671 /
c......................................................................
c variables used in bremliqc
c......................................................................
      data lnac       / 0.08973, -.05821, -.01089, -.01147, -.00656, 
     1                  -.00519 / 
      data lnbc       / -.04969, -.01584, -.00504, -.00281 /
      data lncc, lndc / 0.00945, 0.34529 /
      data lnec       / 0.03390, -.00944, -.01289, -.00589, -.00404,
     1                  -.00330 / 
      data lnfc       / -.02213, -.01136, -.00467, -.00131 /
      data lngc, lnhc / -.02342, 0.24819 /
      data lnic       / 0.00383, -.00710, -.00028, 0.00232, 0.00044,
     1                  0.00158 / 
      data lnjc       / 0.02300, -.01078, 0.00118, -.00089 /
      data lnkc, lnlc / -.01259, 0.07917 /
      data lnpc       / -.00384, 0.00356, -.00184, 0.00146, 0.00031,
     1                  0.00069 / 
      data lnqc       / 0.01052, -.00354, -.00014, -.00018 /
      data lnrc, lnsc / -.00829, 0.05211 /
      data alphalnc   / -.05483, -.01946, 1.86310, -.78873 /
      data betalnc    / -.06711, 0.06859, 1.74360, -.74498 /
      data qnac       / -.01900, 0.02446, -.00419, -.00030, -.00047, 
     1                  -.00099 / 
      data qnbc       / 0.00093, 0.00060, -.00176, 0.00018 /
      data qncc, qndc / -.02458, 0.25843 /
      data qnec       / -.00690, 0.01046, -.00351, 0.00010, -.00003,
     1                  -.00026 / 
      data qnfc       / 0.00289, -.00082, -.00155, -.00008 /
      data qngc, qnhc / -.01194, 0.34437 /
      data alphaqnc   / 0.9859, 2.2468, -10.6808, 11.4582, -4.0100 /
c......................................................................
c variables used in bremliqo
c......................................................................
      data lnao       / 0.10466, -.06740, -.01293, -.01352, -.00776,
     1                  -.00613 / 
      data lnbo       / -.05950, -.01837, -.00567, -.00310 /
      data lnco, lndo / 0.00952, 0.36029 /
      data lneo       / 0.04652, -.01656, -.01489, -.00778, -.00520,
     1                  -.00418 / 
      data lnfo       / -.03076, -.01390, -.00522, -.00161 /
      data lngo, lnho / -.02513, 0.27480 /
      data lnio       / 0.00476, -.00838, -.00011, 0.00244, 0.00046,
     1                  0.00168 / 
      data lnjo       / 0.02455, -.01167, 0.00132, -.00097 /
      data lnko, lnlo / -.01314, 0.08263 /
      data lnpo       / -.00350, 0.00295, -.00184, 0.00166, 0.00032,
     1                  0.00082 / 
      data lnqo       / 0.01231, -.00445, 0.00002, -.00026 /
      data lnro, lnso / -.00921, 0.05786 /
      data alphalno   / -.06597, 0.06048, 1.74860, -.74308 /
      data betalno    / -.07356, 0.10865, 1.70150, -.73653 /
      data qnao       / -.01792, 0.02395, -.00425, -.00055, -.00065, 
     1                  -.00115 / 
      data qnbo       / -.00039, 0.00043, -.00177, 0.00015 /
      data qnco, qndo / -.02491, 0.25207 /
      data qneo       / -.00866, 0.01344, -.00448, -.00002, -.00009,
     1                  -.00041 /
      data qnfo       / 0.00291, -.00091, -.00200, -.00010 /
      data qngo, qnho / -.01541, 0.32800 /
      data alphaqno   / 0.8978, 2.5443, -10.3549, 10.2665, -3.3537 /
 
c......................................................................
c variables used in bremliqf
c......................................................................
      data lnaf       / 0.17444, -.11076, -.02349, -.02283, -.01250,
     1                  -.00971 / 
      data lnbf       / -.10661, -.02860, -.00785, -.00385 /
      data lncf, lndf / 0.00766, 0.40991 /
      data lnef       / 0.11580, -.05891, -.02531, -.01747, -.01021,
     1                  -.00778 / 
      data lnff       / -.07767, -.02516, -.00711, -.00260 /
      data lngf, lnhf / -.03076, 0.36908 /
      data lnif       / 0.00946, -.01493, -.00125, 0.00262, 0.00055,
     1                  0.00209 / 
      data lnjf       / 0.03034, -.01519, 0.00204, -.00135 /
      data lnkf, lnlf / -.01494, 0.09395 /
      data lnpf       / -.00006, -.00222, -.00104, 0.00225, 0.00037,
     1                  0.00140 / 
      data lnqf       / 0.02013, -.00883, 0.00090, -.00071 /
      data lnrf, lnsf / -.01249, 0.07850 /
      data alphalnf   / -.07608, 0.11559, 1.75730, -.79677 /
      data betalnf    / -.08034, 0.14368, 1.73140, -.79467 /
      data qnaf       / -.01326, 0.02179, -.00484, -.00146, -.00135, 
     1                  -.00175 / 
      data qnbf       / -.00567, -.00040, -.00174, 0.00012 /
      data qncf, qndf / -.02710, 0.23276 /
      data qnef       / -.00430, 0.01443, -.00677, -.00142, -.00119,
     1                  -.00151 /
      data qnff       / -.00396, -.00245, -.00278, -.00023 /
      data qngf, qnhf / -.02288, 0.26787 /
      data alphaqnf   / 1.0379, 1.0195, -5.1242, 3.9369, -0.8701 /
c.................................. all done! 
      end 
c******************************************************************** 
c  set up variables for the step to the next shell.
c******************************************************************** 
      subroutine calc(line)
      implicit double precision(a-h,o-z)
c..........................................................................
      common/shells/ sa(400),ra(400),ba(400),pa(400),ta(400),
     1 ea(400),xca(400),fca(400),s(400),r(400),b(400),
     2 p(400),t(400),e(400),xc(400),sk(400),
     3 rk(400),bk(400),pk(400),tk(400)
      common/thermo/u2,up2,ut2,e2,ep2,et2,psi2,pg,o2,op2,ot2,fp,ft,
     1           en2(5),fn,fcc,ce,ci,cif,w,nu
      common/temp/s1,r1,s2,r2,b2,p2,t2,ea2,xc2,xo2,fca2,f2,q2,w2,c
      common/cal/ut1,ot1,et1,op1,o1,ep1,qnp1,qnt1,p1,q1,f1,t1,w1,b1,e1,
     1           qe1,ea1,qn1,up1,qe2,qn2,qnp2,qnt2
      common/xx/sin,sout,smid,grid1,grid2,ucent,cste,kon,kom
      common/contrl/ds,g,sm,wc,it,nite,ja,jb,j,k,l
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      common/corats/ams(10),delmass(10),corat(10),dcorat(10),
     1              ncore, irdold, alph 
      common/prep/iprep
      common/dprep/aa(20,500), ecv(400), ext(400), exr(400)
      common/bldxc/bledc
      logical converg
c..........................................................................
      data ck/1.5e0/
      data alsun, amass/3.90e+33,1.989e+33/
c..........................................................................
  402 format(' quit memory full',/'k = ',i4)
c..........................................................................
      line = 2
      je = 3
      k=k+1
      if ( nite .eq. 1 ) then 
         converg = .true.
      else
         converg = .false.
      endif
c..........................................................................
c if memory full, then make program dump
c..........................................................................
      if ( k .ge. 400) then
         print 402, k
         write(17,402) k
         write(20,402) k
         stop
      endif
c..........................................................................
c otherwise, go on about our business.
c..........................................................................
      sk(k)  = s2
      rk(k)  = r2
      bk(k)  = b2
      pk(k)  = p2
      tk(k)  = t2
      kk=k
 
  420 if ( l .le. ja ) then
         if ( s2 .le. sa(l) ) then
            sfrac = (s2-sa(l))/(sa(l)-sa(l-1))
            ea2   = ea(l)+(ea(l)-ea(l-1))*sfrac
            xc2   = xca(l)+(xca(l)-xca(l-1))*sfrac
            xo2   = 1.-xc2
            fca2  = fca(l)+(fca(l)-fca(l-1))*sfrac
         else
            l=l+1
            go to 420
         endif
      elseif ( w1 .lt. 0.4 ) then
         ea2=e1-1.914535e+08*dlog10(1.+ds/s1)*(1.-2.5*w1)/cei
      endif
c..........................................................................
c because i'm now specifying the c/o profile in the interior, i'm
c going to interpolate seperately here if irdold = 1
c as opposed to interpolating in the 'alda' model when irdold = 0,2.
c this should keep the profile
c from getting spread out through quantization error.  note also
c that in interp, ds is decreased if near a corat boundary, and
c that things are zoned so that a mass shell will be on all 
c corat boundaries. 
c..........................................................................
      if ( irdold .eq. 1 ) then
         do 500 iii = 2,ncore 
            test = 10.**s2
            if ( test .eq. ams(iii) ) then
               xc2 = corat(iii)
               xo2 = 1. - xc2 
               goto 600
            else if ( test .lt. ams(iii) ) then
               xc2 = dcorat(iii) / delmass(iii) * (test - ams(iii-1)) 
     1                 + corat(iii-1)
               xo2 = 1. - xc2 
               goto 600
            endif
 500     continue
      elseif ( irdold .eq. 2 ) then
         if ( s2 .lt. s(jb) ) then
            xo2 = .5 * ( 1. + alph ) * ( 1. - 10.**s2 )**alph
            xc2 = 1. - xo2
         else
            xo2 = 0.
            xc2 = 1.
         endif
      endif
 600  c=cste      
      xc(k) = xc2
      
      if ( s2 .gt. -sin .and. s2 .lt. -sout ) then
         c=c*(dabs(smid+s2)*grid2+grid1) 
      endif
 
      cei=1./(.5+1./ci)
 
      if ( xc2 .gt. .000001 .and. xc2 .lt. .999999) then
         call istatco(p2,t2,0,converg)
      elseif ( xc2 .ge. .999999) then
         call istat1(p2,t2,0,12,converg)
      elseif ( xc2 .le. .000001) then
         call istat1(p2,t2,0,16,converg)
      endif
 
      e(k)=e2
      fcc=(1.-w)*fca2+w*fcc
      if(k.le.1) ucent=u2
      q2=10.**(-3.*r2+s2-u2-1.099210+sm)
      f2=10.**(-4.*r2-p2+2.*s2-8.275214+2.*sm)
      w2=o2*b2*10.**(p2-4.*t2-s2+43.18734-sm)
      wc=-ep2/et2
 
      if (w2 .ge. wc) then
         w2 = wc
         if (kon .le. 0) then 
            c = c/ck
            kon = k 
         endif
      else
         if ( kon .gt. 0 ) then
            kom = kon
            kon = 0 
            c = c * ck
         endif
      endif
 
      qx=10.**(s2-33.22885+sm)
      qn2=(fcc-fn)*qx
      qnp2=-fp*qx
      qnt2=-qx*ft
      qe2=qx*10.**(t2-g)
      if ( k .gt. 1 ) line = 1
c..........................................................................
c  Now load up the variables in aa() if we're calculating the converged
c  model
c..........................................................................
        if ( converg .and. iprep .eq. 1 ) then
           aa( 1,k) = 10.**r2
           star=10.0**(sm)
           aa( 2,k) = 10.**s2 * star
           aa( 3,k) = b2 * alsun
           aa( 4,k) = 10.**t2
           aa( 5,k) = 10.**u2
           aa( 6,k) = 10.**p2
           aa( 7,k) = fn
           cpx  = ut2/wc*10.**(-u2+p2-t2)
           xt   = -ut2/up2
           cv   = cpx*(1.e0-(xt*wc))
           aa( 8,k) = -cv
           aa( 9,k) = 1./up2
           aa(10,k) = xt
           aa(11,k) = ft
           aa(12,k) = fp/up2
c** Paul's opac business (Will have to go to opac3 when comp is mixed)
c only calling opac for carbon here, because we lack O table. 
c Rad opac contribution is zilch here anyway.
c We use 5pt differencing in the core.
           cent = 0.01
           dent = 0.02
           oplo4=opac(u2+dent,t2)
           oplo1=opac(u2+cent,t2)
           oplo2=opac(u2-cent,t2)
           oplo3=opac(u2-dent,t2)
c*** kapr ****
           aa(13,k)=(oplo3-(8.*oplo2)+(8.*oplo1)-oplo4)/(12.*0.01)
c           if(k .gt. 160)then
c             write(17,1901) k,oplo4,oplo1,oplo2,oplo3
c           endif
           oplo2=opac(u2,t2+dent)
           oplo3=opac(u2,t2+cent)
           oplo4=opac(u2,t2-cent)
           oplo1=opac(u2,t2-dent)
c*** kapt ****
           aa(14,k)=(oplo1-(8.*oplo4)+(8.*oplo3)-oplo2)/(12.*0.01)
c           if(k .gt. 160)then
c             write(17,1902) k,oplo2,oplo3,oplo4,oplo1
c           endif
c1901  format(1x,'core ropac',i4,4(1pe12.5,1x))
c1902  format(1x,'core topac',i4,4(1pe12.5,1x))
           aa(15,k) = w2
           aa(16,k) = wc
           aa(17,k) = 0.0d0
           aa(18,k) = o2
           aa(20,k) = xo2
c this is core ledoux term. (set to zero, then reset if need be)
           aa(19,k) = 0.0
c if at center, set Ledoux term to 0 by definition.
           if(k.eq.1)then
             return
           endif
           if(xc2.gt.0.000001 .and. xc2.lt.0.999999)then
c if O fraction is constant, set Ledoux to zero.
             if(aa(20,k).eq.aa(20,k-1))then
               return
              else
c otherwise, compute a Ledoux term.
               call ledouxc
               if(aa(20,k).le.0.0 .or. aa(20,k-1).le.0.0)then
                 return
               endif
               dlo = ( dlog(aa(20,k)) - dlog(aa(20,k-1)) )
               dlp = ( dlog(aa(6,k)) - dlog(aa(6,k-1)) )
               chtor = -aa(9,k)/aa(10,k)
               aa(19,k) = chtor*(dlo/dlp)*bledc
c Note: there is a minus sign difference between this ledoux term and
c the one in Brassard et al. (1991, ApJ, 367, 601).
c This was "corrected" by the absolute value below, but I want to be
c able to see if certain configurations are convectively unstable in the
c core, so I am going to remove the dabs() (MHM April 1998).
c               aa(19,k) = dabs(aa(19,k))
               aa(19,k) = -aa(19,k)
c         write(17,*) k,aa(19,k),chtor,dlo/dlp,bledc
             endif
           endif
	endif
      return
      end 
c********************************************************************
c  this routine returns the h, he, and c mass fractions given as
c  input:  xmass (position in the envelope), amhyhe, amheca.
c
c  this is the first diffusion version.  the profiles returned are
c  derived under several assumptions (some questionable).  in
c  short, we can do better, but this is going to be it for now.
c  we're doing a pulsation study, so it's mostly important that
c  the profiles be smooth.  this version will not mix h into he
c  by convection
c  MAW 12 feb 89
c  Modified by PAB to remove discontinuity in He abundance when
c helium layer is only 100x thicker than hydrogen layer. 09 05 91
c Here, I let both H and C be trace elements between zone(1,2)
c and zone(2,1). 09 09 91 (This one works in giving a smooth profile.)
c********************************************************************
        subroutine ccomp
        implicit double precision(a-h,o-z)
        common/je/je
        common/comp/amhyhe,amheca,x(4)
        common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho, 
     1      ol1x,rl1x,el1x,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
        common/alpha/alph(4),qmix1,qmix2,zone(2,2)
        common/idiffus/idiffus
      ame = 1.e0-(xmass/sm)
c...................................................................
c determine surface composition
c...................................................................
      if(amhyhe.lt.1.0e-15.and.amheca.lt.1.0e-15) je=3
      if(amhyhe.lt.1.0e-15.and.amheca.gt.1.0e-15) je=2
      if(amhyhe.gt.1.0e-15) je=1
c...................................................................
c  if it's a diffusion equilibrium, then proceed, otherwise, jump
c  down to the discontinuity code
c...................................................................
c determine local composition and load comp vector x accordingly
c...................................................................
c......................... hydrogen surface layer .......................
      if ( idiffus .eq. 1 ) then
        if ( je .eq. 1 ) then 
          if(zone(1,2) .lt. zone(2,1))then
c...................................................................
c Normal case. We compute profiles without a care in the world.
c  pure h 
c...................................................................
           if ( ame .le. zone(1,1) ) then
              x(1) = 1.0
              x(2) = 0.0
              x(3) = 0.0
              x(4) = 0.0
c...................................................................
c  lower h layer (includes he tail)
c...................................................................
           elseif ( ame .gt. zone(1,1) .and. ame .le. qmix1 ) then
              xx = ame/qmix1
              yy = func1(xx)
              x(1) = yy
              x(2) = 1.0  - yy
              x(3) = 0.0
              x(4) = 0.0
c...................................................................
c  upper he layer
c...................................................................
           elseif ( ame .gt. qmix1  .and. ame .le. zone(1,2) ) then
              xx = ame/qmix1
              yy = func2(xx)
              x(1) = yy
              x(2) = 1.0  - yy
              x(3) = 0.0
              x(4) = 0.0
c...................................................................
c  pure he
c...................................................................
           elseif ( ame .gt. zone(1,2) .and. ame .le. zone(2,1) ) then
              x(1) = 0.
              x(2) = 1.0
              x(3) = 0.0
              x(4) = 0.0
c...................................................................
c  lower he layer
c...................................................................
           elseif ( ame .gt. zone(2,1) .and. ame .le. qmix2) then
              xx = ame/qmix2
              yy = func3(xx)
              x(1) = 0.
              x(2) = yy
              x(3) = 1.0  - yy
              x(4) = 0.0
c...................................................................
c  upper c layer
c...................................................................
           elseif ( ame .gt. qmix2 .and. ame .le. zone(2,2) ) then
              xx = ame/qmix2
              yy = func4(xx)
              x(1) = 0.
              x(2) = yy
              x(3) = 1.0  - yy
              x(4) = 0.0
c...................................................................
c  pure c 
c...................................................................
           elseif ( ame .gt. zone(2,2) ) then
              x(1) = 0.0
              x(2) = 0.0
              x(3) = 1.0
              x(4) = 0.0
           endif
          else
c...................................................................
c Special case where M_H is only 100x less than M_He. 
c We compute profiles very carefully to avoid discontinuous slopes.
c...................................................................
c compute changeover point from H as trace to C as trace
c...................................................................
           zond1 = dlog10(zone(1,2))
           zond2 = dlog10(zone(2,1))
           zond3 = (zond1 + zond2) / 2.
           change = 10.**(zond3)
c...................................................................
c  pure h  (same as before)
c...................................................................
           if ( ame .le. zone(1,1) ) then
              x(1) = 1.0
              x(2) = 0.0
              x(3) = 0.0
              x(4) = 0.0
c...................................................................
c  lower h layer (includes he tail) (again, same as before)
c...................................................................
           elseif ( ame .gt. zone(1,1) .and. ame .le. qmix1 ) then
              xx = ame/qmix1
              yy = func1(xx)
              x(1) = yy
              x(2) = 1.0  - yy
              x(3) = 0.0
              x(4) = 0.0
c...................................................................
c  upper he layer (up to point where C should become a trace element)
c...................................................................
           elseif ( ame .gt. qmix1  .and. ame .le. zone(2,1) ) then
              xx = ame/qmix1
              yy = func2(xx)
              x(1) = yy
              x(2) = 1.0  - yy
              x(3) = 0.0
              x(4) = 0.0
c...................................................................
c  Middle of He layer (between zone(1,2) and Zone (2,1))
c...................................................................
           elseif ( ame .gt. zone(2,1)  .and. ame .le. zone(1,2) ) then
              xx = ame/qmix1
              yy = func2(xx)
              x(1) = yy
              yhe = 1.0 - yy
              zz = ame/qmix2
              ww = func3(zz)
	      if (zz.le.1.0) then
                ww = func3(zz)
	      else
                ww = func4(zz)
	      endif
	      if (ww.lt.0.0) write(*,*) 'yikes ww < 0!',zz,ww
c changing following lines so that there is no negative helium abundance
c MHM (4/17/98)
              x(2) = ww - yy
              x(3) = 1.0 - ww
c              x(2) = ww
c              x(3) = 1.0 - ww - yy
              x(4) = 0.0
c...................................................................
c  lower He layer. (Below where H is a trace element)
c...................................................................
           elseif ( ame .gt. zone(1,2) .and. ame .le. qmix2) then
              xx = ame/qmix2
              yy = func3(xx)
              x(1) = 0.
              x(2) = yy
              x(3) = 1.0  - yy
c...................................................................
c  upper c layer (same as before)
c...................................................................
           elseif ( ame .gt. qmix2 .and. ame .le. zone(2,2) ) then
              xx = ame/qmix2
              yy = func4(xx)
              x(1) = 0.
              x(2) = yy
              x(3) = 1.0  - yy
              x(4) = 0.0
c...................................................................
c  pure c  (same as before):
c...................................................................
           elseif ( ame .gt. zone(2,2) ) then
              x(1) = 0.0
              x(2) = 0.0
              x(3) = 1.0
              x(4) = 0.0
           endif
          endif
c......................... helium surface layer .......................
        elseif ( je .eq. 2 ) then
c...................................................................
c  pure he
c...................................................................
           if ( ame .le. zone(2,1) ) then
              x(1) = 0.
              x(2) = 1.0
              x(3) = 0.0
              x(4) = 0.0
c...................................................................
c  lower he layer
c...................................................................
           elseif ( ame .gt. zone(2,1) .and. ame .le. qmix2) then
              xx = ame/qmix2
              yy = func3(xx)
              x(1) = 0.
              x(2) = yy
              x(3) = 1.0  - yy
              x(4) = 0.0
c...................................................................
c  upper c layer
c...................................................................
           elseif ( ame .gt. qmix2 .and. ame .le. zone(2,2) ) then
              xx = ame/qmix2
              yy = func4(xx)
              x(1) = 0.
              x(2) = yy
              x(3) = 1.0  - yy
              x(4) = 0.0
c...................................................................
c  pure c 
c...................................................................
           elseif ( ame .gt. zone(2,2) ) then
              x(1) = 0.0
              x(2) = 0.0
              x(3) = 1.0
              x(4) = 0.0
           endif
c......................... carbon surface layer .......................
        elseif ( je .eq. 3 ) then
           x(1) = 0.0
           x(2) = 0.0
           x(3) = 1.0
           x(4) = 0.0
        endif
      else
c...................................................................
c  come here for discontinuity profiles
c...................................................................
c determine local composition
c...................................................................
      if(je.ne.1) goto 100
      if(ame.gt.amhyhe.and.amheca.gt.1.0e-15) je=2
      if(ame.gt.amhyhe.and.amheca.lt.1.0e-15) je=3
100   continue
      if(je.ne.2) goto 101
      if(ame.gt.amheca) je=3
101   continue
c...................................................................
c load comp vector X accordingly
c...................................................................
      if ( je .eq. 1 ) then
        x(1) = 1.0
        x(2) = 0.0
        x(3) = 0.0
        x(4) = 0.0
      else if ( je .eq. 2 ) then
        x(1) = 0.0
        x(2) = 1.0
        x(3) = 0.0
        x(4) = 0.0
      else if ( je .eq. 3 ) then
        x(1) = 0.0
        x(2) = 0.0
        x(3) = 1.0
        x(4) = 0.0
      endif
      endif
      if (x(1).lt.0.0 .or. x(2).lt.0.0 .or. x(3).lt.0.0 .or. x(4).lt.0.0) then
        write(*,110) x(1),x(2),x(3),x(4),xx,func3(xx),func4(xx),func1(xx),func2(xx),ame
      endif
110   format('Warning: negative Xs:',9(f6.3,x),e12.5)
        return
        end
c***************************************************************************
c this function returns the (carbon) thermal conductivity computed for a
c specific dens, temp, and gamma using the results if itoh et al (1983)
c for the liquid metal regime.
c***************************************************************************
      function cndlc(dens,t,gamma)
      implicit double precision (a-z)
      common /cdatl/ a(4), b(3), c(3), d(4), e(3), f(3)
 
      data at, z, za /12., 6., .5/
c...........................................................
c  the following data from itoh et al. are for carbon (z=6) 
c...........................................................
      t8 = 10.**(t -8)
      d6 = 10.**(dens - 6 )
      d6mu = d6 / 2.
      d6mu23 = d6mu**(2./3.)
 
      rs    = 1.388e-02 / d6mu**(1./3.) 
 
      x = 0.45641 * dlog( gamma ) - 1.31636
      x2 = x*x
      x3 = x2 * x
 
      asum = a(1) + a(2) * x + a(3) * x2 + a(4) * x3
      bsum = b(1) + b(2) * x + b(3) * x2
      csum = c(1) + c(2) * x + c(3) * x2
      dsum = d(1) + d(2) * x + d(3) * x2 + d(4) * x3
      esum = e(1) + e(2) * x + e(3) * x2
      fsum = f(1) + f(2) * x + f(3) * x2
 
      sminus1 = asum * ( 1. + bsum * rs + csum * rs*rs )
      splus1  = dsum * ( 1. + esum * rs + fsum * rs*rs )
 
      cstd6  = 1.018 * d6mu23 
      tmpsum = 1. + cstd6
      s = sminus1 - splus1 * cstd6 / tmpsum
 
      opacity = 2.363e+17 * d6 * t8 / ( at * tmpsum * s )
      cndlc  = dlog10( opacity )
 
      return
      end 
c***************************************************************************
c this function returns the (helium) thermal conductivity computed for a
c specific dens, temp, and gamma using the results if itoh et al (1983)
c for the liquid metal regime.
c***************************************************************************
      function cndlhe(dens,t,gamma)
      implicit double precision (a-z)
 
      common /hedatl/ a(4), b(3), c(3), d(4), e(3), f(3)
 
      data at, z, za /4., 2., .5/
c...........................................................
c  the following data from itoh et al. are for helium (z=2) 
c...........................................................
      t8 = 10.**(t - 8.)
      d6 = 10.**(dens - 6. )
      d6mu = d6 / 2.
      d6mu23 = d6mu**(2./3.)
 
      rs    = 1.388e-02 / d6mu**(1./3.) 
 
      x = 0.45641 * dlog( gamma ) - 1.31636
      x2 = x*x
      x3 = x2 * x
 
      asum = a(1) + a(2) * x + a(3) * x2 + a(4) * x3
      bsum = b(1) + b(2) * x + b(3) * x2
      csum = c(1) + c(2) * x + c(3) * x2
      dsum = d(1) + d(2) * x + d(3) * x2 + d(4) * x3
      esum = e(1) + e(2) * x + e(3) * x2
      fsum = f(1) + f(2) * x + f(3) * x2
 
      sminus1 = asum * ( 1. + bsum * rs + csum * rs*rs )
      splus1  = dsum * ( 1. + esum * rs + fsum * rs*rs )
 
      cstd6  = 1.018 * d6mu23 
      tmpsum = 1. + cstd6
      s = sminus1 - splus1 * cstd6 / tmpsum
 
      opacity = 2.363e+17 * d6 * t8 / ( at * tmpsum * s )
      cndlhe = dlog10( opacity )
 
      return
      end 
c***************************************************************************
c this function returns the (hydrogen) thermal conductivity computed for a
c specific dens, temp, and gamma using the results if itoh et al (1983)
c for the liquid metal regime.
c***************************************************************************
      function cndlhy(dens,t,gamma)
      implicit double precision (a-z)
      common /hydatl/ a(4), b(3), c(3), d(4), e(3), f(3)
      data at, z, za /1., 1., .5/
c...........................................................
c  the following data from itoh et al. are for hydrogen (z=1) 
c...........................................................
      t8 = 10.**(t -8.)
      d6 = 10.**(dens - 6. )
      d6mu = d6
      d6mu23 = d6mu**(2./3.)
 
      rs    = 1.388e-02 / d6mu**(1./3.) 
 
      x = 0.45641 * dlog( gamma ) - 1.31636
      x2 = x*x
      x3 = x2 * x
 
      asum = a(1) + a(2) * x + a(3) * x2 + a(4) * x3
      bsum = b(1) + b(2) * x + b(3) * x2
      csum = c(1) + c(2) * x + c(3) * x2
      dsum = d(1) + d(2) * x + d(3) * x2 + d(4) * x3
      esum = e(1) + e(2) * x + e(3) * x2
      fsum = f(1) + f(2) * x + f(3) * x2
 
      sminus1 = asum * ( 1. + bsum * rs + csum * rs*rs )
      splus1  = dsum * ( 1. + esum * rs + fsum * rs*rs )
 
      cstd6  = 1.018 * d6mu23 
      tmpsum = 1. + cstd6
      s = sminus1 - splus1 * cstd6 / tmpsum
 
      opacity = 2.363e+17 * d6 * t8 / ( at * tmpsum * s )
      cndlhy = dlog10( opacity )
 
      return
      end 
c***************************************************************************
c this function returns the (oxygen) thermal conductivity computed for a
c specific dens, temp, and gamma using the results if itoh et al (1983)
c for the liquid metal regime.
c***************************************************************************
      function cndlo(dens,t,gamma)
      implicit double precision (a-z)
 
      common /odatl/ a(4), b(3), c(3), d(4), e(3), f(3)
 
      data at, z, za /16., 8., .5/
c...........................................................
c  the following data from itoh et al. are for oxygen (z=8) 
c...........................................................
      t8 = 10.**(t -8)
      d6 = 10.**(dens - 6 )
      d6mu = d6 / 2.
      d6mu23 = d6mu**(2./3.)
 
      rs    = 1.388e-02 / d6mu**(1./3.) 
 
      x = 0.45641 * dlog( gamma ) - 1.31636
      x2 = x*x
      x3 = x2 * x
 
      asum = a(1) + a(2) * x + a(3) * x2 + a(4) * x3
      bsum = b(1) + b(2) * x + b(3) * x2
      csum = c(1) + c(2) * x + c(3) * x2
      dsum = d(1) + d(2) * x + d(3) * x2 + d(4) * x3
      esum = e(1) + e(2) * x + e(3) * x2
      fsum = f(1) + f(2) * x + f(3) * x2
 
      sminus1 = asum * ( 1. + bsum * rs + csum * rs*rs )
      splus1  = dsum * ( 1. + esum * rs + fsum * rs*rs )
 
      cstd6  = 1.018 * d6mu23 
      tmpsum = 1. + cstd6
      s = sminus1 - splus1 * cstd6 / tmpsum
 
      opacity = 2.363e+17 * d6 * t8 / ( at * tmpsum * s )
      cndlo  = dlog10( opacity )
 
      return
      end 
c***************************************************************************
c this function returns the thermal conductivity computed for a specific
c dens, temp, and gamma using the results if itoh et al (1984) for carbon
c in the xtal regime.
c***************************************************************************
      function cndsc(dens,t,gamma)
      implicit double precision (a-z)
c......................................................
c  declare variables (find the values in the block data
c  piece of code).
c......................................................
      common/cdats/ a(5), b(3), e(5), f(3), i(5), j(3), p(5), q(3),
     1          alpha(4), beta(4), c, d, g, h, k, l, r, s
      data at, z, twopi, pisqr /12.,6., 6.283185308, 9.869604404/
 
      x13 = -1./3.
      x23 = -2./3.
      t8 = 10.**t / 1.e+08
      d6 = 10.**dens / 1.e+06 
      d6mu = d6 * z / at
      d6mu23 = d6mu**(2./3.)
c......................................................
c compute the interpolation coefficients v and w
c......................................................
      u = twopi * ( dens - 3. ) / 9.
      v = alpha(1) + alpha(2) * gamma**x13 + alpha(3) * gamma**x23
     1    + alpha(4) / gamma
      w = beta(1) + beta(2) * gamma**x13 + beta(3) * gamma**x23
     1    + beta(4) / gamma
c......................................................
c set up the variables (sines and cosines) for the loops
c......................................................
      su  = dsin(u)
      su2 = su * su 
      cu2 = 1. - su2
      cu  = dsqrt( cu2 )
      s2u = 2. * su * cu
      c2u = cu2 - su2
      s3u = su * ( 3. - 4. * su2 )
      c3u = cu * ( 4. * cu2 - 3. )
      c4u = -8. * cu2 * su2 + 1.
c......................................................
c  here's the loops, unwrapped
c......................................................
      asum = a(1) + a(2) * cu    + a(3) * c2u
     1            + a(4) * c3u + a(5) * c4u
      bsum = b(1) * su + b(2) * s2u + b(3) * s3u
     1       + c*u + d
      esum = e(1) + e(2) * cu    + e(3) * c2u
     1            + e(4) * c3u + e(5) * c4u
      fsum = f(1) * su + f(2) * s2u + f(3) * s3u
     1       + g*u + h
      isum = i(1) + i(2) * cu    + i(3) * c2u
     1            + i(4) * c3u + i(5) * c4u
      jsum = j(1) * su + j(2) * s2u + j(3) * s3u
     1       + k*u + l
      psum = p(1) + p(2) * cu    + p(3) * c2u
     1            + p(4) * c3u + p(5) * c4u
      qsum = q(1) * su + q(2) * s2u + q(3) * s3u
     1       + r*u + s
 
      isig171 = asum + bsum
      isig5k  = esum + fsum
      ikap171 = isum + jsum
      ikap5k  = psum + qsum
 
      isig    = (1.-v) * isig171 + v * isig5k
      ikap    = (1.-w) * ikap171 + w * ikap5k
 
      smgamma = .07832 * (z/at) * dsqrt(d6) / t8
      sg2     = smgamma * smgamma
 
      g0      = 13.00 / dsqrt( 1. + 0.0174 * sg2 ) 
      g2   = (sg2 / pisqr) * (1. + 0.0118 * sg2)**(-3./2.)
 
      fkap    = isig * g0 + ikap * g2
      cstd6   = 1.018 * d6mu23
      nukap   = 9.554e+16 * t8 * dsqrt( 1. + 1./cstd6 ) * fkap
 
      opacity = 4.146d+15 * d6mu / dsqrt( 1. + cstd6 )
     1          * t8 * 1.e+18 / nukap
 
      cndsc  = dlog10( opacity )
 
      return
      end 
c***************************************************************************
c this function returns the thermal conductivity computed for a specific
c dens, temp, and gamma using the results if itoh et al (1984) for helium
c in the xtal regime.
c***************************************************************************
      function cndshe(dens,t,gamma)
      implicit double precision (a-z)
c......................................................
c  declare variables (find the values in the block data
c  piece of code).
c......................................................
      common/hedats/ a(5), b(3), e(5), f(3), i(5), j(3), p(5), q(3),
     1          alpha(4), beta(4), c, d, g, h, k, l, r, s
      data at, z, twopi, pisqr /4.,2., 6.283185308, 9.869604404/
 
      x13 = -1./3.
      x23 = -2./3.
      t8 = 10.**t / 1.e+08
      d6 = 10.**dens / 1.e+06 
      d6mu = d6 * z / at
      d6mu23 = d6mu**(2./3.)
c......................................................
c compute the interpolation coefficients v and w
c......................................................
      u = twopi * ( dens - 3. ) / 9.
      v = alpha(1) + alpha(2) * gamma**x13 + alpha(3) * gamma**x23
     1    + alpha(4) / gamma
      w = beta(1) + beta(2) * gamma**x13 + beta(3) * gamma**x23
     1    + beta(4) / gamma
c......................................................
c set up the variables (sines and cosines) for the loops
c......................................................
      su  = dsin(u)
      su2 = su * su 
      cu2 = 1. - su2
      cu  = dsqrt( cu2 )
      s2u = 2. * su * cu
      c2u = cu2 - su2
      s3u = su * ( 3. - 4. * su2 )
      c3u = cu * ( 4. * cu2 - 3. )
      c4u = -8. * cu2 * su2 + 1.
c......................................................
c  here's the loops, unwrapped
c......................................................
      asum = a(1) + a(2) * cu    + a(3) * c2u
     1            + a(4) * c3u + a(5) * c4u
      bsum = b(1) * su + b(2) * s2u + b(3) * s3u
     1       + c*u + d
      esum = e(1) + e(2) * cu    + e(3) * c2u
     1            + e(4) * c3u + e(5) * c4u
      fsum = f(1) * su + f(2) * s2u + f(3) * s3u
     1       + g*u + h
      isum = i(1) + i(2) * cu    + i(3) * c2u
     1            + i(4) * c3u + i(5) * c4u
      jsum = j(1) * su + j(2) * s2u + j(3) * s3u
     1       + k*u + l
      psum = p(1) + p(2) * cu    + p(3) * c2u
     1            + p(4) * c3u + p(5) * c4u
      qsum = q(1) * su + q(2) * s2u + q(3) * s3u
     1       + r*u + s
 
      isig171 = asum + bsum
      isig5k  = esum + fsum
      ikap171 = isum + jsum
      ikap5k  = psum + qsum
 
      isig    = (1.-v) * isig171 + v * isig5k
      ikap    = (1.-w) * ikap171 + w * ikap5k
 
      smgamma = .07832 * (z/at) * dsqrt(d6) / t8
      sg2     = smgamma * smgamma
 
      g0      = 13.00 / dsqrt( 1. + 0.0174 * sg2 ) 
      g2   = (sg2 / pisqr) * (1. + 0.0118 * sg2)**(-3./2.)
 
      fkap    = isig * g0 + ikap * g2
      cstd6   = 1.018 * d6mu23
      nukap   = 9.554e+16 * t8 * dsqrt( 1. + 1./cstd6 ) * fkap
 
      opacity = 4.146d+15 * d6mu / dsqrt( 1. + cstd6 )
     1          * t8 * 1.e+18 / nukap
 
      cndshe = dlog10( opacity )
 
      return
      end 
c***************************************************************************
c this function returns the thermal conductivity computed for a specific
c dens, temp, and gamma using the results if itoh et al (1984) for oxygen
c in the xtal regime.
c***************************************************************************
      function cndso(dens,t,gamma)
      implicit double precision (a-z)
c......................................................
c  declare variables (find the values in the block data
c  piece of code).
c......................................................
      common/odats/ a(5), b(3), e(5), f(3), i(5), j(3), p(5), q(3),
     1          alpha(4), beta(4), c, d, g, h, k, l, r, s
      data at, z, twopi, pisqr /16.,8., 6.283185308, 9.869604404/
 
      x13 = -1./3.
      x23 = -2./3.
      t8 = 10.**t / 1.e+08
      d6 = 10.**dens / 1.e+06 
      d6mu = d6 / 2.
      d6mu23 = d6mu**(2./3.)
c......................................................
c compute the interpolation coefficients v and w
c......................................................
      u = twopi * ( dens - 3. ) / 9.
      v = alpha(1) + alpha(2) * gamma**x13 + alpha(3) * gamma**x23
     1    + alpha(4) / gamma
      w = beta(1) + beta(2) * gamma**x13 + beta(3) * gamma**x23
     1    + beta(4) / gamma
c......................................................
c set up the variables (sines and cosines) for the loops
c......................................................
      su  = dsin(u)
      su2 = su * su 
      cu2 = 1. - su2
      cu  = dsqrt( cu2 )
      s2u = 2. * su * cu
      c2u = cu2 - su2
      s3u = su * ( 3. - 4. * su2 )
      c3u = cu * ( 4. * cu2 - 3. )
      c4u = -8. * cu2 * su2 + 1.
c......................................................
c  here's the loops, unwrapped
c......................................................
      asum = a(1) + a(2) * cu    + a(3) * c2u
     1            + a(4) * c3u + a(5) * c4u
      bsum = b(1) * su + b(2) * s2u + b(3) * s3u
     1       + c*u + d
      esum = e(1) + e(2) * cu    + e(3) * c2u
     1            + e(4) * c3u + e(5) * c4u
      fsum = f(1) * su + f(2) * s2u + f(3) * s3u
     1       + g*u + h
      isum = i(1) + i(2) * cu    + i(3) * c2u
     1            + i(4) * c3u + i(5) * c4u
      jsum = j(1) * su + j(2) * s2u + j(3) * s3u
     1       + k*u + l
      psum = p(1) + p(2) * cu    + p(3) * c2u
     1            + p(4) * c3u + p(5) * c4u
      qsum = q(1) * su + q(2) * s2u + q(3) * s3u
     1       + r*u + s
 
      isig171 = asum + bsum
      isig5k  = esum + fsum
      ikap171 = isum + jsum
      ikap5k  = psum + qsum
 
      isig    = (1.-v) * isig171 + v * isig5k
      ikap    = (1.-w) * ikap171 + w * ikap5k
 
      smgamma = .07832 * (z/at) * dsqrt(d6) / t8
      sg2     = smgamma * smgamma
 
      g0      = 13.00 / dsqrt( 1. + 0.0174 * sg2 ) 
      g2   = (sg2 / pisqr) * (1. + 0.0118 * sg2)**(-3./2.)
 
      fkap    = isig * g0 + ikap * g2
      cstd6   = 1.018 * d6mu23
      nukap   = 9.554e+16 * t8 * dsqrt( 1. + 1./cstd6 ) * fkap
 
      opacity = 4.146d+15 * d6mu / dsqrt( 1. + cstd6 )
     1          * t8 * 1.e+18 / nukap
 
      cndso  = dlog10( opacity )
 
      return
      end 
c***************************************************************************
c  cshell steps from the center shell to the next shell.
c  gshell steps from shell n to shell n+1
c See Schwarzschild & Harm 1965, ApJ, 142, 855
c for a partial explanation of variable meanings
c***************************************************************************
      subroutine cshell
      implicit double precision(a-h,o-z)
      common/coef/hrp(400),hrt(400),hra(400),hbp(400),hbt(400),hba(400),
     1            hpp(400),hpt(400),hpa(400),htp(400),htt(400),hta(400)
      common/cal/ut1,ot1,et1,op1,o1,ep1,qnp1,qnt1,p1,q1,f1,t1,w1,b1,e1,
     1 qe1,ea1,qn1,up1,qe2,qn2,qnp2,qnt2
      common/temp/s1,r1,s2,r2,b2,p2,t2,ea2,xc2,xo2,fca2,f2,q2,w2,c
      common/thermo/u2,up2,ut2,e2,ep2,et2,psi2,pg,o2,op2,ot2,fp,ft,
     1 en2(5),fn,fcc,ce,ci,cif,w,nu
      common/contrl/ds,g,sm,wc,it,nite,ja,jb,j,k,l
 
      data fnat/0.4342945/
c......................................................
      hrp(1)=-(up2)*.333333
      hrt(1)=-(ut2)*.333333
      hra(1)=-.333333*(u2+3.*r2-s2+.622089-sm)
      hpp(1)=0
      hpt(1)=0
      hpa(1)=0
      hbp(1)=(-qe2*ep2+qnp2)*fnat
      hbt(1)=-qe2*(e2-ea2+et2*fnat)+qnt2*fnat
      hba(1)=-b2+(-qe2*(e2-ea2)+qn2)*fnat
      htp(1)=0
      htt(1)=0
      hta(1)=0
      it=-1
      return
c......................................................
      entry gshell
      gs=2.*fnat/(s2-s1)
      ar=gs*(r1-r2)+fnat*(q1+q2)
      ap=gs*(p1-p2)-fnat*(f1+f2)
      gp=2.*fnat/(p1-p2)
      gt=gp*(t1-t2)/(p1-p2)
      at=gp*(t1-t2)-fnat*(w1+w2)
      ab=gs*(b2-b1)+fnat*(qe1*(e1-ea1)+qe2*(e2-ea2)-qn1-qn2)
      c1=q1*up1+hrp(k-1)*(-gs+3.*q1)
      c2=q1*ut1+hrt(k-1)*(-gs+3.*q1)
      c3=gs+3.*q2
      c4=q2*up2
      c5=q2*ut2
      car=ar+hra(k-1)*(gs-3.*q1)
      c6=-gs-f1-4.*f1*hrp(k-1)
      c7=-4.*f1*hrt(k-1)
      c8=-4.*f2
      c9=gs-f2
      cap=ap+4.*f1*hra(k-1)
      c10=gt+w1+fnat*w1*(op1/o1+hbp(k-1)/b1)
      c11=-gp-4.*w1+fnat*w1*(ot1/o1+hbt(k-1)/b1)
      c12=fnat*w2/b2
      c13=-gt+w2+w2*op2*fnat/o2
      c14=gp-4.*w2+w2*ot2*fnat/o2
      cat=at-fnat*w1*hba(k-1)/b1
      c15=gs*hbp(k-1)+fnat*(qnp1-qe1*ep1)
      c16=-qe1*(e1-ea1)+fnat*(qnt1-qe1*et1)+gs*hbt(k-1)
      c17=-gs
      c18=fnat*(qnp2-qe2*ep2) 
      c19=-qe2*(e2-ea2)+fnat*(qnt2-qe2*et2)
      cab=ab-gs*hba(k-1)
      c20=c1*c8-c3*c6
      c21=c2*c8-c3*c7
      c22=c4*c8-c3*c9
      c23=c5*c8
      carp=c8*car-c3*cap
      c24=c12*c15-c10*c17
      c25=c12*c16-c11*c17
      c26=c12*c18-c13*c17
      c27=c12*c19-c14*c17
      cabt=c12*cab-c17*cat
      c28=c20*c25-c21*c24
      c29=c22*c25-c21*c26
      c30=c23*c25-c21*c27
      cp=c25*carp-c21*cabt
      c32=c20*c26-c22*c24
      c33=c20*c27-c23*c24
      ct=c20*cabt-c24*carp
      cd1=-1./c28
      cd2=-1./c17
      cd3=-1./c3
      hpp(k)=c29*cd1
      hpt(k)=c30*cd1
      hpa(k)=-cp*cd1
      htp(k)=c32*cd1
      htt(k)=c33*cd1
      hta(k)=-ct*cd1
      hbp(k)=( c15*hpp(k)+c16*htp(k)+c18)*cd2
      hbt(k)=( c15*hpt(k)+c16*htt(k)+c19)*cd2
      hba(k)=( c15*hpa(k)+c16*hta(k)-cab)*cd2
      hrp(k)=( c1*hpp(k)+c2*htp(k)+c4)*cd3
      hrt(k)=( c1*hpt(k)+c2*htt(k)+c5)*cd3
      hra(k)=( c1*hpa(k)+c2*hta(k)-car)*cd3
      return
      end 
c**************************************************************************
c four-point lagrange interpolation via interpolating polynomial.
c**************************************************************************
      subroutine dali(x,arg,val,y)
      implicit double precision(a-h,o-z)
c      dimension arg(1),val(1) 
      dimension arg(*),val(*) 
 
      do 6 j=2,4
        iend=j-1
        do 2 i=1,iend
          h=arg(i)-arg(j)
          val(j)=(val(i)*(x-arg(j))-val(j)*(x-arg(i)))/h
2       continue
6     continue
      y=val(4)
 
      return
      end 
c**************************************************************************
c  writes out the model according to the switches (ipn's) that have
c  been set.
c**************************************************************************
      subroutine end(line)
      implicit double precision(a-h,o-z)
      common/shells/ sa(400),ra(400),ba(400),pa(400),ta(400),
     1 ea(400),xca(400),fca(400),s(400),r(400),b(400),
     2 p(400),t(400),e(400),xc(400),sk(400),
     3 rk(400),bk(400),pk(400),tk(400)
      common/coef/hrp(400),hrt(400),hra(400),hbp(400),hbt(400),hba(400),
     1            hpp(400),hpt(400),hpa(400),htp(400),htt(400),hta(400)
      common/contrl/ds,g,sm,wc,it,nite,ja,jb,j,k,l
      common/times/tmax1,tmax2,tmax3,time,time1,f,dg,dg1,g1,ip5,ip6,ip8
      common/vca/modnr
      common/dvca/sg
      common/deldg/deldg
      common/fin/ifin
      common/tfit/tfit(100),told,ifit,itfit,nite1
      common/surf/u(2,3),v(2,3),ww(2,3),rm,bm,is,ks,ls,ms,kstart,npass
      common/stuck/dgcavg,istuck
      common/gne/gnew
      dimension stash2(8000)
      dimension x(400,7),y(400,7),z(400,5)
      equivalence(stash2(1),sa(1))
      equivalence (sa(1),x(1,1)),(s(1),y(1,1)),(sk(1),z(1,1))
c..........................................................................
603   format(' sk= ',1pe16.8,'  dr2=',0pf12.7,'  db2=',1pe12.4,
     1 '  dp2= ',0pf11.6,'  dt2= ',f11.6)
701   format('  nite=',i12,'  jmax=',i12,'  smax=',1pe16.8,
     1 '  dtmax=',e12.4)
704   format(' s=',1pe16.8,'  r=',0pf12.7,'  b=',1pe12.4,'  p=',0pf11.6,
     1 '  t=',0pf11.6,'  e= ',1pe14.7,'  xc=',0pf8.6,'  xo=',f8.6)
718   format (//14h dtmax too big//)
720   format(' COARSE approximation, time= ',f8.3) 
727   format(' VERY COARSE approximation, time= ',f8.3)
c..........................................................................
      line = 3
 
      call stitch(hbp(k),hbt(k),hba(k),hrp(k),hrt(k),hra(k),dp2,dt2)
 
      jb=k
      dtmax=0.0
 
      do 611 j=1,jb 
         k=jb-j+1
         dr2=hrp(k)*dp2+hrt(k)*dt2+hra(k)
         db2=hbp(k)*dp2+hbt(k)*dt2+hba(k)
         if ( ip5.gt.0 .or. (ip5.eq.0 .and. nite .le. 1)) then
            write(17, 603)sk(k),dr2,db2,dp2,dt2
         endif
         s(k)=sk(k) 
         r(k)=rk(k)+f*dr2
         b(k)=bk(k)+f*db2
         p(k)=pk(k)+f*dp2
         t(k)=tk(k)+f*dt2
         if(dabs(dtmax).lt.dabs(dt2)) then
            dtmax=dt2
            smax=sk(k)
         endif
         dp1=hpp(k)*dp2+hpt(k)*dt2+hpa(k)
         dt1=htp(k)*dp2+htt(k)*dt2+hta(k)
         dp2=dp1
         dt2=dt1
  611 continue
 
      write(17,701) nite,jb,smax,dtmax
      nite=nite-1
      if ( ip6 ) 706,702,703
  702 if ( nite ) 703,703,714 
  703 write(17, 704) ((y(j,i),i=1,8),j=1,jb)
      if ( ip5 .gt. 0 .and. ip6 .gt. 0 ) stop
c**************************************************************
c  Stash the interior shells
c  Call eprep to calculate converged envelope, store it in aa,
c  and write it to tape50!
c**************************************************************
  706 if(nite .le. 0 ) then
c***  Read the interior shells back in!
         nshint = jb
         ifin = 1
	 write(11) stash2
	 rewind 11
c*** write out model to tape9
         call write3
c*** write out complete model to tape 50
	 call eprep(nshint)
         ifin = 0
	 read(11) stash2
	 rewind 11
c....................................................................
         if ( ip8 .ge. 0 ) then
            pext=10.**(g-g1)
            call arline(x,y,z,ja,jb,5,ip8,pext)
         endif
         call armove(x,y,jb,7)
         ja=jb
         if(ip8.gt.0) call armove(y,z,jb,5)
         line=1
	 if (tfit(ifit).lt.1.d-03) then
	   write(*,*) 'Converged to the',ifit-1,' specified temperatures'
	   write(*,*) 'Exiting'
	   stop
	 endif
         return
      endif
714   if ( nite .gt. 1 ) then 
         if ( dabs(dtmax) .ge. tmax1 ) then
            line=2
            return
         else
            dg=dmin1(dg1,dg+.5*time)
            time=dmin1(time1,time+.1)
c put convergence criteria for specific temperatures here
            bl=dlog10(b(jb))
            tel=bl/4.-rm/2.+9.18458 
            if ( (itfit.eq.0 .and. tel.gt.log10(tfit(ifit)) ) .or. 
     1            abs(10**tel-tfit(ifit)).lt.0.1 ) then
              write(*,*) 'Done: tel tfit ifit itfit',10**tel,tfit(ifit),ifit,itfit
     	      told2= told
     	      told = tel
	      if ( abs(10**tel-tfit(ifit)).lt.0.1 ) itfit=1
	      if (itfit.eq.1) then
	        ifit=ifit+1
	        dg = dg + 0.1
     	      endif
	      itfit=0
	      istuck=0
              nite = 1
              line = 2
c having converged, we need to store the new g-value, so that the
c age can later be calculated (correctly)
	      gnew=g
c if we're too close to the next target temperature (within 50 K), then
c go ahead and try to converge to it
	      if (itfit.eq.0 .and. abs(10**tel-tfit(ifit)) .lt. 50.0 ) then
	        write(*,*) 'lengthening timestep to include next t(i)'
                nite = nite1
                line = 2
	        itfit=1
		told=told2
c                gcor=(10**told-tfit(ifit))/(10**told-10**tel)
c	        gcor=log10(1.+0.40*(gcor - 1.))
c                g = g + gcor
	      endif
            else 
             nite = nite1
             line = 2
c             gcor=log10( (10**told-tfit(ifit))/(10**told-10**tel) )
             gcor=(10**told-tfit(ifit))/(10**told-10**tel)
	     if (istuck.gt.25 .and. abs(gcor).gt.0.05) then
	       fac=float(istuck-25)/100.
	       gcor=log10(1.+(1.0+fac)*(gcor - 1.))
	     else
	       gcor=log10(1.+0.40*(gcor - 1.))
	     endif
c	     gcor=log10(1.+0.40*(gcor - 1.))
c	     gcor=log10(gcor)
             g = g + gcor
c             write(*,707) 'Last iteration: tel tfit gcor',10**tel,tfit(ifit),gcor
             write(*,707) 10**tel,tfit(ifit),gcor
707          format('Last iteration: tel tfit gcor ',2f9.2,x,f7.4)
	     itfit=1
	     istuck=istuck+1
             return
            endif
            return
         endif
      else
         if ( dabs(dtmax) .lt. tmax2 ) then
            time=time*.8
            dg=dg-time
            write(*,720) time
            write(17,720) time
            write(20,720) time
            modnr = modnr - 1 
            sg=sg-10.**(g-7.499095)
            g = g + deldg
            nite = 1
            line = 1
            return
         elseif ( dabs(dtmax) .lt. tmax3 ) then
            time=time*.8
            dg=dg-time
            write(*,727) time
            write(17,727) time
            write(20,727) time
            modnr = modnr - 1 
            sg=sg-10.**(g-7.499095)
            g = g + deldg
            nite = 1
            line = 1
            return
         else
            write(*,718)
            write(17,718)
            write(20,718)
            modnr = modnr - 1 
            sg=sg-10.**(g-7.499095)
            g = g + deldg
            nite = 1
            line = 1
            return
         endif
      endif
      end 
c*******************************************************************************
c     calculates white dwarf envelopes
c...............................................................................
c     the variables are r the radius,   m the log of the mass within a
c     sphere of radius r,   p the log of the pressure,   t the log of the
c     temperature,   el1 the degeneracy parameter,   ol1 the log of the
c     total opacity,   rl1 the log of the density,   atg the adiabatic
c     gradient,   rtg the radiative gradient,   phs the mixing length,
c     drdtp the derivative of the density(log) with respect to the
c     temperature(log) at constant pressure,   cp the specific heat per
c     gram at constant pressure,   ttg the actual temperature gradient.
c...............................................................................
c     the control parameters are:
c     nmax,  maximum number of shells
c     nsuff,  indicates if max number of shells is exceeded 
c     nlim1,  shell number at the bottom of the convection zone
c     kind,  shell number at the top of the convection zone 
c     nlim,  total number of shells for a model
c     1/nhp,  fraction of appropriate scale height used as integration step
c     iter,  total number of iterations 
c     nlum,  number of luminosities
c     lmax,  number of isotherms in eos table
c     ml,  number of density points per isotherm (eos)
c     lmam,  number of isotherms for which optical depths are computed
c...............................................................................
c     the units are cgs throughout
c*******************************************************************************
      subroutine env
      implicit double precision(a-h,o-z)
      double precision m
      logical writeit
      common/shells/r(800),m(800),p(800),t(800),el1(800),ol1(800),
     1 rl1(800),atg(800),rtg(800),phs(800)
      parameter (nd=38)
c      common/tenv/rhok(3,60,35),eta(3,60,35),pp(3,60,35),datg(3,60,35),
c     1 od(3,18,35),tk(3,60),uu(3,60,35),xtt(3,60,35),xrt(3,60,35)
      common/tenv/rhok(3,60,nd),eta(3,60,nd),pp(3,60,nd),datg(3,60,nd),
     1 od(3,18,nd),tk(3,60),uu(3,60,nd),xtt(3,60,nd),xrt(3,60,nd)
      common/thresh/thresh
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      common/xcomp/xcomp(400,3)
      common/slr/sl(11)
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1 ,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
      common/d/ ml(3,60),nmax,kind,lmax,nsuff,nlim,nlim1,j,iter,lmam, 
     1 nlum,ko,nhp
      common/grad/ drdtp(400),cp(400),ttg(400),drdtpx,cpx,pecx,ttgx,
     1 deptx,dpdr,dtdr,dmdr,dr,xr,xt,cv,drdptx
      common/surf/u(2,3),v(2,3),w(2,3),rm,bm,is,ks,ls,ms,kstart,npass 
      common/contrl/ds,g,smass,wc,idummy(7)
      common/terp/stpms,rat1
      common/prop/ bn2(400),ac2(400)
      common/writeit/writeit
      common/oldval/bold,teold 
      common/fin/ifin
      common/dfin/rfin,blfin
      common/prep/iprep
      common/dprep/aa(20,500), ecv(400), ext(400), exr(400)
      common /mixl/ aml,mls
      data dlummax, dtemax /0.1d0, 0.5d0/
c...............................................................................
c fundamental constants (Pi,Avagadro's number,Boltzmann const.)
c...............................................................................
      bk=1.380622e-16
      ano=6.022169e+23
      pi=3.1415927e0
      stop=stpms+smass-33.298635
      sm=10.**(smass-33.298635)
      gslast=1e8
      xmass = sm
      if ( ifin .eq. 0 ) then
        rs=10.**(w(2,is)-10.8425968)
        sl(1)=10.**w(1,is)
        writeit = .false.
      elseif ( ifin .eq. 1 ) then
        rs = 10.**(rfin-10.8425968)
        sl(1) = 10.**blfin
      endif
c...............................................................................
c     compute the surface gravity gs and scale the optical depth tables
c     accordingly
c...............................................................................
      gs=27398.*sm/(rs*rs)
      glog=dlog10(gslast/gs)
      do 360 jm=1,3 
         do 260 l=1,lmam
            kmax=ml(jm,l)
            do 160 k=1,kmax
               od(jm,l,k)=od(jm,l,k)+glog
 160        continue
 260     continue
 360  continue
c...............................................................................
c  set chem comp for gray atm integration
c  specify a luminosity and compute the effective temperature te
c...............................................................................
      call ccomp
      j = 1
      nsuff=1
      nlim1=0
      am=(x(1)*az(1) + x(2)*az(2) * x(3)*az(3)) * 1.660531e-24
      te=5800.*(sl(j)/(rs*rs))**0.25
c...............................................................................
c... if stepped far enough in h-r diagram, then write out envelope
c... to tape6
c...............................................................................
      if ( ifin .eq. 1 ) then
         dlum = dabs( bold - bm ) 
         dteff = dabs( teold - dlog10(te) ) 
         if ( dlum .gt. dlummax .or. dteff .gt. dtemax ) then
            bold = bm
            teold = dlog10( te ) 
            writeit = .true.
            write(6,*) 'writing envelope to tape6.  teff =',te
            write(6,*) '--------------------------------------------'
         endif
      endif
      if ( writeit ) write(26,103) amhyhe,amheca
103   format(1x,' amhyhe=',1pe14.4,'  amheca=',1pe14.4)
c...............................................................................
c  integrate gray atmosphere
c...............................................................................
      call gray
      it=2
      nhp=8
      iter=1
c...............................................................................
c     start the inward integration with values provided by the gray
c     subroutine,  if there is convection take the geometrical depth
c     as zero
c...............................................................................
      p(1)=ps
      t(1)=ts
      r(1)=rs*6.96e+10
      m(1)=dlog10(sm)
      dept=1e-20
      deptx=dept
      kind1=0
c...............................................................................
      do 3 n=1,nmax 
         ko=n
         tl=t(n)
         pl=p(n)
         xmass=10.**m(n)
 
         call ccomp 
 
         am=(x(1)*az(1) + x(2)*az(2) * x(3)*az(3)) * 1.660531e-24
         do 105 iii = 1,3
            xcomp(n,iii) = x(iii)
105      continue
c...............................................................................
c  check that our (p,t) point is on the eeos table
c    (inserted 6/13/88)
c  skip over check if teff > thresh (30,000)
c...............................................................................
         if (te .gt. thresh) goto 987
         itchk = (tl - tk(je,1)) / (tk(je,2) - tk(je,1)) + 1
         if ( tl .lt. tk(je,1) .or. tl .gt. tk(je,lmax) ) then
            write(26,*) 'subroutine env' 
            write(26,321) tl, tk(je,1), tk(je,lmax), je
321         format(' off eeos.  t:',f5.2,' tk(1), tk(lmax): ',2f6.2,
     1           '    je: ',i1)
            if ( tl .lt. tk(je,1) ) itchk = 1
            if ( tl .gt. tk(je,lmax) ) itchk = lmax
         endif
 
         mltmp = ml(je,itchk) 
         if ( pl .lt. pp(je,itchk,1) .or.
     1      pl .gt. pp(je,itchk,mltmp ) ) then
            write(26,*) 'subroutine env' 
            write(26,432) pl, tl, je, itchk, pp(je,itchk,1), 
     1                 je, itchk, mltmp, pp(je,itchk,mltmp) 
432         format(' off eeos. '/'  p:',f7.3,'  t:',f7.3,/
     1                        '  pp(',i1,',',i2.2,',1): ',f7.3,/
     2                        '  pp(',i1,',',i2.2,',',i2.2,'):',f7.3) 
         endif
c...............................................................................
c  end eeos check
c...............................................................................
987   continue
c...............................................................................
c     given p and t at r call the subroutine state and store the values
c     of el1,rl1,ol1,atg,rtg,drdtp,cp
c...............................................................................
         call estatem
         ol1x = opac3(rl1x,tl)
         rtgx = 7.732749e+9*10.**(ol1x+pl-4.*tl)*sl(j)/xmass
         ol1(n)=ol1x
         rtg(n)=rtgx
         el1(n)=el1x
         rl1(n)=rl1x
         atg(n)=atgx
         drdtp(n)=drdtpx
         cp(n)=cpx
         gam1=((xt**2)*10.e0**(p(n)-rl1x-t(n))/cv)+xr
         bn2(n)=-drdtpx*(rtgx-atgx)*((6.67e-8*1.989e+33*
     1          10.e0**m(n)/r(n)**2)**2)*10.e0**(rl1x-p(n)) 
         ac2(n)=gam1*10.e0**(p(n)-rl1x)/r(n)**2
         dpdr=-27398.*(10.**m(n))*rho*(6.96e+10/r(n))**2
         dmdr=4.*pi*rho*r(n)**2
         if ( ifin .eq. 1 ) then
            ecv(n) = cv
            ext(n) = xt
            exr(n) = xr
	 endif
c...............................................................................
c     define the integration step as a fraction of the pressure scale 
c     height
c...............................................................................
         dr=10.**p(n)/nhp/dpdr
c...............................................................................
c     Schwarzschild's test, if radiative equilibrium take rtg
c     as the actual gradient, if convective define the mixing length
c     as the smaller of either the local pressure scale height or the 
c     geometrical depth from the top of the convection zone and call
c     gradt to evaluate the true gradient
c...............................................................................
         if ( atg(n) .le. rtg(n))  then 
            nlim1=n 
            kind=n-kind1
            kind1=kind1+1
            phs(n)=dabs(dr*nhp)
            dept=dept+dabs(dr) 
            deptx=dept
c comment out the next line to remove the Bohm-Stuckl prescription
c            if ( phs(n) .gt. dept ) phs(n)=dept
            pecx=9./8./(1.601743e+07*cpx*10.**(ol1x+2.5*rl1x+m(n)
     1           -3.*t(n)-0.5*p(n))*(dabs(drdtpx))**0.5
     2           *(phs(n)**2)*(6.96e+10/r(n))**2)
            pecx = pecx/aml/aml
            if(mls.eq.0)then
call Bohm-Vitense version
              call gradt
             else
              pecx = pecx * (8./9.)/dsqrt(2.d0)
              call gradt2
            endif
            ttg(n)=ttgx
         else
            ttg(n)=rtg(n)
            kind1=0 
            dept=0. 
         endif
 
         dtdr=ttg(n)*dpdr*10.**(t(n)-p(n))
c...............................................................................
c     evaluate r,p,t,m at the next shell and quit if the envelope
c     mass fraction exceeds -stpms or the temperature exceeds tk(lmax)
c...............................................................................
         call intg
 
	 je=3
c	 write(*,*) je,lmax
         if ( t(n+1) .gt. tk(je,lmax) ) then
            write (26,*)' off eos. t(n+1) gt tk(lmax)',
     1                   t(n+1),tk(je,lmax),n
	    goto 13
	 elseif ( stop .gt. m(n) ) then
            write (26,*)' stop gt m(n).  n=',n, ' n -> n-1 '
	    goto 13
         elseif ( n+2 .gt. nmax ) then
 	    write (26,*)' too many shells.', n+2
            goto 11
         endif
         nlim=n+1
3     continue
c...............................................................................
c end of large loop (envelope integration) at statement 3
c if we go to 11, then we ran out of memory, and didn't integrate
c all the way to the fitting point.
c...............................................................................
11    nsuff=-1
c...............................................................................
13    plim2=10.**p(nlim)
      plim1=plim2
      acu=0.
      n = n-1
 
      call write4
      if ( writeit ) writeit = .false.
 
      gslast=gs
 
      return
      end 
c*****************************************************************
c EPREP -- white dwarf carbon stratified prep code subroutine
c  writes prepped models to tape50
c*****************************************************************
      subroutine eprep(nshint)
      implicit double precision(a-h,o-z)
      double precision m
      common/shells/r(800),m(800),p(800),t(800),el1(800),ol1(800),
     1rl1(800),atg(800),rtg(800),phs(800)
      common/xcomp/xcomp(400,3)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      common/bldx/bled
      common/terp/stpms,rat1
      common/temp/s1,r1,s2,r2,b2,p2,t2,e2,xc2,xo2,fca2,f2,q2,w2,c
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1 ,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
      common/d/ ml(3,60),nmax,kind,lmax,nsuff,nlim,nlim1,jjj,iter,lmam, 
     1 nlum,ko,nhp
      common/prep/iprep
      common/dprep/aa(20,500), ecv(400), ext(400), exr(400)
      common/grad/ drdtp(400),cp(400),ttg(400),drdtpx,cpx,pecx,ttgx,
     1 deptx,dpdr,dtdr,dmdr,dr,xr1,xt1,cv1,drdptx
      common/prop/ bn2(400),ac2(400)
      common/kapder/rkapr(800),rkapt(800)
      common/dfin/rfin,blfin
      common/opacfin/ifinal
      common/alpha/alph(4),qmix1,qmix2,zone(2,2)
      logical first
      data amsun,alsun,const/1.989e+33,3.90e+33,3.947092e+09/
      data xx/.999999/
      data first/.true./
c.................................................................
987   format(a80)
1002  format(1x,4e16.9)
1003  format(4e16.9)
1004  format(1x,16i4)
1005  format(16i4)
c.................................................................
      icmp = 0
      small = 1.e-4
      nl=nshint
      xc2 = 1.
      xo2 = 0.
c.................................................................
c  call env for the converged model
c.................................................................
      call env
      ifin = 0
      nel=nlim

      if( iprep .eq. 0 )then
c ******* no reason to continue here **********
        return
      endif
      nel = nel - 1
      do 34 n=1,nel
	 index = nel-n+1
c******* these guys are placed in reverse order *****
         rrr = r(index)
         amm = m(index)
	 ttt = t(index)
         rho = rl1(index)
         ppp = p(index)

         aa(1,nl+n)=r(index)
	 x(1) = xcomp(index,1)
	 x(2) = xcomp(index,2)
	 x(3) = xcomp(index,3)
	 if ( n .eq. nel ) then
           x(1) = 0.
           x(2) = 0.
           x(3) = 1.
	 endif
c mass (grams)
         aa(2,nl+n)=10.**m(index)*amsun
c temperature (kelvins)
         aa(4,nl+n)=10.**t(index)
c density (g/cm^3)
         aa(5,nl+n)=10.**rl1(index)
c pressure (dynes/cm^2)
         aa(6,nl+n)=10.**p(index)
c energy generation rate + derivatives (eps, epsr, and epst)
         aa(7,nl+n)=0.0
         aa(11,nl+n)=0.0
         aa(12,nl+n)=0.0
c**** use delrad as temp. gradient except in convection zone, then use
c**** deltrue from gradt (or gradt2). *********************************
         if(rtg(index).lt.atg(index))then
            tdel=rtg(index)
         else
            tdel=ttg(index)
         endif
c temperature gradient (del)
         aa(15,nl+n)=tdel
c adiabatic temperature gradient (delad)
         aa(16,nl+n)=atg(index)
c helium profile
         aa(17,nl+n)=xcomp(index,2)
c radiative luminosity profile
         aa(3,nl+n)=alsun*(10.0**(blfin))*(ttg(index)/rtg(index))
c heat capacity Cv
         aa(8,nl+n)=ecv(index)
c pressure gradients chi rho and chi T
         aa(9,nl+n)=exr(index)
         aa(10,nl+n)=ext(index)
c opacity
         tl=t(index)
         pl=p(index)
         rhol = rl1(index)
	 oplo = opac3(rhol,tl)
         aa(18,nl+n)=10.**oplo
         ifinal = 1
c**** start 5-point opacity derivative stuff here. ***
c         write(17,*) 'je,x',je,x(1),x(2),x(3)
         if(x(1).eq.1.0 .or. x(2).eq.1.0 .or. x(3).eq.1.0)then
           if(x(1).eq.1.0)then
             je=1
           endif
           if(x(2).eq.1.0)then
             je=2
           endif
           if(x(3).eq.1.0)then
             je=3
           endif
           call opacder(rhol,tl,opcs,dlklr,dlklt)
           aa(13,nl+n)=dlklr
           aa(14,nl+n)=dlklt
c note: replace previous opacity value if we can get it here.
           aa(18,nl+n)=10.**opcs
          else
c 5 point differencing
           oplo5=opac3(rhol+0.02,tl)
           oplo1=opac3(rhol+0.01,tl) 
           oplo2=opac3(rhol-0.01,tl) 
           oplo6=opac3(rhol-0.02,tl)
           oplo7=opac3(rho,tl+0.02)
           oplo3=opac3(rhol,tl+0.01)
           oplo4=opac3(rhol,tl-0.01)
           oplo8=opac3(rho,tl-0.02)
c opacitry derivatives (kapr(13) and kapt(14))
         aa(13,nl+n)=(oplo6-(8.*oplo2)+(8.*oplo1)-oplo5)/(12.*0.01)
         aa(14,nl+n)=(oplo8-(8.*oplo4)+(8.*oplo3)-oplo7)/(12.*0.01)
c           write(17,1901) nl+n,oplo5,oplo1,oplo2,oplo6
c           write(17,1902) nl+n,oplo7,oplo3,oplo4,oplo8
         endif
c if He fraction is constant, set Ledoux to zero.
         if(aa(17,nl+n).eq.aa(17,nl+n-1))then
           aa(19,nl+n) = 0.0
           go to 34
         endif
         if(aa(17,nl+n-1).eq.0.0 .or. aa(17,nl+n).eq.0.0)then
           aa(19,nl+n) = 0.0
           go to 34
         endif
         if(index.eq.nel)then
           aa(19,nl+n) = 0.0
          else
c call up modified ledoux term subroutine
           call ledoux
           dly = ( dlog(aa(17,nl+n)) - dlog(aa(17,nl+n-1)) )
           dlp = ( dlog(aa(6,nl+n)) - dlog(aa(6,nl+n-1)) )
c           bledoux=(1./aa(10,nl+n))*bled*(dly/dlp)
           bledoux=(1./aa(10,nl+n))*bled*(dly/dlp)
c Note: there is a minus sign difference between this ledoux term 
c in the core and the one in Brassard et al. (1991, ApJ, 367, 601).
c This was "corrected" by the absolute value below, but I want to be
c able to see if certain configurations are convectively unstable in the
c core, so I am going to remove the dabs() (MHM April 1998).
c           aa(19,nl+n)=dabs(bledoux)
           aa(19,nl+n)=bledoux
c	   write(*,1901) nl+n,bled,bledoux,x(1),x(2),x(3)
1901   format(i4,x,5(e12.5))
           if(x(1).ne.0.0 .and. x(2).gt.0.99 .and. x(3).ne.0.0 .and. 
     1       bledoux.lt.0.) then
             aa(19,nl+n)= 0.d0
           endif
c           write(17,*) nl+n,bledoux,1./aa(10,nl+n),dly/dlp,bled
         endif
c         write(*,*) nl+n,x(2),bledoux,1./aa(10,nl+n),dly/dlp,bled
   34 continue
c1901  format(1x,'rho opac',i4,4(1pe12.5,1x))
c1902  format(1x,'temp opac',i4,4(1pe12.5,1x))
c***** finis!
      ifinal = 0
c.................. 
c now write it out. 
c.................. 

      nshell=nl+nel
c      call nodoubl(aa,nshell)
      write(50,1010) nshell,nl,nel,amhyhe,amheca,
     1  alph(1),alph(2),alph(3),alph(4),stpms
 1010 format(i5,'  =  ',i4,'  +  ',i4,2(2x,1pe9.3),0p,4(f7.4,x)
     1  1x,1pe10.3)
c..............................................................
c smooth out del and delad at core-envelope boundary
c set mver for appropriate variable
c 1. delad; 2. del; 3. chiT; 4. chiRho
c..............................................................
      do 33 i=1,nshell
        if(i.eq.nl)then
c delad
          diff = aa(16,i+1) - aa(16,i)
          if(diff.gt.0.002)then
            mver = 1
            call smooth(nl,mver)
            write(*,*) 'smoothing delad'
          endif
c del
          diff2 = aa(15,i+1) - aa(15,i)
          if(diff2.gt.0.002)then
            mver = 2
            call smooth(nl,mver)
            write(*,*) 'smoothing del'
          endif
c chi T
          diff3 = aa(10,i+1) - aa(10,i)
          if(diff3.gt.0.002)then
            mver = 3
            call smooth(nl,mver)
            write(*,*) 'smoothing chi T'
          endif
c chi rho
          diff2 = aa(9,i+1) - aa(9,i)
          if(diff4.gt.0.002)then
            mver = 4
            call smooth(nl,mver)
            write(*,*) 'smoothing chi rho'
          endif
        endif
33    continue
c..............................................................
c  log 1-q into place 11
c changed to place 21 by pab
c      alc4pig = -6.07666 - 2. * dlog10(gs)
c      do 1030 i = 1,nshell
c        ame1=1.d0-(aa(2,i)/aa(2,nshell))
c        if (ame1 .ge. 1.d-13) then
c           aa(21,i) =dlog10(ame1)
c        else
c           aa(21,i) = alc4pig + dlog10(aa(6,i))
c        endif
c1030  continue
c..............................................................
c Write out model to tape50
c..............................................................
      do 2 i=1,20
        write(50,1020) (aa(i,n), n=1,nshell)
2     continue
c..............................................................
c commenting out Pierre's stuff for now
c      write(50,1020) (aa(11,n),n=1,nshell)
c      write(50,1020) (aa( 1,n),n=1,nshell)
c      write(50,1020) (aa( 2,n),n=1,nshell)
c      write(50,1020) (aa( 3,n),n=1,nshell)
c      write(50,1020) (aa( 4,n),n=1,nshell)
c      write(50,1020) (aa( 5,n),n=1,nshell)
c      write(50,1020) (aa( 6,n),n=1,nshell)
c      write(50,1020) (aa( 8,n),n=1,nshell)
c      write(50,1020) (aa( 9,n),n=1,nshell)
c      write(50,1020) (aa(10,n),n=1,nshell)
c      write(50,1020) (aa(13,n),n=1,nshell)
c      write(50,1020) (aa(14,n),n=1,nshell)
c      write(50,1020) (aa(15,n),n=1,nshell)
c      write(50,1020) (aa(16,n),n=1,nshell)
c      write(50,1020) (aa(18,n),n=1,nshell)
c      write(50,1020) (aa(17,n),n=1,nshell)
c..............................................................
1020  format(1p,4e22.15)
c special format for Umin Lee's models, since his code pukes
c if there's more than 80 characters per line.
c1020  format(1p,3e24.16)
      return
      end 
c*******************************************************************************
c     3-point lagragian interpolation in the pp-tk grid to find thermo
c     quantities el1,rl1,ol1,atg,rtg,drdtp,cp
c*******************************************************************************
      subroutine estate
      implicit double precision(a-h,o-z)
      parameter (nd=38)
c      common/tenv/rhok(3,60,35),eta(3,60,35),pp(3,60,35),datg(3,60,35),
c     1 od(3,18,35),tk(3,60),uu(3,60,35),xtt(3,60,35),xrt(3,60,35)
      common/tenv/rhok(3,60,nd),eta(3,60,nd),pp(3,60,nd),datg(3,60,nd),
     1 od(3,18,nd),tk(3,60),uu(3,60,nd),xtt(3,60,nd),xrt(3,60,nd)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
c      common/slum/sl(11)
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1  ,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
c     1  ,atgx,rtgx,ueng,tl,pl,xmass,acu,am,bk,an0,pi
      common/d/ ml(3,60),nmax,kind,lmax,nsuff,nlim,nlim1,j,iter,lmam, 
     1  nlum,ko,nhp 
      common/grad/ drdtp(400),cp(400),ttg(400),drdtpx,cpx,pecx,ttgx,
     1  deptx,dpdr,dtdr,dmdr,dr,xr,xt,cv,drdptx
c      common/icheck/icheck
      dimension xtl(3),xrl(3),el(3),rl(3),at(3)
c     dimension ul(3)
      integer k1(3),k2(3),k3(3)
c..............................................................
c trap point in t, after checking if = low boundry or less, high
c boundry or more
c..............................................................
      if ( tl .lt. tk(je,2) ) then
         l1=1
         l2=2
         l3=3
      else if ( tl .gt. tk(je,lmax-1) ) then
         l3=lmax
         l2=lmax-1
         l1=lmax-2
      else
c The last 6 isotherms are at a different
c spacing in log T of 0.25  in the current incarnation of the
c 'envelope' EOS extended to high temperatures.  
         if (tl.lt.7.54d0) then
           l3=int((tl-tk(je,1))/(tk(je,2)-tk(je,1)))+2
           l2=l3-1
           l1=l3-2
         else
           l3=int((tl-tk(je,54))/(tk(je,55)-tk(je,54)))+56
           l2=l3-1
           l1=l3-2
         endif
      endif
c..............................................................
c get fractional distance between mesh points in temp
c lagrange coefficients.
c..............................................................
      dtdt=(tl-tk(je,l2))/(tk(je,l3)-tk(je,l2))
      dt1=tk(je,l1)-tl 
      dt2=tk(je,l2)-tl 
      dt3=tk(je,l3)-tl 
      dtdt1=dt2*dt3/((dt1-dt2)*(dt1-dt3))
      dtdt2=dt1*dt3/((dt2-dt1)*(dt2-dt3))
      dtdt3=dt1*dt2/((dt3-dt1)*(dt3-dt2))
c..............................................................
c Now trap point in p
c Note, we come loop back to 8 for l = l1, l2, l3 
c This is a *slow* (linear) search thru the grid in pressure
c and I wouldn't be surprised if the program spends a large 
c fraction of its time right here
c..............................................................
      l=l1
8     kmax=ml(je,l) 
      lp=l+1-l1
      if ( pl .lt. pp(je,l,2) ) then
         k1(lp)=1
         k2(lp)=2
         k3(lp)=3
      else
         do 16 k=3,kmax
            if ( pl .gt. pp(je,l,k) ) then
               goto 16
            else
               k3(lp)=k
               k2(lp)=k-1
               k1(lp)=k-2
               go to 14
            endif
16       continue
         k3(lp)=kmax
         k2(lp)=kmax-1
         k1(lp)=kmax-2
      endif
c..............................................................
14    kl1=k1(lp)
      kl2=k2(lp)
      kl3=k3(lp)
      dp1=pp(je,l,kl1)-pl
      dp2=pp(je,l,kl2)-pl
      dp3=pp(je,l,kl3)-pl
      dpdp1=(pl-pp(je,l,kl2))*(pl-pp(je,l,kl3))/(pp(je,l,kl1)-
     1      pp(je,l,kl2))/(pp(je,l,kl1)-pp(je,l,kl3))
      dpdp2=(pl-pp(je,l,kl1))*(pl-pp(je,l,kl3))/(pp(je,l,kl2)-
     1      pp(je,l,kl1))/(pp(je,l,kl2)-pp(je,l,kl3))
      dpdp3=(pl-pp(je,l,kl1))*(pl-pp(je,l,kl2))/(pp(je,l,kl3)-
     1      pp(je,l,kl2))/(pp(je,l,kl3)-pp(je,l,kl1))
c..............................................................
c Interpolate in P for each T point
c..............................................................
      el(lp)=eta(je,l,kl1)*dpdp1+eta(je,l,kl2)*dpdp2+eta(je,l,kl3)*
     1       dpdp3
      at(lp)=datg(je,l,kl1)*dpdp1+datg(je,l,kl2)*dpdp2+datg(je,l,kl3)*
     1       dpdp3
      rl(lp)=rhok(je,l,kl1)*dpdp1+rhok(je,l,kl2)*dpdp2+rhok(je,l,kl3)*
     1       dpdp3
c      ul(lp)=uu(je,l,kl1)*dpdp1+uu(je,l,kl2)*dpdp2+uu(je,l,kl3)*
c     1       dpdp3
      xtl(lp)=xtt(je,l,kl1)*dpdp1+xtt(je,l,kl2)*dpdp2+xtt(je,l,kl3)*
     1       dpdp3
      xrl(lp)=xrt(je,l,kl1)*dpdp1+xrt(je,l,kl2)*dpdp2+xrt(je,l,kl3)*
     1       dpdp3
 
      if ( lp .ge. 3 ) goto 19
      l=l+1
      go to 8
c..............................................................
c this is the bottom of the loop
c Now interpolate in T for actual pressure.
c..............................................................
19    el1x=el(2)+0.5d0*dtdt*(el(3)-el(1))+dtdt*dtdt 
     1      *(0.5d0*(el(1)+el(3))-el(2))
      atgx=at(2)+0.5d0*dtdt*(at(3)-at(1))+dtdt*dtdt 
     1      *(0.5d0*(at(1)+at(3))-at(2))
      rl1x=rl(2)+0.5d0*dtdt*(rl(3)-rl(1))+dtdt*dtdt 
     1      *(0.5d0*(rl(1)+rl(3))-rl(2))
c      ul1x=ul(2)+0.5d0*dtdt*(ul(3)-ul(1))+dtdt*dtdt 
c     1      *(0.5d0*(ul(1)+ul(3))-ul(2))
      xt1x=xtl(2)+0.5d0*dtdt*(xtl(3)-xtl(1))+dtdt*dtdt 
     1      *(0.5d0*(xtl(1)+xtl(3))-xtl(2))
      xr1x=xrl(2)+0.5d0*dtdt*(xrl(3)-xrl(1))+dtdt*dtdt 
     1      *(0.5d0*(xrl(1)+xrl(3))-xrl(2))
c..............................................................
c compute final thermo properties
c..............................................................
      xt = xt1x
      xr = xr1x
      drdtpx = xt/xr
      drdptx = -1.0d0/xr
      rho  = 10.d0**rl1x
c      ueng  = 10.d0**ul1x
      cpx  = drdtpx/rho/atgx*10.**(pl-tl)
      cv   = cpx*(1.0d0-(xt*atgx))
      return
      end 
c************************************************************************
c  compute thermodynamic quantities for the compsition just specified 
c  by ccomp.  use the additive volume technique (see istatco for ref).
c  see Fontaine, Graboske & Van Horn 1977, ApJS, 35, 293
c  MAW 2/1/89
c************************************************************************
      subroutine estatem
      implicit double precision(a-h,o-z)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1  ,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
      common/grad/ drdtp(400),cp(400),ttg(400),drdtpx,cpx,pecx,ttgx,
     1  deptx,dpdr,dtdr,dmdr,dr,xr,xt,cv,drdptx
c.................................
c  initialize summation variables
c.................................
        rhotmp  = 0.0
c        otmp    = 0.0
        etatmp  = 0.0
        cptmp   = 0.0
        cpdatmp = 0.0
c        cpdrtmp = 0.0
        deltmp  = 0.0
        dpttmp  = 0.0
        dtptmp  = 0.0
c.......................................................................
c  call estate for each composition i for which x(i) ne 0.
c.......................................................................
        do 100 je = 1,3
           if ( x(je) .gt. 0. ) then
              xx = x(je)
              xje = xx
              call estate
              rhotmp = rhotmp + xje/ rho
c              otmp   = otmp + xje * 10.**(ol1x)
              etatmp = etatmp + xje * el1x
              cptmp  = cptmp + xje * cpx 
              cpdatmp = cpdatmp + xje * cpx * atgx 
c              cpdrtmp = cpdrtmp + xje * cpx * rtgx 
              dpttmp = dpttmp + xje / ( rho * drdptx )
              dtptmp = dtptmp + xje / ( rho * drdtpx )
           endif
100     continue
c...............................................
c  now some final polishing and we're finished
c...............................................
        rho    = 1/rhotmp
        rl1x   = dlog10(rho)
c        ol1x   = dlog10(otmp)
        el1x   = etatmp
        cpx    = cptmp
        atgx   = cpdatmp / cpx
c        rtgx   = cpdrtmp / cpx
        drdptx = 1. / (rho * dpttmp)
        drdtpx = 1. / (rho * dtptmp)
        xr     = -1./ drdptx
        xt     = -drdtpx / drdptx
        cv     = cpx * ( 1. - xt * atgx )
        return
        end
c======================================================================
c Compute Fermi-Dirac integral F1/2 to accuracy of 10^-4 using 
c rational function approximation of Anita (1993, ApJS, 84, 101).
c======================================================================
      function f12(x)       
      implicit double precision(a-h,o-z)
      dimension a1(8),b1(8),a2(11),b2(12)
c       
c low precision formulae
c      data an,m1,k1,m2,k2/0.5d0,2,3,2,2/
c      data a1/2.18168d+01,1.31693d+01,1.00000d+0/
c      data b1/2.46180d+1,2.35546d+1,4.76290d+0,1.34481d-1/
c      data a2/4.73011d-02,5.48433d-01,1.00000d+0/
c      data b2/7.09478d-02,7.37041d-01,3.82065d-01/
c high precision formulae
      data an,m1,k1,m2,k2/0.5d0,7,7,10,11/
      data a1/5.75834152995465d+6,1.30964880355883d+7,
     1        1.07608632249013d+7,3.93536421893014d+6,
     2        6.42493233715640d+5,4.16031909245777d+4,
     3        7.77238678539648d+2,1.00000000000000d+0/
      data b1/6.49759261942269d+6,1.70750501625775d+7,
     1        1.69288134856160d+7,7.95192647756086d+6,
     2        1.83167424554505d+6,1.95155948326832d+5,
     3        8.17922106644547d+3,9.02129136642157d+1/
      data a2/4.85378381173415d-14,1.64429113030738d-11,
     1        3.76794942277806d-09,4.69233883900644d-07,
     2        3.40679845803144d-05,1.32212995937796d-03,
     3        2.60768398973913d-02,2.48653216266227d-01,
     4        1.08037861921488d+00,1.91247528779676d+00,
     5        1.00000000000000d+00/
      data b2/7.28067571760518d-14,2.45745452167585d-11,
     1        5.62152894375277d-09,6.96888634549649d-07,
     2        5.02360015186394d-05,1.92040136756592d-03,
     3        3.66887808002874d-02,3.24095226486468d-01,
     4        1.16434871200131d+00,1.34981244060549d+00,
     5        2.01311836975930d-01,-2.14562434782759d-02/
c............................................
c small eta rational function       
c............................................
      if(x .lt. 2.d0)then
        xx = dexp(x)
        rn = xx + a1(m1)
        do 100 i=m1-1,1,-1
          rn = rn*xx + a1(i)
100     continue
        den = b1(k1+1)
        do 200 i=k1,1,-1
          den = den*xx + b1(i)
200     continue
        f12 = xx * rn / den
       else
c............................................
c large eta rational function       
c............................................
        xx = 1.d0 / x**2
        rn = xx + a2(m2)
        do 300 i=m2-1,1,-1
          rn = rn*xx + a2(i)
300     continue
        den = b2(k2+1)
        do 400 i=k2,1,-1
          den = den*xx + b2(i)
400     continue
        f12 = ((x**(an+1.d0)) * rn) / den
      endif

      return      
      end 
c F -1/2 values
c      data a1/1.71446374704454d+7,3.88148302324068d+7,
c             3.16743385304962d+7,1.14587609192151d+7,
c             1.83696370756153d+6,1.14980998186874d+5,
c             1.98276889924768d+3,1.00000000000000d+0/
c      data b1/9.67282587552899d+6,2.87386436731785d+7,
c              3.26070130734158d+7,1.77657027846367d+7,
c              4.81648022267831d+6,6.13709569333207d+5,
c              3.13595854332114d+4,4.35061725080755d+2/
c      data a2/-4.46620341924942d-15,-1.58654991146236d-12,
c              -4.44467627042232d-10,-6.84738791621745d-08,
c              -6.64932238528105d-06,-3.69976170193942d-04,
c              -1.12295393687006d-02,-1.60926102124442d-01,
c              -8.52408612877447d-01,-7.45519953763928d-01,
c              +2.98435207466372d+00,+1.00000000000000d+00/
c********************************************************************
c  func1 (for diffusion exponents) Top of H/He transition zone.
c********************************************************************
        function func1(x)
        implicit double precision(a-h,o-z)
        common/alpha/alph(4),qmix1,qmix2,zone(2,2)
 
        x5 = x**alph(1)
        result = ( 2. - x5 ) / ( 2. + 3. * x5 )
        func1 = result
 
        return
        end
c*********************************************************************
c  func2 (for diffusion exponents) Bottom of H/He transition zone.
c*********************************************************************
        function func2(x)
        implicit double precision(a-h,o-z)
        common/alpha/alph(4),qmix1,qmix2,zone(2,2)
 
        xx = x**alph(2)
        result =  xx / ( 8. - 3. * xx ) 
        func2 = result
 
        return
        end
c*********************************************************************
c  func3 (for diffusion exponents) Top of He/C transition zone.
c*********************************************************************
        function func3(x)
        implicit double precision(a-h,o-z)
        common/alpha/alph(4),qmix1,qmix2,zone(2,2)
 
        x2 = x**alph(3)
        result = ( 2. - x2 ) / ( 2. + 2. * x2 )
        func3 = result
 
        return
        end
c*********************************************************************
c  func4 (for diffusion exponents) Bottom of He/C transition zone.
c*********************************************************************
        function func4(x)
        implicit double precision(a-h,o-z)
        common/alpha/alph(4),qmix1,qmix2,zone(2,2)
 
        xx = x**alph(4)
        result = xx / ( 6. - 2. * xx )
        func4 = result
 
        return
        end
c*******************************************************************************
c     solution of the cubic equation characteristic of the mixing length
c     theory  (u is essentially the ratio of radiative to convective
c     conductivities)  Bohm-Vitense version  (ML1)
c*******************************************************************************
      subroutine gradt
      implicit double precision(a-h,o-z)
c      common/je/je
c      common/comp/amhyhe,amheca,x(4)
c      common/sl/sl(11)
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1 ,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
      common/grad/ drdtp(400),cp(400),ttg(400),drdtpx,cpx,pecx,ttgx,
     1 deptx,dpdr,dtdr,dmdr,dr,xr,xt,cv,drdptx
c..................................................................
      u=pecx
      if ( u .lt. 1.e-10 ) then
         ttgx=atgx
         return
      elseif ( u .gt. 1.e+10 ) then
         ttgx=rtgx
         return
      endif
c..................................................................
      q=368./729.*u**2
      r=4672./19683.*u**3+4./9.*u*(rtgx-atgx)
      s1=(r+(q**3+r**2)**0.5)**0.3333333
      s2=-q/s1
c..................................................................
      if ( u .le. 1. ) then
         super=(s1+s2+19./27.*u)**2-u**2
         if ( super .le. 0. ) super=0.
         ttgx=atgx+super
         return
      else
         super=9.*(s1+s2-8./27.*u)**3/u/8.
         if ( super .le. 0. ) super=0.
         ttgx=rtgx-super
         return
      endif
      end 
c*******************************************************************************
c     solution of the cubic equation characteristic of the mixing length
c     theory  (u is essentially the ratio of radiative to convective
c     conductivities)  Bohm & Cassinelli version (ML2 or ML3)
c*******************************************************************************
      subroutine gradt2
      implicit double precision(a-h,o-z)
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1 ,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
      common/grad/ drdtp(400),cp(400),ttg(400),drdtpx,cpx,pecx,ttgx,
     1 deptx,dpdr,dtdr,dmdr,dr,xr,xt,cv,drdptx
c..................................................................
      u=pecx
      if ( u .lt. 1.e-10 ) then
         ttgx=atgx
         return
      elseif ( u .gt. 1.e+10 ) then
         ttgx=rtgx
         return
      endif
c..................................................................
c ml2 (and ml3) versions use different constants in q and r.
c..................................................................
      q=17./81.*u**2
      r=26./729.*u**3+1./6.*u*(rtgx-atgx)
      s1=(r+(q**3+r**2)**0.5)**0.3333333
      s2=-q/s1
c..................................................................
      if ( u .le. 1. ) then
         super=(s1+s2+8./9.*u)**2-u**2
         if ( super .le. 0. ) super=0.
         ttgx=atgx+super
         return
      else
         super=3.*(s1+s2-1./9.*u)**3/u
         if ( super .le. 0. ) super=0.
         ttgx=rtgx-super
         return
      endif
      end 
c*******************************************************************************
c     gray atmosphere stratification,   the pressure(log)  is patm, the
c     temperature(log) is tatm and the optical depth(log) is tau
c*******************************************************************************
      subroutine gray
      implicit double precision(a-h,o-z)
      logical writeit
      common/vca/modnr
      common/dvca/sgtmp
      parameter (nd=38)
c      common/tenv/rhok(3,60,35),eta(3,60,35),pp(3,60,35),datg(3,60,35),
c     1 od(3,18,35),tk(3,60),uu(3,60,35),xtt(3,60,35),xrt(3,60,35)
      common/tenv/rhok(3,60,nd),eta(3,60,nd),pp(3,60,nd),datg(3,60,nd),
     1 od(3,18,nd),tk(3,60),uu(3,60,nd),xtt(3,60,nd),xrt(3,60,nd)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      common/slr/sl(11)
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1 ,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
      common/d/ ml(3,60),nmax,kind,lmax,nsuff,nlim,nlim1,j,iter,lmam, 
     1 nlum,ko,nhp
      common/writeit/writeit
      common/oldval/bold,teold 
      common/thresh/thresh
c....................................................................
c format statements 
c....................................................................
  139 format(/'-------------------------------------------------------')
  140 format(/'Model ',i3,'  gray atmosphere structure (wdxd)') 
  142 format(/'       composition (h he c): ',3(1x,f6.3))
  150 format(/8h m/msun=,f6.3,2x,8h r/rsun=,1pe10.3,2x,6h grav=,e11.4,
     1 2x,6h teff=,e11.4,2x,8h l/lsun=,e10.3/)
  141 format(7x,4hlogt,10x,6hlogrho,10x,6hlogtau,9x,4hlogp,12x,3hatg, 
     1 12x,3hrtg,9x,8hlogkappa,9x,3heta)
  200 format(8(3x,e12.5))
  101 format(/26h still radiative when tau=,f8.3) 
  102 format(/27h convection sets in at tau=,f6.3)
  201 format(' new envelope.  gray atm still radiative at tau =',f8.3,
     1   '  teff=',f7.0)
  202 format(' new envelope.  gray atm convective at tau =',f6.3,
     1   '  teff=',f7.0)
c....................................................................
      if ( writeit ) then
         write(26,139)
         write(17,140) modnr
         write(26,140) modnr
         write(26,142) x(1), x(2), x(3)
         write(17,150) sm,rs,gs,te,sl(j) 
         write(26,150) sm,rs,gs,te,sl(j) 
         write(26,141)
      endif
c....................................................................
c     interpolate linearly in tk-od table to find the pressure
c     corresponding to initial tau and tatm
c....................................................................
      tau=-4.
      tatm=dlog10(te*(0.75*0.5773)**0.25)
      patm = dlog10( gs ) + tau + 1.
98    continue
      patm1 = patm
c....................................................................
c  integrate equation of hydrostatic equilibrium using log pressure
c  as independant variable,  quit when convection sets in or when tau=1
c  if radiative equilibrium
c....................................................................
c  integration loops back to statement 34 after incrementing pressure 
c....................................................................
      xmass=sm
34    tl=tatm
      pl=patm
c trying out stop at tau = 0.1 instead of tau = 10.0
      if ( tau .ge. -1. ) then 
         tauz=10.**tau
         if ( writeit ) write(26,101) tauz
         write(17,201) tauz, te
         write(20,201) tauz, te
         write(26,201) tauz, te
         print 201,tauz, te
         ps=patm
         ts=tatm
         return
      endif
c....................................................................
c  check that our (p,t) point is on the eeos table
c    (inserted 6/13/88)
c  skip over check if teff > thresh (30,000)
c....................................................................
      if (te .gt. thresh) goto 987
      itchk = (tl - tk(je,1)) / (tk(je,2) - tk(je,1)) + 1
      if ( tl .lt. tk(je,1) .or. tl .gt. tk(je,lmax) ) then
         write(26,*) 'subroutine gray:'
         write(26,321) tl, tk(je,1), tk(je,lmax), je
         write(20,*) 'subroutine gray:' 
         write(20,321) tl, tk(je,1), tk(je,lmax), je
321      format(' off eeos.  t:',f5.2,' tk(je,1), 
     1       tk(je,lmax): ',2f6.2,'    je: ',i1)
         if ( tl .lt. tk(je,1) ) itchk = 1 
         if ( tl .gt. tk(je,lmax) ) itchk = lmax
      endif
 
      mltmp = ml(je,itchk)
      if ( pl .lt. pp(je,itchk,1) .or.
     1     pl .gt. pp(je,itchk,mltmp ) ) then
         write(26,*) 'subroutine gray:'
         write(26,432) pl, tl, je, itchk, pp(je,itchk,1),
     1                 je, itchk, mltmp, pp(je,itchk,mltmp) 
         write(20,*) 'subroutine gray:' 
         write(20,432) pl, tl, je, itchk, pp(je,itchk,1),
     1                 je, itchk, mltmp, pp(je,itchk,mltmp) 
432      format(' off eeos. '/'  p:',f7.3,'  t:',f7.3,/
     1                        '  pp(',i1,',',i2.2,',1): ',f7.3,/
     2                        '  pp(',i1,',',i2.2,',',i2.2,'):',f7.3) 
      endif
c....................................................................
c  end eeos check
c....................................................................
987   continue
c....................................................................
c  ccomp called before gray, no need to call here 
c....................................................................
      call estatem
      ol1x = opac3(rl1x,tl)
      rtgx = 7.732749e+9*10.**(ol1x+pl-4.*tl)*sl(j)/xmass
 
      if ( writeit ) write(26,200) tl,rl1x,tau,pl,atgx,rtgx,ol1x,el1x
      diff=rtgx-atgx
      if ( atgx .le. 0.0 ) diff = rtgx - 0.40
c....................................................................
c return if convective
c....................................................................
      if(diff .gt. 0.) then
         if ( patm .eq. patm1 ) then
            write(17,543)
            write(20,543)
            print 543, patm-.3
543         format(//'first shell of gray atm convective'/, 
     1               'backing off patm trying again: ',f8.3)
	    patm = patm - .3
            goto 98
         endif
         taux=10.**t1+diff1*(10.**tau-10.**t1)/(diff1-diff) 
         ps=patm
         ts=tatm
         if ( writeit ) write(26,102) taux
         write(6,202) taux, te
         write(17,202) taux, te
         write(20,202) taux, te
         return
      endif
c....................................................................
c otherwise continue with the atm integration
c....................................................................
      diff1=diff
      t1=tau
      dtaudp=(2.30258*10.**(pl+ol1x))/gs
      tau1=10.**tau+0.05*dtaudp
      tau=dlog10(tau1)
      qtau=0.7104-0.1331*dexp(-3.4488d0*tau1)
      tatm=dlog10(te*((tau1+qtau)*0.75)**0.25)
      patm=patm+0.05
      go to 34
c....................................................................
c  goto 34 ==> loop back to top and continue integration
c....................................................................
      end 
c**********************************************************************
c Don Lamb's fit to the Hubbard-Lampe carbon conductive opacities
c**********************************************************************
      function hlcnc(d,t)
      implicit double precision(a-h,o-z)
      dimension a(25),w(25)
      data a/-5.18321e0,2.88323e-2,-1.68700e0,-3.80388e-1,-1.42808e0, 
     1       1.42606e0,-6.77026e-2,2.99633e-1,3.02198e-1,-1.94700e-1, 
     2       -2.72927e-3,4.24382e-2,-9.77295e-2,-1.50302e-2,8.16693e-3,
     3       4.80447e-4,1.04512e-3,-8.95778e-3,1.27411e-2,-3.37754e-4,
     4       4.17955e-5,-7.52181e-5,-1.38498e-4,5.92069e-4,-5.46719e-4/
c....................................................................
      w(1)=1.
      m=0 
      n=1 
      do 2 jmax=1,4 
        n=n+1
        do 1 j=1,jmax 
          m=m+1
          w(n)=d*w(m)
          n=n+1
1       continue
        w(n)=t*w(m)
2     continue
      do 3 i=16,25
        w(i)=d*w(i-5) 
3     continue
      hlcnc=0
      do 4 i=1,25
        hlcnc=hlcnc+a(i)*w(i) 
4     continue
      return
      end 
c**********************************************************************
c Gilles Fontaine's fit to the Hubbard-Lampe Hydrogen conductive opacities
c**********************************************************************
      function hlcnh(d,t)
      implicit double precision(a-h,o-z)
      dimension a(5,5),temp(5), rho(5)
      data a/12.40928,     -2.545420,   -3.019045,    6.746942e-2,
     1    4.686838e-2,-15.29732,      .8154491,   2.387849, 
     2   -6.939705e-2, -4.129124e-2, 5.162517,   -3.839103e-1,
     3   -6.899530e-1,  2.568522e-2, 1.281765e-2, -.6347533,
     4    7.203151e-2,  8.405360e-2,-3.965926e-3,-1.652045e-3,
     5    2.699066e-2, -4.243979e-3,-3.648879e-3, 
     6    2.127303e-4,  7.509063e-5/
c....................................................................
      sum = 0.
      temp(1) = 1.
      rho(1)  = 1.
      do 10 i = 2,5 
        temp(i) = temp(i-1) * t
        rho(i)  = rho(i-1) * d
10    continue
      do 2 i=1,5
        do 1 j=1,5
           sum = sum + a(i,j)*temp(j)*rho(i)
1       continue
2     continue
      hlcnh = sum
      return
      end 
c************************************************************************
c Gilles Fontaine's fit to the Hubbard-Lampe Helium conductive opacities
c************************************************************************
      function hlcnhe(d,t)
      implicit double precision(a-h,o-z)
      dimension a(5,5),temp(5), rho(5)
      data a /9.246777e+0, -2.097036,   -1.267003,  5.490678e-2,
     1    2.463979e-3, -1.265387e+1, 1.716563e-1,   1.088986e+0,
     2   -4.576479e-2, -6.966255e-3, 4.539412e+0, -1.427574e-1,
     3     -3.499339e-1,  1.589662e-2, 3.421386e-3, -5.842722e-1,
     4    3.930197e-2,  4.702301e-2,-2.529960e-3, -5.814372e-4,
     5    2.617848e-2, -2.791279e-3,-2.238884e-3, 
     6    1.442103e-4,  3.234929e-5/
c....................................................................
      sum = 0.
      temp(1) = 1.
      rho(1)  = 1.
      do 10 i = 2,5 
        temp(i) = temp(i-1) * t
        rho(i)  = rho(i-1) * d
10    continue
      do 2 i=1,5
        do 1 j=1,5
           sum = sum + a(i,j)*temp(j)*rho(i)
1       continue
2     continue
      hlcnhe = sum 
      return
      end 
c**************************************************************************
c homology transformation of wdec model 
c..........................................................................
c first group of relations are those that came with the
c code.  The others are empirically derived from
c pre-wd sequences of .601 and .7795 mo 
c**************************************************************************
      subroutine homo(xh,yh,yyh,nshell) 
      implicit double precision(a-h,o-z)
      common/shells/ sa(400),ra(400),ba(400),pa(400),ta(400),
     1 ea(400),xca(400),fca(400),s(400),r(400),b(400),
     2 p(400),t(400),e(400),xc(400),sk(400),
     3 rk(400),bk(400),pk(400),tk(400)
      common/contrl/ds,g,sm,wc,it,nite,ja,jb,j,k,l
      common/thermo/u2,up2,ut2,e2,ep2,et2,psi2,pg,o2,op2,ot2,fp,ft,
     1 en2(5),fn,fcc,ce,ci,cif,w,nu
      common/temp/s1,r1,s2,r2,b2,p2,t2,ea2,xc2,xo2,fca2,f2,q2,w2,c
c....................................................................
      stt=2.*xh+5.*yh-yyh
      str=-xh-4.*yh+yyh
      stp=6.*xh+16.*yh-4.*yyh 
      stb=10.**(3.*xh+4.*yh)
c....................................................................
      open(18,file='homo.dat',status='unknown') 
      do 1 j=1,nshell
         t(j)=t(j)+stt
         r(j)=r(j)+str
         p(j)=p(j)+stp
         b(j)=b(j)*stb
         xc2 = xc(j)
         xo2 = 1-xc2
         if ( xc2 .gt. .000001 .and. xc2 .lt. .999999) then 
            call istatco(p(j),t(j),0,.true.)
         elseif ( xc2 .ge. .999999) then
            call istat1(p(j),t(j),0,12,.true.)
         elseif ( xc2 .le. .000001) then
            call istat1(p(j),t(j),0,16,.true.)
         endif
         e(j)=e2
c         write(18,11)s(j),r(j),b(j),p(j),t(j),e(j),xc2,xo2,u2
         write(18,11)s(j),r(j),b(j),p(j),t(j),e(j),xc2
1     continue
c11    format(e15.8,f9.6,1pe11.4,0pf10.6,f9.6,e14.7,f8.6,f8.6,e16.6)
11    format(e15.8,f9.6,1pe11.4,0pf10.6,f9.6,e14.7,f8.6)
      close(18)
      write(*,*) 'homology transform stored in homo.dat'
      stop
      return
      end 
c**********************************************************************
c Given an array XX of length N, and given a value X, returns a 
c value jlo such that x is between xx(jlo) and xx(jlo+1).  XX must
c be monotonic, either increasing or decreasing.
c**********************************************************************
      subroutine hunt(xx,n,x,jlo)
      implicit double precision(a-h,o-z)
      dimension xx(n)
      logical ascnd
c Is table ascending (.TRUE.) or decending (.FALSE.)?
      ascnd = xx(n) .gt. xx(1)
c Input guess not useful, go straight to bisection
      if ( jlo .le. 0 .or. jlo .gt. n) then
          jlo = 0
          jhi = n + 1
          goto 3
      endif
c set hunting increment
      inc = 1
c hunt up.
      if ( x .ge. xx(jlo) .eqv. ascnd) then
1         jhi = jlo + inc
          if (jhi .gt. n) then
              jhi = n+1
          else if ( x .ge. xx(jhi) .eqv. ascnd) then
              jlo = jhi
              inc = inc + inc
              goto 1
          endif
      else
c hunt down
          jhi = jlo
2         jlo = jhi - inc
          if ( jlo .lt. 1 ) then
              jlo = 0
          else if ( x.lt.xx(jlo) .eqv.ascnd) then
              jhi = jlo
              inc= inc+inc
              goto 2
          endif
      endif
3     if ( jhi-jlo .eq. 1) return
      jm = (jhi+jlo) / 2
      if ( x .gt. xx(jm) .eqv. ascnd) then
          jlo=jm
      else
          jhi = jm
      endif
      goto 3
      end
c**********************************************************************
c interpolate model from old to new meshes
c**********************************************************************
      subroutine interp
      implicit double precision(a-h,o-z)
      common/shells/ sa(400),ra(400),ba(400),pa(400),ta(400),
     1 ea(400),xca(400),fca(400),s(400),r(400),b(400),
     2 p(400),t(400),e(400),xc(400),sk(400),
     3 rk(400),bk(400),pk(400),tk(400)
      common/cal/ut1,ot1,et1,op1,o1,ep1,qnp1,qnt1,p1,q1,f1,t1,w1,b1,e1,
     1 qe1,ea1,qn1,up1,qe2,qn2,qnp2,qnt2
      common/thermo/u2,up2,ut2,e2,ep2,et2,psi2,pg,o2,op2,ot2,fp,ft,
     1 en2(5),fn,fcc,ce,ci,cif,w,nu
      common/temp/s1,r1,s2,r2,b2,p2,t2,ea2,xc2,xo2,fca2,f2,q2,w2,c
      common/terp/stpms,rat1
      common/contrl/ds,g,sm,wc,it,nite,ja,jb,j,k,l
      common/corats/ams(10),delmass(10),corat(10),dcorat(10),
     1              ncore, irdold, alph 
c.......................................................................
      if (nite.ge.20) niteo = 30
      s1=s2
      r1=r2
      b1=b2
      p1=p2
      t1=t2
      up1=up2
      ut1=ut2
      o1=o2
      ot1=ot2
      op1=op2
      e1=e2
      ep1=ep2
      et1=et2
      ea1=ea2
      q1=q2
      f1=f2
      w1=w2
      qe1=qe2
      qn1=qn2
      qnp1=qnp2
      qnt1=qnt2
c.......................................................................
c now that we've saved all the variables for the previous shell,
c let's go about stepping to the next shell.
c if irdold = 1 and we're near a mass where corat was defined on input, then
c reduce ds.
c.......................................................................
      fac = 1.
c      write(17,*) 'factors',c,fac,q1,f1
60    ds=c/dmax1(fac,q1,f1)
c.......................................................................
c 0.008 was the value I used before 2/28/91
c Matt uses .005 here. Gets lots more shells.
c.......................................................................
      dsx=dlog10(10.**s1+.008)-s1
c.......................................................................
c Matt's new value as of 2/28/91 
c want -1.0 instead of -4.0 to  prevent undesirable jumps in core
c.......................................................................
      if(s1.gt.-1.0.and.s1.lt.-.02)then
c      write(17,*) 'first if',j,nite,s1,dsx
        ds=dsx
      endif
      if( nite .le. 20 .and. nite .ge. 10 
     1 .or.(nite.eq.0.and.niteo.le.20.and. niteo .ge. 10)) then
c      write(17,*) 'first if',j,nite,s1,dsx
	 ds = .90*ds
	 niteo = nite
      elseif ( nite .lt. 10 .and. nite .ge. 2 
     1 .or.(nite.eq.0.and.niteo.lt.10.and. niteo .ge. 2)) then
c      write(17,*) 'first if',j,nite,s1,dsx
	 ds = .80*ds
	 niteo = nite
      endif
      s2=s1+ds
c.......................................................................
c make sure that corat boundaries are mesh points 
c.......................................................................
      if ( irdold .eq. 1 ) then
         do 100 i = 2,ncore-1 
            qi = dlog10( ams(i) )
            if ( s1 .lt. qi  .and. s2 .ge. qi ) then
               s2 = qi
               ds = qi - s1
               goto 200
            endif
100      continue
      endif
 
200   if ( s2 .ge. stpms ) then
         it=1
         s2=stpms
         ds=stpms-s1
      endif
 
415   if ( j .le. jb ) then
         if ( s2 .le. s(j) ) then
            sfrac=(s2-s(j))/(s(j)-s(j-1))
            p2=p(j)+(p(j)-p(j-1))*sfrac 
            b2=b(j)+(b(j)-b(j-1))*sfrac 
            r2=r(j)+(r(j)-r(j-1))*sfrac 
            t2=t(j)+(t(j)-t(j-1))*sfrac 
            return
         endif
         j=j+1
         go to 415
      endif
 
      cs=dlog10(1.+ds/s1)
      p2=p1+cs
      t2=t1+w1*cs
      r2=r1+ds*q1
      b2=b1
 
      return
      end 
c*******************************************************************************
c     Runge-Kutta-Gill method of integration.  See Ralston & Wiff
c     Math Methods for Digital Computers  p.110  (Wiley,1960)
c*******************************************************************************
      subroutine intg
      implicit double precision(a-h,o-z)
      double precision k(4,5),m 
      dimension y(4,5),q(4,5),d(4,5)
      dimension ax(5),bx(5),cx(5)
      parameter (nd=38)
c      common/tenv/rhok(3,60,35),eta(3,60,35),pp(3,60,35),datg(3,60,35),
c     1 od(3,18,35),tk(3,60),uu(3,60,35),xtt(3,60,35),xrt(3,60,35)
      common/tenv/rhok(3,60,nd),eta(3,60,nd),pp(3,60,nd),datg(3,60,nd),
     1 od(3,18,nd),tk(3,60),uu(3,60,nd),xtt(3,60,nd),xrt(3,60,nd)
      common/shells/r(800),m(800),p(800),t(800),el1(800),ol1(800),
     1 rl1(800),atg(800),rtg(800),phs(800)
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1 ,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
      common/d/ ml(3,60),nmax,kind,lmax,nsuff,nlim,nlim1,j,iter,lmam, 
     1 nlum,ko,nhp
      common/slr/sl(11)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      common/grad/ drdtp(400),cp(400),ttg(400),drdtpx,cpx,pecx,ttgx,
     1 deptx,dpdr,dtdr,dmdr,dr,xr,xt,cv,drdptx
      common/thresh/thresh
      common/mixl/aml,mls
c.......................................................................
      data ax/0.e0,5.e-1,2.928932190e-1,1.7071067810e0,1.6666666667e-1/ 
      data bx/0.e0,2.e0,1.e0,1.e0,2.e0/
      data cx/0.e0,5.e-1,2.928932190e-1,1.7071067810e0,5.e-1/
c.......................................................................
      dept=deptx
      if ( ko .le. 1 ) then
         q(1,1)=0.
         q(2,1)=0.
         q(3,1)=0.
         q(4,1)=0.
      else
         q(1,1)=q(1,5)
         q(2,1)=q(2,5)
         q(3,1)=q(3,5)
         q(4,1)=q(4,5)
      endif
 
      h=dr
      n=ko
      y(1,1)=r(n)
      y(2,1)=p(n)
      y(3,1)=t(n)
      y(4,1)=m(n)
      d(1,2)=1.
      d(2,2) =0.4342945*dpdr*10.**(-y(2,1))
      d(3,2)=0.4342945*dtdr*10.**(-y(3,1))
      d(4,2)=0.4342945*dmdr/(1.989e+33*10.**y(4,1))
 
      do 1 jx=2,4
         mx=jx-1
         kx=jx+1
         do 2 ix=1,4
            k(ix,jx)=h*d(ix,jx)
            y(ix,jx)=y(ix,mx)+ax(jx)*(k(ix,jx)-bx(jx)*q(ix,mx))
            q(ix,jx)=q(ix,mx)+3.*ax(jx)*(k(ix,jx)-bx(jx)*q(ix,mx))
     1               -cx(jx)*k(ix,jx)
2        continue
         pl=y(2,jx) 
         tl=y(3,jx) 
         xmass=10.**y(4,jx)
c.......................................................................
         call ccomp 
         call estatem
         ol1x = opac3(rl1x,tl)
         rtgx = 7.732749e+9*10.**(ol1x+pl-4.*tl)*sl(j)/xmass
         d(1,kx)=1. 
         d(2,kx)=-0.4342945*27398.*10.**(y(4,jx)-y(2,jx))*rho*(6.96e+10/
     1            y(1,jx))**2 
c.......................................................................
c     the mixing length is evaluated locally (shell n, halfway, shell n+1)
c.......................................................................
         if ( rtgx .ge. atgx ) then
            phsx=dabs(0.4342945/d(2,kx)) 
            dept=dept+dabs(y(1,mx)-y(1,jx))
            if ( phsx .gt. dept ) phsx = dept
            pecx = 1.601743e+07 * cpx * 
     1             10.**(ol1x+2.5*rl1x+y(4,jx)-3.*y(3,jx)
     2             -0.5*y(2,jx)) * dsqrt(dabs(drdtpx))
     3             *phsx**2*(6.96e+10/y(1,jx))**2 
            pecx=9./8./pecx
            pecx=pecx/aml/aml 
c.......................................................................
c pick between ml1 and ml3 versions of mixing-length theory 
c.......................................................................
            if(mls.eq.0)then
c ml1
               call gradt
             else
c ml3
               pecx=(8./9.)*3./4.242440687*pecx
               call gradt2
             endif
         else
            ttgx=rtgx
         endif
         d(3,kx)=ttgx*d(2,kx) 
         d(4,kx)=0.4342945*4.*pi*rho*y(1,jx)**2/(1.989e+33*10.**y(4,jx))
1     continue
 
      do 7 ix=1,4
         k(ix,5)=h*d(ix,5)
         y(ix,5)=y(ix,4)+ax(5)*(k(ix,5)-bx(5)*q(ix,4))
         q(ix,5)=q(ix,4)+3.*ax(5)*(k(ix,5)-bx(5)*q(ix,4))-cx(5)*k(ix,5)
7     continue
 
      r(n+1)=y(1,5) 
      p(n+1)=y(2,5) 
      t(n+1)=y(3,5) 
      m(n+1)=y(4,5) 
      return
      end 
c***********************************************************************
c compute the thermodynamic variables for a specific P, T
c point.  This version searches either the C or O eos table, depending
c on the composition and returns the thermodynamic properties.
c***********************************************************************
      subroutine istat1(p2,t2,iw,iswch,converg)
      implicit double precision(a-h,o-z)
      common/temp/s1,r1,s2,r2,b2,px2,tx2,e2,xc2,xo2,fca2,fx2,q2,w2,c
      common/tablec/tablec(6,39,87)
      common/tableo/tableo(6,36,87)
      common/sizec/tminc,tmaxc,deltc,pminc,pmaxc,delpc,pmc(50),ipc(50),
     1   iphasec(50),ioverc,itc
      common/sizeo/tmino,tmaxo,delto,pmino,pmaxo,delpo,pmo(50),ipo(50),
     1   iphaseo(50),iovero,ito
      common/thermo/d2,dp,dt,e,ep,et,psi,pg,o2,op,ot,fp,ft, 
     1   en2(5),fn2,fcc2,ce,ci,cif,w,nu 
      common/neut/en(5),fn
      common/iii/ iii
      common/ixswch/ixswch
      common/je/je
      logical converg
      dimension porder(4),torder(4),iorder(4),jorder(4),
     1          xorder(4), yorder(4)
      dimension thermo(24)
      equivalence (d2,thermo(1))
      data delta/2.e-3/
      data fnat/4.342945e-1/
c....................................................
c initialize some variables
c....................................................
      xc = xc2
      xo = xo2
      je=3
      iii=0
      p=p2
      t=t2
      if ( iswch .eq. 12 ) then
         ci=1./(xc/12.+xo/16.) 
      else if ( iswch .eq. 16 ) then
         ci=1./(xo/16.+(1-xo)/21.45)
      else
         write(*,*) 'istat1: iswch not set.  exiting.'
         stop
      endif
c.......................................................................
c find eos box and interpolate.
c We search carbon if ISWCH is 12 and oxygen if ISWCH is 16.
c.......................................................................
      if( iswch .eq. 12 ) then
        call select(t,tminc,deltc,torder,iorder,1,itc,converg)
        kmax=6
        if( iw .ge. 1 )  kmax=1
          do 5 k=1,kmax
            do 4 i=1,4
              indexc=iorder(i)
              pfrzc = phasec(t)
              if( p .lt. pfrzc ) then 
                jminc=iphasec(indexc)+1
                jmaxc=ipc(indexc)
                pstartc=pmaxc-ioverc*delpc
               else 
                jminc=1
                jmaxc=iphasec(indexc) 
                pstartc=pmaxc
              endif
 
              call select(p,pstartc,delpc,porder,jorder,jminc,jmaxc, 
     1          converg)
 
              do 3 j=1,4
                xorder(j)=tablec(k,indexc,jorder(j))
3             continue
              call dali(p,porder,xorder,yorder(i))
4           continue
            l=k
            if(k.gt.4) l=k+1
            call dali(t,torder,yorder,thermo(l))
5        continue
c....................................................
c  or if oxygen, then come here
c....................................................
      else if ( iswch .eq. 16 ) then
         call select(t,tmino,delto,torder,iorder,1,ito,converg)
         kmax=6
         if ( iw .ge. 1 )  kmax=1
         do 35 k=1,kmax
            do 34 i=1,4
               indexo=iorder(i)
               pfrzo = phaseo(t)
               pfrzc = pfrzo
               if ( p .lt. pfrzo ) then 
                  jmino=iphaseo(indexo)+1
                  jmaxo=ipo(indexo)
                  pstarto=pmaxo-iovero*delpo
               else 
                  jmino=1
                  jmaxo=iphaseo(indexo) 
                  pstarto=pmaxo
               endif
 
               call select(p,pstarto,delpo,porder,jorder,jmino,jmaxo, 
     1          converg)
 
               do 33 j=1,4
                  xorder(j)=tableo(k,indexo,jorder(j))
33             continue
               call dali(p,porder,xorder,yorder(i))
34          continue
            l=k
            if(k.gt.4) l=k+1
            call dali(t,torder,yorder,thermo(l))
35       continue
      endif
      d  = d2
c...................................................................
c Now that we've interpolated the main quantities, we need to
c calculate the opacity, carbon burning and neutrino luminosities
c and their derivitives wrt rho and t
c Note that the carbon burning luminosity is reduced by a factor
c of 5 arbitrarily (otherwise, runaway).
c...................................................................
      n  = -1
20    x1=49.4+d-2*t/3-36550*(1+7e-1*10.**(t-10))**(1./3)*10.**(-t/3)
 
      if ( x1 .le. -30. ) then
         fcc=0
      else
         xtmp = .999999
         fcc=4.368*xtmp*xtmp*10.**x1
         fcc=fcc/5. 
      endif
 
      if (iw .ge. 1) then
         fcc2=fcc
         return
      endif
 
      if (nu .le. 0) then
         fn=0
      elseif(nu .eq. 1)then
c call venerable BPS neutrino rates
         call neu(d,t)
      elseif(nu .eq. 2)then
call Munkata etal Plasmon, Photo, and Pair rates
         call neutrino(d,t)
      elseif(nu .eq. 3)then
call Itoh etal neutrino rates
         call gnutrino(d,t)
      endif
c...................................................................
      f=fn-fcc*w
      o=10.**opac(d,t)
      if(n) 14,15,16
c...................................................................
14    o2=o
      f2=f
      fn2=fn
      fcc2=fcc
      do 17 i=1,5
17    en2(i)=en(i)
      e  = 10.**e
      et = e * et
      ep = (dt/fnat)*10.**(p-d-t)
      pg=dlog10(10.**p-10.**(4*t-14.59836))
      d=d2+delta
      n=0 
      go to 20
c...................................................................
15    od=(o-o2)/delta
      fd=(f-f2)/delta
      d=d2
      t=t2+delta
      n=1 
      go to 20
c...................................................................
16    ot=(o-o2)/delta
      ft=(f-f2)/delta
      op=od*dp
      ot=ot+od*dt
      fp=fd*dp
      ft=ft+fd*dt
      return
      end 
c***********************************************************************
c compute the thermodynamic variables for a specific P, T
c point.  This version searches both the c and o eos tables,
c and then interpolates to the desired composition using the
c additive volume technique
c***********************************************************************
      subroutine istatco(p2,t2,iw,converg)
      implicit double precision(a-h,o-z)
      common/temp/s1,r1,s2,r2,b2,px2,tx2,e2,xc2,xo2,fca2,fx2,q2,w2,c
      common/tablec/tablec(6,39,87)
      common/tableo/tableo(6,36,87)
      common/sizec/tminc,tmaxc,deltc,pminc,pmaxc,delpc,pmc(50),ipc(50),
     1   iphasec(50),ioverc,itc
      common/sizeo/tmino,tmaxo,delto,pmino,pmaxo,delpo,pmo(50),ipo(50),
     1   iphaseo(50),iovero,ito
      common/thermo/d2,dp,dt,e,ep,et,psi,pg,o2,op,ot,fp,ft, 
     1   en2(5),fn2,fcc2,ce,ci,cif,w,nu 
      common/thermoc/dc,dpc,dtc,ec,epc,etc,psic
      common/thermoo/do,dpo,dto,eo,epo,eto,psio
      common/neut/en(5),fn
      common/iii/ iii
      common/ixswch/ixswch
      common/je/je
      logical converg
      dimension porderc(4),torderc(4),iorderc(4),jorderc(4),
     1          xorderc(4), yorderc(4)
      dimension pordero(4),tordero(4),iordero(4),jordero(4),
     1          xordero(4), yordero(4)
      dimension thermoc(7), thermoo(7)
      equivalence (dc,thermoc(1))
      equivalence (do,thermoo(1))
      data delta/2e-3/
      data fnat/4.342945e-1/
c...................................................................
c initialize some variables
c...................................................................
      xc = xc2
      xo = xo2
      je=3
      iii=0
c old ion molecular weight
c      ciold=1/(xc2/12+(1-xc2)/cif)
      ci=1/(xc/12.+xo/16.)
      p=p2
      t=t2
c...................................................................
c find eos box and interpolate.
c note that we're doing both tables in parallel.
c...................................................................
      call select(t,tminc,deltc,torderc,iorderc,1,itc,converg)
      call select(t,tmino,delto,tordero,iordero,1,ito,converg)
 
      kmax=6
      if ( iw .ge. 1 )  kmax=1
 
      do 5 k=1,kmax 
 
         do 4 i=1,4 
 
            indexc=iorderc(i) 
            indexo=iordero(i) 
 
            if ( ixswch .eq. 1 ) then
               pfrzc = phasec(t)
               pfrzo = pfrzc
            elseif ( ixswch .eq. 2 ) then
               pfrzo = phaseo(t)
               pfrzc = pfrzo
            elseif ( ixswch .eq. 3 ) then
               pfrzc = xc * phasec(t) + xo * phaseo(t)
               pfrzo = pfrzc
            elseif ( ixswch .eq. 4 ) then
               pfrzc = phasec(t)
               pfrzo = phaseo(t)
            else
               write(17,*) 'ixswch not set. exiting.'
               write(20,*) 'ixswch not set. exiting.'
               print *, 'ixswch not set. exiting.'
               stop 
            endif
 
            if ( p .lt. pfrzc ) then
               jminc=iphasec(indexc)+1
               jmaxc=ipc(indexc)
               pstartc=pmaxc-ioverc*delpc
            else
               jminc=1
               jmaxc=iphasec(indexc)
               pstartc=pmaxc
            endif
 
            if ( p .lt. pfrzo ) then
               jmino=iphaseo(indexo)+1
               jmaxo=ipo(indexo)
               pstarto=pmaxo-iovero*delpo
            else
               jmino=1
               jmaxo=iphaseo(indexo)
               pstarto=pmaxo
            endif
 
            call select(p,pstartc,delpc,porderc,jorderc,jminc,jmaxc,
     1          converg)
            call select(p,pstarto,delpo,pordero,jordero,jmino,jmaxo,
     1          converg)
 
            do 3 j=1,4
               xorderc(j)=tablec(k,indexc,jorderc(j))
               xordero(j)=tableo(k,indexo,jordero(j))
3           continue
 
            call dali(p,porderc,xorderc,yorderc(i))
            call dali(p,pordero,xordero,yordero(i))
 
4        continue
 
         l=k
         if(k.gt.4) l=k+1
 
         call dali(t,torderc,yorderc,thermoc(l))
         call dali(t,tordero,yordero,thermoo(l))
 
5     continue
c...................................................................
c c/o interpolations finished.  at this time we've interpolated
c in the c and o tables to get d, dp, dt, e, et, and psi for each
c composition.
c use additive volume technique to
c interpolate the therm quantities for the mixture.
c see Fontaine, Graboske, & Van Horn 1977, ApJS, 35, 293
c see my log 4 for derivation of these interpolations.
c...................................................................
      d2 = -dlog10( xc/10.**dc + xo/10.**do )
      d  = d2
c...................................................................
c Now that we've interpolated the main quantities, we need to
c calculate the opacity, carbon burning and neutrino luminosities
c and their derivitives wrt rho and t
c Note that the carbon burning luminosity is reduced by a factor
c of 5 arbitrarily (otherwise, runaway).
c...................................................................
      n  = -1
20    x1=49.4+d-2*t/3-36550*(1+7e-1*10.**(t-10))**(1./3)*10.**(-t/3)
 
      if ( x1 .le. -30. ) then
         fcc=0
      else
         fcc=4.368*xc2*xc2*10.**x1
         fcc=fcc/5. 
      endif
 
      if (iw .ge. 1) then
         fcc2=fcc
         return
      endif
 
      if (nu .le. 0) then
         fn=0
      elseif(nu .eq. 1)then
c call venerable BPS neutrino rates
         call neu(d,t)
      elseif(nu .eq. 2)then
call Munkata etal Plasmon, Photo, and Pair rates
         call neutrino(d,t)
      elseif(nu .eq. 3)then
call Itoh etal neutrino rates
         call gnutrino(d,t)
      endif
 
      f=fn-fcc*w
      o=10.**opac(d,t)
      if(n) 14,15,16
c...................................................................
14    o2=o
      f2=f
      fn2=fn
      fcc2=fcc
      do 17 i=1,5
17    en2(i)=en(i)
      dt  = 1./( d * ( xc/(dc*dtc) + xo/(do*dto) ) )
      dp  = 1./( d * ( xc/(dc*dpc) + xo/(do*dpo) ) )
      e  = xc * 10.**ec + xo * 10.**eo
      et = xc * etc * 10.**ec + xo * eto * 10.**eo
      ep = 10.**(p-t)/fnat * (xc*dtc*10.**(-dc) + xo*dto*10.**(-do))
      psi = xc * psic + xo * psio
      pg=dlog10(10.**p-10.**(4*t-14.59836))
      d=d2+delta
      n=0 
      go to 20
c...................................................................
15    od=(o-o2)/delta
      fd=(f-f2)/delta
      d=d2
      t=t2+delta
      n=1 
      go to 20
c...................................................................
16    ot=(o-o2)/delta
      ft=(f-f2)/delta
      op=od*dp
      ot=ot+od*dt
      fp=fd*dp
      ft=ft+fd*dt
      return
      end 
c**************************************************************************
c compute modified Ledoux B term to include composition gradient
c effects in the Brunt-Vaisala frequency
c Pressure is the dependent variable throught and I sum partial pressures.
c In this version I change only the He abundance at the H/He interface.
c while I keep the sum of the abundances unity at the He/C interface.
c This duplicates the numerical differencing results, but the physical 
c justification eludes me at present.
c PAB Sept 10, 1991
c Changed the way the partial pressure additions are done (MHM, Jan 1999).
c These changes can be found by searching for "MHM" in comments.
c**************************************************************************
      subroutine ledoux
      implicit double precision(a-h,o-z)
c EOS data
      parameter (nd=38)
c      common/tenv/rhok(3,60,35),eta(3,60,35),pp(3,60,35),datg(3,60,35),
c     1 od(3,18,35),tk(3,60),uu(3,60,35),xtt(3,60,35),xrt(3,60,35)
      common/tenv/rhok(3,60,nd),eta(3,60,nd),pp(3,60,nd),datg(3,60,nd),
     1 od(3,18,nd),tk(3,60),uu(3,60,nd),xtt(3,60,nd),xrt(3,60,nd)
c composition info
      common/je/je
      common/comp/amhyhe,amheca,x(4)
c need for lmax
      common/d/ ml(3,60),nmax,kind,lmax,nsuff,nlim,nlim1,j,iter,lmam, 
     1 nlum,ko,nhp
c has log T=tl, log Rho=rho and log P=pl
c also has stellar mass=sm and current mass=xmass
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1  ,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
c diffusion profile info
      common/alpha/alph(4),qmix1,qmix2,zone(2,2)
c ledoux term value
      common/bldx/bled
c      real*8 rhot1(35),rhot2(35),rhot3(35),rhot4(35)
c      real*8 ptmp1(35),ptmp2(35),ptmp3(35),ptmp4(35)
c      real*8 y21(35),y22(35),y23(35),y24(35)
      real*8 rhot1(nd),rhot2(nd),rhot3(nd),rhot4(nd)
      real*8 ptmp1(nd),ptmp2(nd),ptmp3(nd),ptmp4(nd)
      real*8 y21(nd),y22(nd),y23(nd),y24(nd)
      real*8 temp(60)
c.................................
c  initialize summation variables
c.................................
      bled    = 0.0
c step size increments in composition
      del     = 0.01
c      write(17,*) 'Ledoux P,T,Rho',pl,tl,rho
c.............................
c This routine is complicated because we have to consider the
c H/He interface and He/C interfaces.
c We also have to make sure to avoid non-physical compositions
c such as X<0 or X>1.
c.............................
c H/He interface
      if(x(1).gt.0.0 .and. x(2).gt.0.0)then
        if(x(3) .gt. x(1))then
c Very thin He buffer layer. Call H/He or He/C transition zone
c depending on case involved.
c          write(*,*) 'jump',x(1),x(2),x(3)
          go to 45
        endif
c first, locate T point on grid for Helium
        do 992 i=1,lmax
          temp(i) = tk(2,i)
992     continue
        call hunt(temp,lmax,tl,ktlo)
        kthi = ktlo + 1
c set size of temporary arrays
        kmax1 = ml(2,ktlo)
        kmax2 = ml(2,kthi)
c fill up temporary 1-D arrays for rho and T
c He arrays
        do 10 k=1,kmax1
          rhot1(k) = rhok(2,ktlo,k)
          ptmp1(k) =   pp(2,ktlo,k)
10      continue
        do 20 k=1,kmax2
          rhot2(k) = rhok(2,kthi,k)
          ptmp2(k) =   pp(2,kthi,k)
20      continue
c first, locate T point on grid for Hydrogen
        do 993 i=1,lmax
          temp(i) = tk(1,i)
993     continue
        call hunt(temp,lmax,tl,ktlo)
        kthi = ktlo + 1
c set size of temporary arrays
        kmax3 = ml(1,ktlo)
        kmax4 = ml(1,kthi)
c H arrays
        do 30 k=1,kmax3
          rhot3(k) = rhok(1,ktlo,k)
          ptmp3(k) =   pp(1,ktlo,k)
30      continue
        do 40 k=1,kmax4
          rhot4(k) = rhok(1,kthi,k)
          ptmp4(k) =   pp(1,kthi,k)
40      continue
c set BC's so second der is zero there
        yp1 = 1.0e+40
        ypn = 1.0e+40
c call for spline coefficients
        call spline(rhot1,ptmp1,kmax1,yp1,ypn,y21)
        call spline(rhot2,ptmp2,kmax2,yp1,ypn,y22)
        call spline(rhot3,ptmp3,kmax3,yp1,ypn,y23)
        call spline(rhot4,ptmp4,kmax4,yp1,ypn,y24)
c now compute P for each isotherm at a given rho,comp point
        call splint(rhot1,ptmp1,y21,kmax1,rho,p11)
        call splint(rhot2,ptmp2,y22,kmax2,rho,p12)
        call splint(rhot3,ptmp3,y23,kmax3,rho,p13)
        call splint(rhot4,ptmp4,y24,kmax4,rho,p14)
c        write(17,*) 'He pressure L/H',p11,p12
c        write(17,*) 'H pressure L/H',p13,p14
c linear interpolation for P at the actual temp.
        frac=(tl-tk(je,ktlo))/(tk(je,kthi)-tk(je,ktlo))
        pone=p11+frac*(p12-p11)
        ptwo=p13+frac*(p14-p13)
        pone=10.**(pone)
        ptwo=10.**(ptwo)
c        write(17,*) 'frac, He, H',frac,pone,ptwo
c        write(*,*) 'frac, He, H',x(1),x(2),frac,p11,p12
c        write(*,*) 'frac, He, H',x(1),x(2),frac,p13,p14
c        write(*,*) 'kmax(1...4)',kmax1,kmax2,kmax3,kmax4
c.................................................................
c now to consider specific composition mixtures!
c CASE 1: x(1) or x(2) is less than del (use a 3 point formula)
c CASE 1.1: Nearly pure helium: Treat first point as pure He.
c           treat second point as H / He mixture 
c.................................................................
           if(x(1).lt.del)then
             diff = 1.0-x(2)
c changed to take care of diff=0.0 case (MHM Jan. 1999)
	     if (diff.lt.1.d-05) diff=1.d-05
c             xone = x(2) - diff
c since x(1)<del, we are "close enough" to pure x(2) that
c I'm setting xtwo=1.0 and xone=1.0-diff. Now the total change
c in abundance is just diff instead of 2*diff (MHM Jan 1999)
             xone = 1.0 - diff
             xtwo = 1.0
c compute P for mixture by partial pressures.
c pmix1 is for x(2)-diff; pmix2 is for x(2)+diff
c             pmix1 = xone*pone + (diff)*ptwo
c changed above line to the following so that respective mass
c fractions for pone and ptwo add up to one (MHM Jan. 1999)
             pmix1 = xone*pone + (1.-xone)*ptwo
             pmix1 = dlog(pmix1)
c             pmix2 = pone + (diff)*ptwo
c same change as for pmix1 (MHM)
             pmix2 = xtwo*pone + (1.-xtwo)*ptwo
             pmix2 = dlog(pmix2)
c compute d ln P/d ln Y and return
c             bled=(x(2)/(2.*diff))*(pmix2-pmix1)
c changed previous line to the following (MHM)
             bled=(1./(diff))*(pmix1-pmix2)
c        write(17,*) 'Pure He in H',xone,xtwo,pmix1,pmix2
c        write(17,*) 'Ledoux',diff,bled
             return
c.................................................................
c CASE 1.2: Nearly pure hydrogen: Treat first point as pure H.
c           treat second point as H / He mixture 
c.................................................................
           else if(x(2).lt.del)then
             diff=x(2)
c changed to take care of diff=0.0 case (MHM Jan. 1999)
	     if (diff.lt.1.d-05) diff=1.d-05
c same comments as for Case 1.1, except now we set xone=0.0, 
c and xtwo=diff (MHM, Jan 1999)
             xone = 0.0
c             xtwo = x(2) + diff
             xtwo = diff
c compute P for mixture by summing partial pressures.
c pmix1 is for x(2)-diff; pmix2 is for x(2)+diff
c             pmix1 = x(1)*ptwo
             pmix1 = xone*pone + (1.-xone)*ptwo
             pmix1 = dlog(pmix1)
c             pmix2 = xtwo*pone + x(1)*ptwo
             pmix2 = xtwo*pone + (1.-xtwo)*ptwo
             pmix2 = dlog(pmix2)
c             bled=(x(2)/(2.*diff))*(pmix2-pmix1)
             bled=(x(2)/(diff))*(pmix1-pmix2)
c        write(17,*) 'Pure H in He',xone,xtwo,pmix1,pmix2
c        write(*,*) 'Pure H in He',xone,xtwo,pone,ptwo,bled
             return
           endif
c.................................................................
c CASE 2: We are not near 0 or 1 in composition
c we can use 3-point differencing formula with impunity.
c.................................................................
           xone = x(2) - del
           xtwo = x(2) + del
c           pmix1 = xone*pone + x(1)*ptwo
c           pmix2 = xtwo*pone + x(1)*ptwo
c changed previous two lines to the following two lines, again
c so that the total abundance adds up to 1 (MHM, Jan 1999)
           pmix1 = xone*pone + (1.-xone)*ptwo
           pmix2 = xtwo*pone + (1.-xtwo)*ptwo
           pmix1 = dlog(pmix1)
           pmix2 = dlog(pmix2)
c compute d ln P/d ln Y and return
c           bled=(x(2)/(2.*del))*(pmix2-pmix1)
           bled=(x(2)/(2.*del))*(pmix1-pmix2)
c        write(17,*) 'H/He mix',xone,xtwo,pmix1,pmix2
           return
         endif
c finished with H/He mixtures
c.................................................................
c He/C interface
      if(x(2).gt.0.0 .and. x(3).gt.0.0)then
c first, locate T point on grid for Helium
45      do 994 i=1,lmax
          temp(i) = tk(2,i)
994     continue
        call hunt(temp,lmax,tl,ktlo)
        kthi = ktlo + 1
c set size of temporary arrays
        kmax1 = ml(2,ktlo)
        kmax2 = ml(2,kthi)
c fill up temporary 1-D arrays for rho and T
c He arrays
        do 50 k=1,kmax1
          rhot1(k) = rhok(2,ktlo,k)
          ptmp1(k) =   pp(2,ktlo,k)
50      continue
        do 60 k=1,kmax2
          rhot2(k) = rhok(2,kthi,k)
          ptmp2(k) =   pp(2,kthi,k)
60      continue
c first, locate T point on grid for Carbon
        do 995 i=1,lmax
          temp(i) = tk(3,i)
995     continue
        call hunt(temp,lmax,tl,ktlo)
        kthi = ktlo + 1
c set size of temporary arrays
        kmax3 = ml(3,ktlo)
        kmax4 = ml(3,kthi)
c C arrays
        do 70 k=1,kmax3
          rhot3(k) = rhok(3,ktlo,k)
          ptmp3(k) =   pp(3,ktlo,k)
70      continue
        do 80 k=1,kmax4
          rhot4(k) = rhok(3,kthi,k)
          ptmp4(k) =   pp(3,kthi,k)
80      continue
c set BC's so second der is zero there
        yp1 = 1.0e+40
        ypn = 1.0e+40
c call for spline coefficients
        call spline(rhot1,ptmp1,kmax1,yp1,ypn,y21)
        call spline(rhot2,ptmp2,kmax2,yp1,ypn,y22)
        call spline(rhot3,ptmp3,kmax3,yp1,ypn,y23)
        call spline(rhot4,ptmp4,kmax4,yp1,ypn,y24)
c the following line is a kludge to fix an unknown array problem
c in the previous 4 lines
c now compute P for each isotherm at a given rho,comp point
        call splint(rhot1,ptmp1,y21,kmax1,rho,p11)
        call splint(rhot2,ptmp2,y22,kmax2,rho,p12)
        call splint(rhot3,ptmp3,y23,kmax3,rho,p13)
        call splint(rhot4,ptmp4,y24,kmax4,rho,p14)
c        write(17,*) 'He pressure L/H',p11,p12
c        write(17,*) 'C pressure L/H',p13,p14
c linear interpolation for P at the actual temp.
        frac=(tl-tk(je,ktlo))/(tk(je,kthi)-tk(je,ktlo))
        pone=p11+frac*(p12-p11)
        ptwo=p13+frac*(p14-p13)
        pone=10.**(pone)
        ptwo=10.**(ptwo)
c        write(17,*) 'frac, He, C',frac,pone,ptwo
c.................................................................
c now to consider specific composition mixtures!
c CASE 3: x(3) or x(2) is less than del (use a 3 point formula)
c CASE 3.1: Nearly pure helium: Treat first point as pure He.
c           treat second point as He / C mixture 
c.................................................................
         if(x(3).lt.del)then
           diff = 1.0-x(2)
c changed to take care of diff=0.0 case (MHM Jan. 1999)
         if (diff.lt.1.d-05) diff=1.d-05
c           xone = x(2) - diff
c since x(3)<del, we are again "close enough" to pure x(2) that
c I'm setting xtwo=1.0 and xone=1.0-diff. Now the total change
c in abundance is just diff instead of 2*diff (MHM Jan 1999)
           xone = 1. - diff
           xtwo = 1.0
c compute P for mixture by summing partial pressures.
c pmix1 is for x(2)-diff; pmix2 is for x(2)+diff
c note that I have not changed the following lines. They were
c already "correct". (MHM, Jan. 1999)
           pmix1 = xone*pone + (1.0-xone)*ptwo
           pmix2 = xtwo*pone + (1.0-xtwo)*ptwo
           pmix1 = dlog(pmix1)
           pmix2 = dlog(pmix2)
c           bled=(x(2)/(2.*diff))*(pmix1-pmix2)
           bled=(1./(diff))*(pmix1-pmix2)
c        write(17,*) 'Pure He in C',xone,xtwo,pmix1,pmix2
c        write(*,*) 'Pure He in C',xone,xtwo,pmix1,pmix2,bled
c	write(*,*) 'Pure He: xca xhe xh',x(3),x(2),x(1),del
           return
c.................................................................
c CASE 3.2: Nearly pure Carbon: Treat first point as pure C.
c           treat second point as He / C mixture 
c.................................................................
         else if(x(2).lt.del)then
           diff=x(2)
c changed to take care of diff=0.0 case (MHM Jan. 1999)
         if (diff.lt.1.d-05) diff=1.d-05
           xone = 0.0
c           xtwo = x(2) + diff
           xtwo = diff
c compute P for mixture by additive volume method.
c pmix1 is for x(2)-diff; pmix2 is for x(2)+diff
           pmix1 = xone*pone + (1.0-xone)*ptwo
           pmix2 = xtwo*pone + (1.0-xtwo)*ptwo
           pmix1 = dlog(pmix1)
           pmix2 = dlog(pmix2)
c compute d ln P/d ln Y and return.
c           bled=(x(2)/(2.*diff))*(pmix1-pmix2)
           bled=(x(2)/(diff))*(pmix1-pmix2)
c        write(17,*) 'Pure C in He',xone,xtwo,pmix1,pmix2
c        write(*,*) 'Pure C in He',xone,xtwo,pmix1,pmix2,bled
c	write(*,*) 'Pure C: xca xhe xh',x(3),x(2),x(1),del
           return
         endif
c.................................................................
c CASE 2: We are not near 0 or 1 in composition
c we can use 3-point differencing formula with impunity.
c.................................................................
        xone = x(2) - del
        xtwo = x(2) + del
        pmix1 = xone*pone + (1.0-xone)*ptwo
        pmix2 = xtwo*pone + (1.0-xtwo)*ptwo
        pmix1 = dlog(pmix1)
        pmix2 = dlog(pmix2)
c compute d ln P/d ln Y and return.
        bled=(x(2)/(2.*del))*(pmix1-pmix2)
c        write(17,*) 'He/C mix',xone,xtwo,pmix1,pmix2
c        write(*,*) 'He/C mix',xone,xtwo,pmix1,pmix2,bled
c	write(*,*) 'He/C:    xca xhe xh',x(3),x(2),x(1),del
        return
      endif
c finished with He/C mixtures.
c.................................................................
c If no transition zone, then set bled to 0.0
      bled = 0.0
      return
      end
c**************************************************************************
c compute modified Ledoux B term to include composition gradient
c effects in the Brunt-Vaisala frequency
c This is the version that computes the B term in the core.
c I vary both the C and O abundance at the C/O interface.
c PAB 1 October 1991
c**************************************************************************
      subroutine ledouxc
      implicit double precision(a-h,o-z)
c composition info
c      common/je/je
c      common/comp/amhyhe,amheca,x(4)
c has log T=t2, log Rho=u2, C abun.=xc2, O abun.=xo2, and log P=p2
      common/temp/s1,r1,s2,r2,b2,p2,t2,ea2,xc2,xo2,fca2,f2,q2,w2,c
      common/thermo/u2,up2,ut2,e2,ep2,et2,psi2,pg,o2,op2,ot2,fp,ft,
     1           en2(5),fn,fcc,ce,ci,cif,w,nu
c ledoux term value
      common/bldxc/bledc
      real*8 x(4)
c.................................
c DEL is step size increment in composition
c.................................
      bledc = 0.0
      del  = 0.01
      x(1) = 0.0
      x(2) = 0.0
      x(3) = xc2
      x(4) = xo2
c      write(17,*) 'P,T',p2,t2
c.................................................................
c C/O interface
c I am computing partial ln P/partial ln O here.
c now to consider specific composition mixtures!
c CASE 3: x(4) or x(3) is less than del (use a 3 point formula)
c CASE 3.1: Nearly pure oxygen: Treat first point as pure O.
c           treat second point as C/O mixture 
c.................................................................
         if(x(3).lt.del)then
           diff = 1.0-x(4)
           xone = x(4) - diff
           xc2 = 1.0 - xone
           xo2 = xone
           call istatco(p2,t2,1,false)
           dens1 = 10.**(u2)
           xtwo = 1.0
           xc2 = 0.0
           xo2 = xtwo
           call istatco(p2,t2,1,false)
           dens2 = 10.**(u2)
           xc2 = x(3)
           xo2 = x(4)
           bledc=(xo2/(2.*diff))*(dlog(dens2)-dlog(dens1))
c        write(17,*) 'pure O Ledoux',x(4),dens1,dens2,bledc
           return
c.................................................................
c CASE 3.2: Nearly pure Carbon: Treat first point as pure C.
c           treat second point as C/O mixture 
c.................................................................
         else if(x(4).lt.del)then
           diff = x(4)
           xone = 0.0
           xc2 = 1.0 - xone
           xo2 = xone
           call istatco(p2,t2,1,false)
           dens1 = 10.**(u2)
           xtwo = x(4) + diff
           xc2 = 1.0 - xtwo
           xo2 = xtwo
           call istatco(p2,t2,1,false)
           dens2 = 10.**(u2)
           xc2 = x(3)
           xo2 = x(4)
           bledc=(xo2/(2.*diff))*(dlog(dens2)-dlog(dens1))
c        write(17,*) 'pure C Ledoux',x(4),dens1,dens2,bledc
           return
         endif
c.................................................................
c CASE 2: We are not near 0 or 1 in composition
c we can use 3-point differencing formula with impunity.
c.................................................................
           xone = x(4) - del
           xo2 = xone
           xc2 = 1.0 - xone
           call istatco(p2,t2,1,false)
           dens1 = 10.**(u2)
           xtwo = x(4) + del
           xo2 = xtwo
           xc2 = 1.0 - xtwo
           call istatco(p2,t2,1,false)
           dens2 = 10.**(u2)
           xc2 = x(3)
           xo2 = x(4)
           bledc0=(xo2/(2.*del))*(dlog(dens2)-dlog(dens1))
           call istatco(p2,t2,1,false)
           dens3 = 10.**(u2)
           bledc1=((xo2+del/2.)/(del))*(dlog(dens2)-dlog(dens3))
           bledc2=((xo2-del/2.)/(del))*(dlog(dens3)-dlog(dens1))
c the following lines are a kludge to correct for a discontinuity
c in the density-composition relationship across the crystallization
c boudary. dlog(rho)/dlnX is artificially high when X=xone and X=xtwo
c are on different sides of the crystallization boundary. We check for
c this case with the following line and compute on just one or the other
c side of the boundary (bledc1 *or* bledc2)
	   if ( abs(bledc1-bledc2)/bledc0 .gt. 0.5 ) then 
	     if (bledc1 .lt. bledc2 ) then
	       bledc=bledc1
	     else
	       bledc=bledc2
	     endif
	     write(*,*) bledc,u2,t2,abs(bledc1-bledc2)/bledc0
	   else
	     bledc=bledc0
	   endif
c        write(17,*) 'mixture Ledoux',x(4),dens1,dens2,bledc
           return
      end
c finished with C/O mixture.
c**************************************************************************
c 1.  Photo-, Plasma, Pair, and Recombination neutrino emissivities are
c     calculated following
c     Beaudet, Petrosian, & Salpeter 1967, ApJ, 150, 979.
c 2.  Bremsstrahlung neutrino emissivity is calculated following
c     Festa & Ruderman 1969, Phys. Rev., 180, 1227.
c 3.  See also:
c     Cazzola, De Zotti, & Saggion 1971, Phys. Rev., D3, 1722
c     Flowers 1973, ApJ, 180, 911.
c 4.  Note: According to more recent studies, these overestimate the real 
c     neutrino rates by factors of 2 or more.
c**************************************************************************
      subroutine neu(u2,t2)
      implicit double precision(a-h,o-z)
      common/neut/en(5),fn
      common/temp/s1,r1,s2,r2,b2,p2,tx2,e2,xc2,xo2,fca2,f2,q2,w2,c
      common/je/je
c      common/comp/amhyhe,amheca,x(4)
      common/c/az(3),zz3(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
c.................................................................
      data a1,a2,a3,a4/5.167e-1,-3.104e0,-2.801e0,7.577e0/
c.................................................................
      data a10,a11,a12,b11,b12,b13,c1/2.320e-7,8.449e-8,1.787e-8,
     12.581e-2,1.734e-2,6.990e-4,5.6457e-1/
      data a20,a21,a22,b21,b22,b23,c2/4.886e10,7.580e10,6.023e10,
     16.290e-3,7.483e-3,3.061e-4,1.5654e00/
      data a30,a31,a32,b31,b32,b33,c3/6.002e19,2.084e20,1.872e21,
     19.383e-1,-4.141e-01,5.829e-2,5.5924e00/
c.................................................................
      z=zz3(je)
      if ( je .eq. 1 ) then
         ce=1.
      else
         ce=2.
         if ( je .eq. 3 ) then
            azx = 1. / ( xc2/12. + xo2/16. )
            z   = azx / 2.
         endif
      endif
      rho=10.**u2/ce 
c x is lambda and y is xi from BPS
      x=10.**(t2-9.77307)
      y=rho**(1./3)/(1000*x)
      yyy=y*y*y
      xx=x*x
      zz=z*z
      xyy=x*y*y
      do 1 i=1,5
        en(i)=0
1     continue
      fn=0
c****************************************************
c     plasma neutrinos (dominant at most luminosities)
c****************************************************
      if(x.lt.1e-3) go to 8
      if(y.gt.60) go to 5
      fpl=dexp(-c1*y)*(a10+y*(a11+y*a12))/(yyy+(b11+(b12+b13/x)/x)/x)
      en(1)=rho*rho*fpl/ce
c****************************************************
c     photoneutrinos (negligible for us)
c****************************************************
      if(y.gt.15) go to 3
      fph=dexp(-c2*y)*(a20+y*(a21+y*a22))/(yyy+(b21+(b22+b23/x)/x)/x)
      en(2)=xx*xx*x*fph/ce
c****************************************************
c     pair neutrinos (noticable at high luminosities)
c****************************************************
    3 if(x.lt.4e-2.or.y.gt.5) go to 4
      fpa=dexp(-c3*y)*(a30+y*(a31+y*a32))/(yyy+(b31+(b32+b33/x)/x)/x)
      g=1-xx*(13.04-xx*(133.5+xx*(1534.+xx*918.6)))
      en(3)=g*fpa/(rho*ce*dexp(2.d0/x))
c****************************************************
c     recombination neutrinos (a small contributor)
c****************************************************
4     if(y.gt.10) go to 5
      u=24.280*xyy*ce**(2./3) 
      if(u.gt.30) go to 5
      v=zz/(3.7772e04*x)
      frc=xx/dexp(u+v)
      en(4)=1.850e-4*zz*zz*rho*frc/ce
c****************************************************
c     gas bremsstrahlung neutrinos (dominant at low L)
c****************************************************
5     u=u2-dlog10(ce)
      v=a1*u
      fbr=4.775e-1*atan(1.707e0*(u-t2)+4.566e0)+5.250e0
      w=a2+v+(t2-8)*fbr-dsqrt(v*v+a3*u+a4)
      en(5)=(2/ce)*z*10.**w
c.................................................................
      do 7 i=1,5
         fn=fn+en(i)
7     continue
8     return
      end 
c********************************************************************
c  Neutrino rates from Munkata, Kohyama and Itoh (1985), Ap. J.  296, 197.
c    (Plasma, Photo, Pair)
c  and Beaudet, Petrosian, and Salpeter (1967), ApJ, 150, 979.
c    (Recomb.)
c  Bremsstrahlung from Lamb thesis (DeZotti)
c
c Standardized for ISUEVO by SDK: December 1990
c Modified for WDXD by PAB: Sept. 1990 (Fixed rho/mue problem Feb 1993)
c********************************************************************
       subroutine neutrino(u2,t2)
       implicit double precision (a-h,o-z)
       common/neut/en(5),fn
       common/temp/s1,r1,s2,r2,b2,p2,tx2,e2,xc2,xo2,fca2,f2,q2,w2,c
       common/je/je
       common/c/az(3),zz3(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,
     1 el1x,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
       double precision mue
c********************************************************************
       t=10.d0**t2
       rho=10.d0**u2
       if(je.eq.1)then
         ce = 1.
        else
         ce = 2.
c note: MUE was AZX in old neutrino routine
         if(je.eq.3)then
           mue = 1./((xc2*6./12.)+(xo2*8./16.))
           zbar = (xc2*6.d0) + (xo2*8.d0)
         endif
       endif
c.........................................................................
c   zbar is the mean charge on a nucleus, for bremm and pair.  
c.........................................................................
       flam=t/5.9302d9
       cv=0.5d0+2.0d0*0.217d0
       cvp=1.d0-cv
       ca=0.5d0
       cap=1.d0-ca
       n = 2
       call plasma(flam,rho,mue,cv,cvp,ca,cap,n,qplas)
       call photo(flam,rho,mue,cv,cvp,ca,cap,n,qphot)
       call pair(flam,rho,mue,cv,cvp,ca,cap,n,qpair)
       call recomb(flam,t,rho,zbar,qrec)
       call bremm(t,rho,mue,zbar,qbrem)
       eplas=qplas/rho
       en(1) = eplas
       ephot=qphot/rho
       en(2) = ephot
       epair=qpair/rho
       en(3) = epair
       erec=qrec/rho
       en(4) = erec
       ebrem=qbrem/rho
       en(5) = ebrem
       etot=eplas+ephot+epair+erec+ebrem
       fn=etot
       return
       end
c********************************************************************
       subroutine plasma(flam,rho,mue,cv,cvp,ca,cap,n,qplas)
       implicit double precision (a-h,o-z)
       dimension a(0:2),b(3)
       double precision mue
       xi=(((rho/mue)/1d9)**(0.33333333333d0))/flam
       a(0)=2.320d-7
       a(1)=8.449d-8
       a(2)=1.787d-8
       b(1)=2.581d-2
       b(2)=1.734d-2
       b(3)=6.990d-4
       c=0.56457d0
       call qcalc(a,b,c,xi,flam,f)
       qplas=(cv**2+n*cvp**2)*((rho/mue)**3)*f
       return
       end
c********************************************************************
       subroutine photo(flam,rho,mue,cv,cvp,ca,cap,n,qphot)
       implicit double precision (a-h,o-z)
       dimension a(0:2),b(3)
       double precision mue
       xi=(((rho/mue)/1e9)**(1./3.))/flam
       a(0)=4.886d10
       a(1)=7.580d10
       a(2)=6.023d10
       b(1)=6.290d-3
       b(2)=7.483d-3
       b(3)=3.061d-4
       c=1.5654d0
       call qcalc(a,b,c,xi,flam,f)
       qphot=(rho/mue)*(flam**5.)*f
       temp=1.875d8*flam+1.654d8*flam**2+8.499d8*flam**3-1.604d8*flam**4
       smq=0.666d0*((1.+2.045d0*flam)**-2.066d0)/(1.d0+rho/mue/temp)
       temp=(cv**2-ca**2)+n*(cvp**2-cap**2)
       temp=temp/((cv**2+ca**2)+n*(cvp**2+cap**2))
       bigq=0.5*((cv**2+ca**2)+n*(cvp**2+cap**2))*(1.d0-temp*smq)
       bigq=bigq*0.893e9*flam**8*((1+143.8d0*flam**3.555d0)**0.3516d0)
       qphot=bigq*xi**3*dexp((0.556d0*xi**4.48d0)/(150.d0+xi**3.48d0))*f
       return
       end
c********************************************************************
       subroutine pair(flam,rho,mue,cv,cvp,ca,cap,n,qpair)
       implicit double precision (a-h,o-z)
       dimension a(0:2),b(3)
       double precision mue
       xi=(((rho/mue)/1e9)**(1./3.))/flam
       a(0)=6.002e19
       a(1)=2.084e20
       a(2)=1.872e21
       b(1)=9.383e-1
       b(2)=-4.141e-1
       b(3)=5.829e-2
       c=5.5924
       call qcalc(a,b,c,xi,flam,f)
       g=1.d0-13.04d0*(flam**2)+133.5d0*(flam**4)+1534d0*(flam**6)
     1       +918.6d0*(flam**8)
       gef=g*dexp(-2.d0/flam)*f
       temp=1.d0/(10.7480d0*flam**2+0.3967d0*flam**0.5d0+1.0050d0)
       smq=temp*(1.d0+(rho/mue)/
     1     (7.692d7*flam**3+9.715d6*flam**0.5d0))**(-0.3)
       cvm=(cv**2-ca**2)+n*(cvp**2-cap**2)
       cvp=(cv**2+ca**2)+n*(cvp**2+cap**2)
       qpair=0.5d0*cvp*(1.d0+(cvm/cvp)*smq)*gef
       return
       end
c********************************************************************
c Also handles rates for pair, photo-, and plasma rates in Munkata etal.
c (1985) paper.
       subroutine qcalc(a,b,c,xi,flam,f)
       implicit double precision (a-h,o-z)
       dimension a(0:2),b(3)
       top=(a(0)+a(1)*xi+a(2)*(xi**2.))*dexp(-c*xi)	
       bottom=(xi**3.)+(b(1)/flam)+(b(2)/(flam**2))+(b(3)/(flam**3))
       f=top/bottom
       return
       end
c********************************************************************
       subroutine bremm(t,rho,mue,z,qbrem)
       implicit double precision (a-h,o-z)
       dimension b(3),c(4)
       double precision mue
       a=5.167d-1
       b(1)=7.577d0
       b(2)=-2.801d0
       b(3)=0.267d0
       c(1)=0.4775d0
       c(2)=1.707d0
       c(3)=-9.090d0
       c(4)=5.250d0
       x=dlog10(rho/mue)
       y=dlog10(t/1.d8)
       g=c(1)*atan(c(2)*(x-y)+c(3))+c(4)
       f=y*g+a*x-((b(1)+b(2)*x+(b(3)*(x**2)))**(0.5d0))
       eps=7.870d-3*(2.d0/mue)*z*(10.d0**(f))
       qbrem=eps*rho
       return
       end
c********************************************************************
       subroutine recomb(flam,t,rho,z,qrec)
       implicit double precision (a-h,o-z)
       zeta=1.57d5*z*z/t
       temp=1.850d-4*(z**6)*((2.d0*z)**(-2))*rho*rho*flam*flam
       qrec=dexp(-zeta-(2.428d-5*(rho**(0.66667d0))/flam))*temp
       return
       end
c**************************************************************************
c  Neutrino rates from Itoh, Adachi, Nakagawa, Kohyama, & Munkata, 1989, 
c  ApJ, 339, 354. (+Errata 1990, ApJ, 360, 741)
c    (Plasma, Photo, Pair)
c Recombination neutrinos from Kohyama, Itoh, Obama, & Mutoh 1993, ApJ,
c  415, 267
c  Bremsstrahlung from:
c 1. Liquid Metal: Itoh & Kohyama, 1983, ApJ, 275, 858.
c 2. Crystal Lattice: Itoh etal., 1984, ApJ, 279, 413.
c 3. Low T Quantum Corr.: Itoh etal., 1984, ApJ, 280, 787.
c 4. Phonon Contr.: Itoh etal., 1984, ApJ, 285, 304.
c 5. Partially Degen. e: Munkata, Kohyama & Itoh, 1987, ApJ, 316, 708.
c Prepared for WDXD by PAB: Oct. 1990
c Last messed with on 08 17 94
c**************************************************************************
       subroutine gnutrino(u2,t2)
       implicit double precision (a-h,o-z)
       common/neut/en(5),fn
      common/temp/s1,r1,s2,r2,b2,p2,tx2,e2,xc2,xo2,fca2,f2,q2,w2,c
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
       common/rhomooe/rhomooe
       common/phys/flam,gamma,degtemp,tfermi
       double precision mue
       data gamc,gamo,gamhe,gamh/3577200.,5778200.,573264.,227500./
       data gamfe/4.019747d+7/
c initialize
       t=10.d0**t2
       rho=10.d0**u2
       do 10 i=1,5
         en(i) = 0.0
10     continue
       fn = 0.0
       qbrem = 0.0
       qplas = 0.0
       qphot = 0.0
       qpair = 0.0
       qrec  = 0.0
c.................................................................
c Escape clause (enter when T < 10^7K or Rho < 1 g/cc)
c With this, I don't need an escape clause for flam
c.................................................................
       if(t2.lt.7.0 .or. u2.lt.0.0)then
         return
       endif
c compute gamma and mue for mixture
       if(je.eq.1)then
c assume pure H for now. (Note: for H, I shouldn't be here anyway!)
         ce = 1.
         mue = 1.
         gamma = gamh * 10.**(u2/3. - t2)
         zbar = 1.0
         aavg = 1.0
        else
c         ce = 2.
         if(je.eq.2)then
c pure helium
           omue = 1./(x(2)/2.)
           gamma = gamhe * 10.**(u2/3. - t2)
           zbar = 2.0
	   aavg = 4.0
          else
c helium, carbon, oxygen, and iron (the latter for tests)
           gammahe = gamhe * 10.**(u2/3. - t2)
           gammac  = gamc  * 10.**(u2/3. - t2)
           gammao  = gamo  * 10.**(u2/3. - t2)
c           gammaf  = gamfe * 10.**(u2/3. - t2)
c iron in mix
c           gamma   = x(3)*gammac + x(4)*gammao + x(5)*gammaf
c           zbar = (x(3)*6.0 + x(4)*8.0 + x(5)*26.0)
c           omue = (x(3)*6./12.) + (x(4)*8./16.) + (x(5)*26./56.) 
           gamma = xc2*gammac + xo2*gammao
           omue = xc2*6./12. + xo2*8./16.
           zbar = xc2*6.0 + xo2*8.0
	   aavg = xc2*12.0 + xo2*16.0
         endif
       endif
c.................................................................
c zbar is the mean charge on a nucleus, for bremm and pair.  
c flam was x in neu 
c.................................................................
       flam=t/5.9302d+9
       mue = 1.d0 / omue
       rhomooe = rho/mue
       rhomue23 = (rhomooe)**(2./3.)
       sqfac = 1. + 1.018*rhomue23
       tfermi = 5.9302d+9*(dsqrt(sqfac) - 1.)
       degtemp = t/tfermi
       y = (rhomooe)**(1./3.)/(1000.*flam)
c       write(6,1001) flam,rhomooe,tfermi,y
1001  format(4(1x,1pe11.4))
       s2theta = 0.217
       cv=0.5d0+2.0d0*s2theta
       cvp=1.d0-cv
       ca=0.5d0
       cap=1.d0-ca
       if(y .le. 100.)then
         call plasmon(flam,rho,t2,mue,cv,cvp,ca,cap,qplas)
       endif
       if(y .le. 100.)then
         call photon(flam,t2,rho,mue,cv,cvp,ca,cap,qphot)
       endif
c       if(flam.ge.0.10 .or. y.le.40.)then
       if(flam.ge.0.10)then
         call pairn(flam,rho,mue,cv,cvp,ca,cap,qpair)
       endif
       call recombine(aavg,zbar,t,rho,cv,cvp,ca,cap,qrec)       
c       if(y.le.20.)then
c         call recomb(flam,t,rho,zbar,qrec)
c       endif
c bremsstrahlung from partially degenerate electrons.
       if(u2.gt.4.0 .and. degtemp.gt.0.3)then
         call brempde(u2,t2,gamma,qbrempde)
c         write(*,*) 'brempde',degtemp,qbrempde
         qbrem = qbrempde
         if(qbrem.lt.0.0)then
           qbrem = 0.0
         endif
c skip over other neutrino rates if we came here.
         go to 50
       endif
c if gamma < 1 or rho < 10^4 g/cc,
c then skip bremsstrahlung neutrino rates.
c       if(gamma.gt.0.5 .and. u2.ge.4.0)then
c       if(gamma.gt.0.5 .and. u2.ge.1.0)then
       if(u2.ge.1.0)then
         if(gamma.gt.1.0 .and. gamma.lt.171.0)then
           if(x(2).gt.0.00001)then
             call bremliqh(u2,t2,gamma,qbremlh)
            else 
             qbremlh = 0.0
           endif
           if(x(3).gt.0.00001)then
             call bremliqc(u2,t2,gamma,qbremlc)
            else 
             qbremlc = 0.0
           endif
           if(x(4).gt.0.00001)then
             call bremliqo(u2,t2,gamma,qbremlo)
            else 
             qbremlo = 0.0
           endif
c           if(x(5).gt.0.00001)then
c             call bremliqf(u2,t2,gamma,qbremlf)
c            else 
c             qbremlf = 0.0
c           endif
           qbrem = qbremlh*x(2) + qbremlc*x(3) + qbremlo*x(4)
c           qbrem =  qbremlc*x(3) + qbremlo*x(4) + qbremlf*x(5)
c           write(7,1022) t2,u2,gamma,qbremlc,qbremlo,qbremlf
c1022    format(5x,'T,Rho,Gamma,C,O,Fe =',6(1x,1pe12.5))
         elseif(gamma.ge.171.0)then
c don't have solid helium
c           if(x(2).gt.0.00001)then
c             call bremsolh(u2,t2,gamma,qbremsh)
c            else 
c             qbremsh = 0.0
c           endif
           if(x(3).gt.0.00001)then
             call bremsolc(u2,t2,gamma,qbremsc)
            else 
             qbremsc = 0.0
           endif
           if(x(4).gt.0.00001)then
             call bremsolo(u2,t2,gamma,qbremso)
            else 
             qbremso = 0.0
           endif
c           if(x(5).gt.0.00001)then
c             call bremsolf(u2,t2,gamma,qbremsf)
c            else 
c             qbremsf = 0.0
c           endif
           qbrem = qbremsc*x(3) + qbremso*x(4)
c           qbrem = qbremsh*x(2) + qbremsc*x(3) + qbremso*x(4) 
c     1           + qbremsf*x(5)
           if(qbrem.lt.0.0)then
             write(*,*) 'Ooops!',qbrem
             qbrem = 0.0
           endif
         endif
       endif
c convert to lumonisities
50     eplas=qplas/rho
       en(1) = eplas
       ephot=qphot/rho
       en(2) = ephot
       epair=qpair/rho
       en(3) = epair
       erec=qrec/rho
       en(4) = erec
c output of Bremsstrahlung routines already is divided by rho
       ebrem=qbrem
       en(5) = ebrem
       etot=eplas+ephot+epair+erec+ebrem
       fn=etot
       return
       end
c***********************************************************************
c Plasmon Neutrinos
c Have temp dependent coefficients when in low T regime.
c I'm including coefficients valid both above and below
c log T = 7.8  The entire T range is covered.
c***********************************************************************
       subroutine plasmon(flam,rho,t2,mue,cv,cvp,ca,cap,qplas)
       implicit double precision (a-h,o-z)
       dimension a(0:3),b(3)
       double precision mue
       xi=(((rho/mue)/1d9)**(0.33333333333d0))/flam
       if(t2.lt.7.8)then
         a(0) =  2.787d-7*t2*t2 - 3.936d-6*t2 + 1.408d-5
         a(1) =  5.408d-7*t2*t2 - 7.715d-6*t2 + 2.751d-5
         a(2) = -1.812d-7*t2*t2 + 2.731d-6*t2 - 1.024d-5
         a(3) =  1.639d-8*t2*t2 - 2.445d-7*t2 + 9.137d-7
         b(1) = 1.148d-2
         b(2) = 1.209d-1
         b(3) = 2.432d-4
         c    = -0.7683 + 0.1798*t2
        else
         a(0) = 2.320d-7
         a(1) = 8.449d-8
         a(2) = 1.787d-8
         a(3) = 0.0
         b(1) = 2.581d-2
         b(2) = 1.734d-2
         b(3) = 6.990d-4
         c    = 0.56457d0
       endif
       call qcalc2(a,b,c,xi,flam,f)
       n = 2
       qplas=(cv**2+n*cvp**2)*((rho/mue)**3)*f
       return
       end
c***********************************************************************
c Photoneutrinos
c a(i) terms are different in the newest paper. Pain to crank out too.
c Coefficients valid for 10^7 to 10^8K, then 10^8 to 10^9K, and
c finally above 10^9K. 
c***********************************************************************
       subroutine photon(flam,t2,rho,mue,cv,cvp,ca,cap,qphot)
       implicit double precision (a-h,o-z)
       dimension a(0:2),b(3),c(3,7),d(3,5)
       double precision mue
       xi=(((rho/mue)/1.d9)**(1./3.))/flam
c.......................................................
c b(i) terms same as before. Load 'em up.
c.......................................................
       b(1)=6.290d-3
       b(2)=7.483d-3
       b(3)=3.061d-4
c......................................................
c Calculate a(i) numerical factors.
c and tabulated coefficients for sines and cosines.
c tabulated stuff is in form c(i,j), starting with 
c i=1, j=1 instead of 0,0
c......................................................
      pi = 3.1415926535898
      half = 1.d0 / 2.d0
      facpi = 5.d0 * pi / 3.d0 
c 10^7 to 10^8 K
      if(t2.ge.7.0 .and. t2.lt.8.0)then
        tau = t2 - 7.0
        cee = 0.5654 + tau
        c(1,1) = 1.008d+11 * half
        c(1,2) = 0.0
        c(1,3) = 0.0
        c(1,4) = 0.0
        c(1,5) = 0.0
        c(1,6) = 0.0
        c(1,7) = 0.0
        c(2,1) =  8.156d+10 * half
        c(2,2) =  9.728d+8
        c(2,3) = -3.806d+9
        c(2,4) = -4.384d+9
        c(2,5) = -5.774d+9
        c(2,6) = -5.249d+9
        c(2,7) = -5.153d+9 * half
        c(3,1) =  1.067d+11 * half
        c(3,2) = -9.782d+9
        c(3,3) = -7.193d+9
        c(3,4) = -6.936d+9
        c(3,5) = -6.893d+9
        c(3,6) = -7.041d+9
        c(3,7) = -7.193d+9 * half
        d(1,1) = 0.0
        d(1,2) = 0.0
        d(1,3) = 0.0
        d(1,4) = 0.0
        d(1,5) = 0.0
        d(2,1) = -1.879d+10
        d(2,2) = -9.667d+9
        d(2,3) = -5.602d+9
        d(2,4) = -3.370d+9
        d(2,5) = -1.825d+9
        d(3,1) = -2.919d+10
        d(3,2) = -1.185d+10
        d(3,3) = -7.270d+9
        d(3,4) = -4.222d+9
        d(3,5) = -1.560d+9
c 10^8 to 10^9 K
      elseif(t2.ge.8.0 .and. t2.lt.9.0)then
        tau = t2 - 8.0
        cee = 1.5654
        c(1,1) =  9.889d+10 * half
        c(1,2) = -4.524d+8
        c(1,3) = -6.088d+6
        c(1,4) =  4.269d+7
        c(1,5) =  5.172d+7
        c(1,6) =  4.910d+7
        c(1,7) =  4.388d+7 * half
        c(2,1) =  1.813d+11 * half
        c(2,2) = -7.556d+9
        c(2,3) = -3.304d+9
        c(2,4) = -1.031d+9
        c(2,5) = -1.764d+9
        c(2,6) = -1.851d+9
        c(2,7) = -1.928d+9 * half
        c(3,1) =  9.750d+10 * half
        c(3,2) =  3.484d+9
        c(3,3) =  5.199d+9
        c(3,4) = -1.695d+9
        c(3,5) = -2.865d+9
        c(3,6) = -3.395d+9
        c(3,7) = -3.418d+9 * half
        d(1,1) = -1.135d+8
        d(1,2) =  1.256d+8
        d(1,3) =  5.149d+7
        d(1,4) =  3.436d+7
        d(1,5) =  1.005d+7
        d(2,1) =  1.652d+9
        d(2,2) = -3.119d+9
        d(2,3) = -1.839d+9
        d(2,4) = -1.458d+9
        d(2,5) = -8.956d+8
        d(3,1) = -1.549d+10
        d(3,2) = -9.338d+9
        d(3,3) = -5.899d+9
        d(3,4) = -3.035d+9
        d(3,5) = -1.598d+9
c Temperature is greater than 10^9 K
      elseif(t2.ge.9.0)then
        tau = t2 - 9.0
        cee = 1.5654
        c(1,1) =  9.581d+10 * half
        c(1,2) =  4.107d+8
        c(1,3) =  2.305d+8
        c(1,4) =  2.236d+8
        c(1,5) =  1.580d+8
        c(1,6) =  2.165d+8
        c(1,7) =  1.721d+8 * half
        c(2,1) =  1.459d+12 * half
        c(2,2) =  1.314d+11
        c(2,3) = -1.169d+11
        c(2,4) = -1.765d+11
        c(2,5) = -1.867d+11
        c(2,6) = -1.983d+11
        c(2,7) = -1.896d+11 * half
        c(3,1) =  2.424d+11 * half
        c(3,2) = -3.669d+9
        c(3,3) = -8.691d+9
        c(3,4) = -7.967d+9
        c(3,5) = -7.932d+9
        c(3,6) = -7.987d+9
        c(3,7) = -8.333d+9 * half
        d(1,1) =  4.724d+8
        d(1,2) =  2.976d+8
        d(1,3) =  2.242d+8
        d(1,4) =  7.937d+7
        d(1,5) =  4.859d+7
        d(2,1) = -7.094d+11
        d(2,2) = -3.697d+11
        d(2,3) = -2.189d+11
        d(2,4) = -1.273d+11
        d(2,5) = -5.705d+10
        d(3,1) = -2.254d+10
        d(3,2) = -1.551d+10
        d(3,3) = -7.793d+9
        d(3,4) = -4.489d+9
        d(3,5) = -2.185d+9
      endif
c......................................................
c set up the variables (sines and cosines) for the loops
c......................................................
      su1 = dsin(facpi*1.d0*tau)
      su2 = dsin(facpi*2.d0*tau)
      su3 = dsin(facpi*3.d0*tau)
      su4 = dsin(facpi*4.d0*tau)
      su5 = dsin(facpi*5.d0*tau)
      cu1 = dcos(facpi*1.d0*tau)
      cu2 = dcos(facpi*2.d0*tau)
      cu3 = dcos(facpi*3.d0*tau)
      cu4 = dcos(facpi*4.d0*tau)
      cu5 = dcos(facpi*5.d0*tau)
      cu  = dcos(facpi*10.d0*tau)
c......................................................
c  Sum up the c's and d's to make a's
c......................................................
      csum1 = c(1,1) + c(1,2) * cu1 + c(1,3) *cu2 + 
     1        c(1,4) * cu3 + c(1,5) * cu4 + c(1,6) * cu5
      dsum1 = d(1,1) * su1 + d(1,2) * su2 + d(1,3) *su3 + 
     1        d(1,4) * su4 + d(1,5) * su5 
      a(0)  = csum1 + dsum1 + c(1,7) * cu
      csum2 = c(2,1) + c(2,2) * cu1 + c(2,3) *cu2 + 
     1        c(2,4) * cu3 + c(2,5) * cu4 + c(2,6) * cu5
      dsum2 = d(2,1) * su1 + d(2,2) * su2 + d(2,3) *su3 + 
     1        d(2,4) * su4 + d(2,5) * su5 
      a(1)  = csum2 + dsum2 + c(2,7) * cu
      csum3 = c(3,1) + c(3,2) * cu1 + c(3,3) *cu2 + 
     1        c(3,4) * cu3 + c(3,5) * cu4 + c(3,6) * cu5
      dsum3 = d(3,1) * su1 + d(3,2) * su2 + d(3,3) *su3 + 
     1        d(3,4) * su4 + d(3,5) * su5 
      a(2)  = csum3 + dsum3 + c(3,7) * cu
c......................................................
c There, now we have the a(i) coefficients! Lot's o'work.
c......................................................
       call qcalc(a,b,cee,xi,flam,f)
       n=2
       temp=1.875d8*flam+1.653d8*flam**2+8.499d8*flam**3-1.604d8*flam**4
       smq=0.666d0*((1.+2.045d0*flam)**-2.066d0)/(1.d0+rho/mue/temp)
       temp=(cv**2-ca**2)+n*(cvp**2-cap**2)
       temp=temp/((cv**2+ca**2)+n*(cvp**2+cap**2))
       bigq=0.5*((cv**2+ca**2)+n*(cvp**2+cap**2))*(1.d0-temp*smq)
       qphot=bigq*f*(rho/mue)*(flam**5)
       return
       end
c***********************************************************************
c Pair production neutrinos
c Munkata etal version is unchanged except for log T > 10, which
c we better not hit, although I've included the appropriate coefficients.
c***********************************************************************
       subroutine pairn(flam,rho,mue,cv,cvp,ca,cap,qpair)
       implicit double precision (a-h,o-z)
       dimension a(0:2),b(3)
       double precision mue
       xi=(((rho/mue)/1e9)**(1./3.))/flam
       a(0)=6.002e19
       a(1)=2.084e20
       a(2)=1.872e21
       n = 2
c pick out appropriate b's and c's according to temp
c flam = 1.6863 is T = 10^10 K
       if(flam.lt.1.6863)then
         b(1)=9.383e-1
         b(2)=-4.141e-1
         b(3)=5.829e-2
         c=5.5924
        else
         b(1)= 1.2383
         b(2)=-8.141e-1
         b(3)=0.0
         c=4.9924
       endif
       call qcalc(a,b,c,xi,flam,f)
       g=1.d0-13.04d0*(flam**2)+133.5d0*(flam**4)+1534d0*(flam**6)
     1       +918.6d0*(flam**8)
       gef=g*dexp(-2.d0/flam)*f
       temp=1.d0/(10.7480d0*flam**2+0.3967d0*flam**0.5d0+1.0050d0)
       smq=temp*(1.d0+(rho/mue)/
     1     (7.692d7*flam**3+9.715d6*flam**0.5d0))**(-0.3)
       cvm=(cv**2-ca**2)+n*(cvp**2-cap**2)
       cvp=(cv**2+ca**2)+n*(cvp**2+cap**2)
       qpair=0.5d0*cvp*(1.d0+(cvm/cvp)*smq)*gef
       return
       end
c***********************************************************************
c this version is called by the plasma routine, which has a new a^3 term
c***********************************************************************
       subroutine qcalc2(a,b,c,xi,flam,f)
       implicit double precision (a-h,o-z)
       dimension a(0:3),b(3)
       top=(a(0)+a(1)*xi+a(2)*(xi**2.)+a(3)*(xi**3.))*dexp(-c*xi)	
       bottom=(xi**3.)+(b(1)/flam)+(b(2)/(flam**2))+(b(3)/(flam**3))
       f=top/bottom
       return
       end
c======================================================================
c Recombination neutrino rates of Kohyama, Itoh, Obama, & Mutoh,
c 1993, ApJ, 415, 267.
c These are much lower than Beaudet, Petrosian, & Salpeter rates,
c and are not believed to be important for white dwarf stars.
c======================================================================
      subroutine recombine(aavg,zbar,t,rho,cv,cvp,ca,cap,qrec)       
      implicit double precision(a-h,o-z)
      common/comp/amhyhe,amheca,x(4)
      real n
      n = 2.0
      zetfac = 1.579d+5 * zbar * zbar / t
c gnoofac for case where A/Z = 2 
c no H version
      gnoofac = 5.526d+7*rho/(t**1.5)
c    H version
      if(x(1) .ne. 0.0)then
        gnoofac = gnoofac * (1.d0 + (0.992*x(1)/1.008))
      endif
c gnoofac for case where A/Z ne 2 due to heavy elements.
c      gnoofac = 5.526d+7*rho/(t**1.5)) * 2.d0*zbar/aavg
c now determine gnoo from gnoofac via inverse of F-D integral
      gnoo = finv(gnoofac)
c if nu falls outside the range, exit
      if(gnoo.lt.-20.0 .or. gnoo.gt.10.0)then
        qrec = 0.d0
        return
      endif
c now compute Q from pieces
      qfac1 = (cv*cv + 1.5*ca*ca) + (n*(cvp*cvp +1.5*cap*cap))
      qfac2 = 2.649d-18 * zbar**14 * rho / aavg
      qfac3 = 1.d0 / (1.d0 + dexp(zetfac+gnoo))
c determine Q for recombination neutrinos. Qo for comparison to paper.
c      qzero = qfac2*qfac3*rjay(zetfac,gnoo)
      qrec = qfac1*qfac2*qfac3*rjay(zetfac,gnoo)
      return      
      end 
c======================================================================
c Function to determine value of J for each element
c======================================================================
      function rjay(zetfac,gnoo)       
      implicit double precision(a-h,o-z)
      if(gnoo.ge.-20.0 .and. gnoo.lt.0.d0)then
	zfac = zetfac
        rjnum = (0.0151/zfac)+(0.242/zfac**2.25)+(1.21/zfac**4.55)
        rjden = 1.d0 + 0.0371*dexp(0.906*gnoo)*(1.d0+0.928*zfac)
	rjay = rjnum*dexp(gnoo)/rjden
      elseif(gnoo .le. 10.0)then
        den = 1.d0+(-0.0120*gnoo)+(0.029*gnoo*gnoo)+(-0.00104*gnoo**3)
	zfac = zetfac/den
        rjnum = (0.0123/zfac)+(0.266/zfac**2.25)+(1.30/zfac**4.55)
        rjden = 1.d0 + 0.117*dexp(0.897*gnoo)*(1.d0+0.177*zfac)
	rjay = rjnum*dexp(gnoo)/rjden
      endif
      return      
      end 
c======================================================================
c Compute Fermi-Dirac integral F1/2 to accuracy of 10^-4 using 
c rational function approximation of Anita (1993, ApJS, 84, 101).
c======================================================================
      function finv(f)       
      implicit double precision(a-h,o-z)
      dimension a1(3),b1(3),a2(3),b2(3)
c       
c low precision formulae
      data an,m1,k1,m2,k2/0.5d0,2,2,2,2/
      data a1/4.4593646d+01,+1.1288764d+01,1.0000000d+00/
      data b1/3.9519346d+01,-5.7517464d+00,2.6594291d-01/
      data a2/3.4873722d+01,-2.6922515d+01,1.0000000d+00/
      data b2/2.6612832d+01,-2.0452930d+01,1.1808945d+01/
c............................................
c small eta rational function       
c............................................
      if(f .lt. 4.d0)then
        rn = f + a1(m1)
        do 100 i=m1-1,1,-1
          rn = rn*f + a1(i)
100     continue
        den = b1(k1+1)
        do 200 i=k1,1,-1
          den = den*f + b1(i)
200     continue
        finv = dlog(f * rn / den)
       else
c............................................
c large eta rational function       
c............................................
        ff = 1.d0 / f**(1.d0/(1.d0+an))
        rn = ff + a2(m2)
        do 300 i=m2-1,1,-1
          rn = rn*ff + a2(i)
300     continue
        den = b2(k2+1)
        do 400 i=k2,1,-1
          den = den*ff + b2(i)
400     continue
        finv =  rn / (den * ff)
      endif

      return      
      end 
c***************************************************************************
c This subroutine returns the bremsstrahlung neutrino rates computed for 
c Liquid Helium using the results of Itoh, Kohyama, Matsumoto, & Seki 
c 1984, ApJ, 280, 787 and Itoh & Kohyama 1983, ApJ, 275, 858
c These formulae include liquid metal and low temp. quantum correction conts.
c 10 12 92 Put in typo correction from erratum
c***************************************************************************
      subroutine bremliqh(dens,t,gamma,qbremlh)
      implicit double precision (a-z)
c......................................................
c  declare variables (find the values in the block data piece of code).
c......................................................
      common/liqdath/ a(6), b(4), e(6), f(4), i(6), j(4), p(6), q(4),
     1          alpha(4), beta(4), c, d, g, h, k, l, r, s
      common/quandath/ aq(6), bq(4), eq(6), fq(4),
     1          alphaq(5), cq, dq, gq, hq
c...........................................................
c en is the number of neutrino flavors besides electron.
c s2theta is sin(theta)**2, where theta is the Weinberg angle
c...........................................................
      at = 4.
      z = 2.
      twopi = 6.283185308
      pisqr = 9.869604404
      en = 2.
      s2theta = 0.217
      cv = 0.5 + 2.*s2theta
      ca = 0.5
      cvp = 1. - cv
      cap = 1. - ca
      x13 = -1./3.
      xone3 = 1./3.
      x14 = -0.25000000
      x23 = -2./3.
      x24 = -0.50000000
      x34 = -0.75000000
      x43 = 4./3.
      t8 = 10.**t / 1.e+08
      d6 = 10.**dens / 1.e+06 
      alfa = 0.07279 * gamma * (d6**xone3) / ((z*at)**x43)
c.................................................................
c compute the interpolation coefficients v and w.
c NOTE: u is defined differently here than in BREMSOL*.f
c.................................................................
      u = twopi * ( dens - 3. ) / 10.
c liquid variables
      v = alpha(1) + alpha(2) * gamma**x13 + alpha(3) * gamma**x23
     1    + alpha(4) / gamma
      w = beta(1) + beta(2) * gamma**x13 + beta(3) * gamma**x23
     1    + beta(4) / gamma
c quantum correction variables
      vq = alphaq(1) + alphaq(2)*gamma**x14 + alphaq(3)*gamma**x24
     1    + alphaq(4)*gamma**x34 + alphaq(5) / gamma
c.................................................................
c set up the variables (sines and cosines) for the loops
c.................................................................
      su  = dsin(u)
      su2 = su * su 
      cu2 = 1. - su2
      cu  = dsqrt( cu2 )
      s2u = 2. * su * cu
      c2u = cu2 - su2
      s3u = su * ( 3. - 4. * su2 )
      c3u = cu * ( 4. * cu2 - 3. )
      c4u = -8. * cu2 * su2 + 1.
      s4u = su * ( (2.*c3u) + (2.*cu) )
      c5u = cu * ( (16.*cu2*cu2) - (20.*cu2) + 5. )
c.................................................................
c  Liquid contributions
c.................................................................
      asum = a(1) + a(2) * cu + a(3) * c2u
     1            + a(4) * c3u + a(5) * c4u + a(6) * c5u
      bsum = b(1) * su + b(2) * s2u + b(3) * s3u
     1       + b(4) * s4u + c*u + d
      esum = e(1) + e(2) * cu + e(3) * c2u
     1            + e(4) * c3u + e(5) * c4u + e(6) * c5u
      fsum = f(1) * su + f(2) * s2u + f(3) * s3u
     1       + f(4) * s4u + g*u + h
      isum = i(1) + i(2) * cu + i(3) * c2u
     1            + i(4) * c3u + i(5) * c4u + i(6) * c5u
      jsum = j(1) * su + j(2) * s2u + j(3) * s3u
     1       + j(4) * s4u + k*u + l
      psum = p(1) + p(2) * cu + p(3) * c2u
     1            + p(4) * c3u + p(5) * c4u + p(6) * c5u
      qsum = q(1) * su + q(2) * s2u + q(3) * s3u
     1       + q(4) * s4u + r*u + s
c  Quantum correction contributions
      aqsum = aq(1) + aq(2) * cu + aq(3) * c2u
     1       + aq(4) * c3u + aq(5) * c4u + aq(6) * c5u
      bqsum = bq(1) * su + bq(2) * s2u + bq(3) * s3u
     1       + bq(4) * s4u + cq*u + dq
      eqsum = eq(1) + eq(2) * cu    + eq(3) * c2u
     1       + eq(4) * c3u + eq(5) * c4u + eq(6) * c5u
      fqsum = fq(1) * su + fq(2) * s2u + fq(3) * s3u
     1       + fq(4) * s4u + gq*u + hq
c.................................................................
c sum up Liquid contributions
c.................................................................
      fliq1 = asum + bsum
      fliq160  = esum + fsum
      gliq1 = isum + jsum
      gliq160  = psum + qsum

      fliq    = v * fliq1 + (1. - v) * fliq160
      gliq    = w * gliq1 + (1. - w) * gliq160
c.................................................................
c sum up Quantum correction contributions
c.................................................................
      rquan1 = aqsum + bqsum
      rquan160 = eqsum + fqsum

      delfof = (1. - vq)*rquan1 + vq * rquan160
      delgog = (1. - vq)*rquan1 + vq * rquan160
c      write(7,1001) t,dens,gamma,fliq,delfof,gliq
c1001  format(1x,'He',6(1x,1pe12.5))
c.................................................................
c sum up for total neutrino loss contributions in solid phase
c.................................................................
      ftot = (1. - alfa * delfof) * fliq
      gtot = (1. - alfa * delgog) * gliq
c.................................................................
c compute neutrino loss rate
c.................................................................
      first = 0.5*(cv*cv + ca*ca + en*cvp*cvp + en*cap*cap)*ftot
      sec = 0.5*(cv*cv - ca*ca + en*cvp*cvp - en*cap*cap)*gtot
      qbremlh = 0.5738*(z*z/at)*(t8**6)*(first - sec)

      return
      end 
c***************************************************************************
c This subroutine returns the bremsstrahlung neutrino rates computed for 
c Liquid Carbon using the results of Itoh, Kohyama, Matsumoto, & Seki 
c 1984, ApJ, 280, 787 and Itoh & Kohyama 1983, ApJ, 275, 858
c These formulae include liquid metal and low temp. quantum correction conts.
c***************************************************************************
      subroutine bremliqc(dens,t,gamma,qbremlc)
      implicit double precision (a-z)
c......................................................
c  declare variables (find the values in the block data piece of code).
c......................................................
      common/liqdatc/ a(6), b(4), e(6), f(4), i(6), j(4), p(6), q(4),
     1          alpha(4), beta(4), c, d, g, h, k, l, r, s
      common/quandatc/ aq(6), bq(4), eq(6), fq(4),
     1          alphaq(5), cq, dq, gq, hq
c...........................................................
      at = 12.
      z = 6.
      twopi = 6.283185308
      pisqr = 9.869604404
      en = 2.
      s2theta = 0.217
      cv = 0.5 + 2.*s2theta
      ca = 0.5
      cvp = 1. - cv
      cap = 1. - ca
      x13 = -1./3.
      xone3 = 1./3.
      x14 = -0.25000000
      x23 = -2./3.
      x24 = -0.50000000
      x34 = -0.75000000
      x43 = 4./3.
      t8 = 10.**t / 1.e+08
      d6 = 10.**dens / 1.e+06 
      alfa = 0.07279 * gamma * (d6**xone3) / ((z*at)**x43)
c...............................................................
c compute the interpolation coefficients v and w.
c NOTE: u is defined differently here than in BREMSOL*.f
c...............................................................
      u = twopi * ( dens - 3. ) / 10.
c liquid variables
      v = alpha(1) + alpha(2) * gamma**x13 + alpha(3) * gamma**x23
     1    + alpha(4) / gamma
      w = beta(1) + beta(2) * gamma**x13 + beta(3) * gamma**x23
     1    + beta(4) / gamma
c quantum correction variables
      vq = alphaq(1) + alphaq(2)*gamma**x14 + alphaq(3)*gamma**x24
     1    + alphaq(4)*gamma**x34 + alphaq(5) / gamma
c      write(7,1000) alfa,u,v,w,vq
c1000  format(10x,'Alpha, U, V, W, V quan = ',5(1x,1pe12.5))
c...............................................................
c set up the variables (sines and cosines) for the loops
c...............................................................
      su  = dsin(u)
      su2 = su * su 
      cu2 = 1. - su2
      cu  = dsqrt( cu2 )
      s2u = 2. * su * cu
      c2u = cu2 - su2
      s3u = su * ( 3. - 4. * su2 )
      c3u = cu * ( 4. * cu2 - 3. )
      c4u = -8. * cu2 * su2 + 1.
      s4u = su * ( (2.*c3u) + (2.*cu) )
      c5u = cu * ( (16.*cu2*cu2) - (20.*cu2) + 5. )
c...............................................................
c  Liquid contribution sums
c...............................................................
      asum = a(1) + a(2) * cu + a(3) * c2u
     1            + a(4) * c3u + a(5) * c4u + a(6) * c5u
      bsum = b(1) * su + b(2) * s2u + b(3) * s3u
     1       + b(4) * s4u + c*u + d
      esum = e(1) + e(2) * cu + e(3) * c2u
     1            + e(4) * c3u + e(5) * c4u + e(6) * c5u
      fsum = f(1) * su + f(2) * s2u + f(3) * s3u
     1       + f(4) * s4u + g*u + h
      isum = i(1) + i(2) * cu + i(3) * c2u
     1            + i(4) * c3u + i(5) * c4u + i(6) * c5u
      jsum = j(1) * su + j(2) * s2u + j(3) * s3u
     1       + j(4) * s4u + k*u + l
      psum = p(1) + p(2) * cu + p(3) * c2u
     1            + p(4) * c3u + p(5) * c4u + p(6) * c5u
      qsum = q(1) * su + q(2) * s2u + q(3) * s3u
     1       + q(4) * s4u + r*u + s
c      write(7,1002) asum,bsum,esum,fsum
c1002  format(10x,'Carbon F factors, A, B, E, F = ',4(2x,1pe12.5))
c      write(7,1003) isum,jsum,psum,qsum
c1003  format(10x,'Carbon G factors, I, J, P, Q = ',4(2x,1pe12.5))
c  Quantum correction contributions
      aqsum = aq(1) + aq(2) * cu + aq(3) * c2u
     1       + aq(4) * c3u + aq(5) * c4u + aq(6) * c5u
      bqsum = bq(1) * su + bq(2) * s2u + bq(3) * s3u
     1       + bq(4) * s4u + cq*u + dq
      eqsum = eq(1) + eq(2) * cu    + eq(3) * c2u
     1       + eq(4) * c3u + eq(5) * c4u + eq(6) * c5u
      fqsum = fq(1) * su + fq(2) * s2u + fq(3) * s3u
     1       + fq(4) * s4u + gq*u + hq
c      write(7,1004) aqsum,bqsum,eqsum,fqsum
c1004  format(10x,'Carbon F quan factors, A, B, E, F = ',4(2x,1pe12.5))
c...............................................................
c sum up Liquid contributions
c...............................................................
      fliq1 = asum + bsum
      fliq160  = esum + fsum
      gliq1 = isum + jsum
      gliq160  = psum + qsum

      fliq    = v * fliq1 + (1. - v) * fliq160
      gliq    = w * gliq1 + (1. - w) * gliq160
c...............................................................
c sum up Quantum correction contributions
c...............................................................
      rquan1 = aqsum + bqsum
      rquan160 = eqsum + fqsum

      delfof = (1. - vq)*rquan1 + vq * rquan160
      delgog = (1. - vq)*rquan1 + vq * rquan160
c      write(7,1100)
c1100  format(6x,'log T',7x,'log Rho',7x,'Gamma',9x,'F',
c     111x,'dF/F',10x,'G')
c      write(7,1001) t,dens,gamma,fliq,delfof,gliq
c1001  format(6(1x,1pe12.5))
c      write(7,1101) vq,rquan1,rquan160
c1101  format(5x,'Vq, Rquan1, Rquan160 = ',3(1x,1pe12.5))
c.................................................................
c sum up for total neutrino loss contributions in liquid phase
c.................................................................
      ftot = (1. - alfa * delfof) * fliq
      gtot = (1. - alfa * delgog) * gliq
c.................................................................
c compute neutrino loss rate
c...............................................................
      first = 0.5*(cv*cv + ca*ca + en*cvp*cvp + en*cap*cap)*ftot
      sec = 0.5*(cv*cv - ca*ca + en*cvp*cvp - en*cap*cap)*gtot
      qbremlc = 0.5738*(z*z/at)*(t8**6)*(first - sec)
c      write(7,1006) first,sec,qbremlc
c1006  format(5x,'BremLiqC = ',3(1x,1pe12.5))

      return
      end 
c***************************************************************************
c This subroutine returns the bremsstrahlung neutrino rates computed for 
c Liquid Oxygen using the results of Itoh, Kohyama, Matsumoto, & Seki 
c 1984, ApJ, 280, 787 and Itoh & Kohyama 1983, ApJ, 275, 858
c These formulae include liquid metal and low temp. quantum correction conts.
c***************************************************************************
      subroutine bremliqo(dens,t,gamma,qbremlo)
      implicit double precision (a-z)
c......................................................
c  declare variables (find the values in the block data piece of code).
c......................................................
      common/liqdato/ a(6), b(4), e(6), f(4), i(6), j(4), p(6), q(4),
     1          alpha(4), beta(4), c, d, g, h, k, l, r, s
      common/quandato/ aq(6), bq(4), eq(6), fq(4),
     1          alphaq(5), cq, dq, gq, hq
c...........................................................
      at = 16.
      z = 8.
      twopi = 6.283185308
      pisqr = 9.869604404
      en = 2.
      s2theta = 0.217
      cv = 0.5 + 2.*s2theta
      ca = 0.5
      cvp = 1. - cv
      cap = 1. - ca
      x13 = -1./3.
      xone3 = 1./3.
      x14 = -0.25000000
      x23 = -2./3.
      x24 = -0.50000000
      x34 = -0.75000000
      x43 = 4./3.
      t8 = 10.**t / 1.e+08
      d6 = 10.**dens / 1.e+06 
      alfa = 0.07279 * gamma * (d6**xone3) / ((z*at)**x43)
c...........................................................
c compute the interpolation coefficients v and w.
c NOTE: u is defined differently here than in BREMSOL*.f
c...........................................................
      u = twopi * ( dens - 3. ) / 10.
c liquid variables
      v = alpha(1) + alpha(2) * gamma**x13 + alpha(3) * gamma**x23
     1    + alpha(4) / gamma
      w = beta(1) + beta(2) * gamma**x13 + beta(3) * gamma**x23
     1    + beta(4) / gamma
c quantum correction variables
      vq = alphaq(1) + alphaq(2)*gamma**x14 + alphaq(3)*gamma**x24
     1    + alphaq(4)*gamma**x34 + alphaq(5) / gamma
c...........................................................
c set up the variables (sines and cosines) for the loops
c...........................................................
      su  = dsin(u)
      su2 = su * su 
      cu2 = 1. - su2
      cu  = dsqrt( cu2 )
      s2u = 2. * su * cu
      c2u = cu2 - su2
      s3u = su * ( 3. - 4. * su2 )
      c3u = cu * ( 4. * cu2 - 3. )
      c4u = -8. * cu2 * su2 + 1.
      s4u = su * ( (2.*c3u) + (2.*cu) )
      c5u = cu * ( (16.*cu2*cu2) - (20.*cu2) + 5. )
c...........................................................
c  Liquid contributions
c...........................................................
      asum = a(1) + a(2) * cu + a(3) * c2u
     1            + a(4) * c3u + a(5) * c4u + a(6) * c5u
      bsum = b(1) * su + b(2) * s2u + b(3) * s3u
     1       + b(4) * s4u + c*u + d
      esum = e(1) + e(2) * cu + e(3) * c2u
     1            + e(4) * c3u + e(5) * c4u + e(6) * c5u
      fsum = f(1) * su + f(2) * s2u + f(3) * s3u
     1       + f(4) * s4u + g*u + h
      isum = i(1) + i(2) * cu + i(3) * c2u
     1            + i(4) * c3u + i(5) * c4u + i(6) * c5u
      jsum = j(1) * su + j(2) * s2u + j(3) * s3u
     1       + j(4) * s4u + k*u + l
      psum = p(1) + p(2) * cu + p(3) * c2u
     1            + p(4) * c3u + p(5) * c4u + p(6) * c5u
      qsum = q(1) * su + q(2) * s2u + q(3) * s3u
     1       + q(4) * s4u + r*u + s
c  Quantum correction contributions
      aqsum = aq(1) + aq(2) * cu + aq(3) * c2u
     1       + aq(4) * c3u + aq(5) * c4u + aq(6) * c5u
      bqsum = bq(1) * su + bq(2) * s2u + bq(3) * s3u
     1       + bq(4) * s4u + cq*u + dq
      eqsum = eq(1) + eq(2) * cu    + eq(3) * c2u
     1       + eq(4) * c3u + eq(5) * c4u + eq(6) * c5u
      fqsum = fq(1) * su + fq(2) * s2u + fq(3) * s3u
     1       + fq(4) * s4u + gq*u + hq
c...........................................................
c sum up Liquid contributions
c...........................................................
      fliq1 = asum + bsum
      fliq160  = esum + fsum
      gliq1 = isum + jsum
      gliq160  = psum + qsum

      fliq    = v * fliq1 + (1. - v) * fliq160
      gliq    = w * gliq1 + (1. - w) * gliq160
c...........................................................
c sum up Quantum correction contributions
c...........................................................
      rquan1 = aqsum + bqsum
      rquan160 = eqsum + fqsum

      delfof = (1. - vq)*rquan1 + vq * rquan160
      delgog = (1. - vq)*rquan1 + vq * rquan160
c...........................................................
c sum up for total neutrino loss contributions in liquid phase
c...........................................................
      ftot = (1. - alfa * delfof) * fliq
      gtot = (1. - alfa * delgog) * gliq
c...........................................................
c compute neutrino loss rate
c...........................................................
      first = 0.5*(cv*cv + ca*ca + en*cvp*cvp + en*cap*cap)*ftot
      sec = 0.5*(cv*cv - ca*ca + en*cvp*cvp - en*cap*cap)*gtot
      qbremlo = 0.5738*(z*z/at)*(t8**6)*(first - sec)

      return
      end 
c***************************************************************************
c This subroutine returns the bremsstrahlung neutrino rates computed for 
c Liquid Iron using the results of Itoh, Kohyama, Matsumoto, & Seki 
c 1984, ApJ, 280, 787 and Itoh & Kohyama 1983, ApJ, 275, 858
c These formulae include liquid metal and low temp. quantum correction conts.
c***************************************************************************
      subroutine bremliqf(dens,t,gamma,qbremlf)
      implicit double precision (a-z)
c......................................................
c  declare variables (find the values in the block data piece of code).
c......................................................
      common/liqdatf/ a(6), b(4), e(6), f(4), i(6), j(4), p(6), q(4),
     1          alpha(4), beta(4), c, d, g, h, k, l, r, s
      common/quandatf/ aq(6), bq(4), eq(6), fq(4),
     1          alphaq(5), cq, dq, gq, hq
c...........................................................
      at = 56.
      z = 26.
      twopi = 6.283185308
      pisqr = 9.869604404
      en = 2.
      s2theta = 0.217
      cv = 0.5 + 2.*s2theta
      ca = 0.5
      cvp = 1. - cv
      cap = 1. - ca
      x13 = -1./3.
      xone3 = 1./3.
      x14 = -0.25000000
      x23 = -2./3.
      x24 = -0.50000000
      x34 = -0.75000000
      x43 = 4./3.
      t8 = 10.**t / 1.d+08
      d6 = 10.**dens / 1.d+06 
      alfa = 0.07279 * gamma * (d6**xone3) / ((z*at)**x43)
c...........................................................
c compute the interpolation coefficients v and w.
c NOTE: u is defined differently here than in BREMSOL*.f
c...........................................................
      u = twopi * ( dens - 3. ) / 10.
c liquid variables
      v = alpha(1) + alpha(2) * gamma**x13 + alpha(3) * gamma**x23
     1    + alpha(4) / gamma
      w = beta(1) + beta(2) * gamma**x13 + beta(3) * gamma**x23
     1    + beta(4) / gamma
c quantum correction variables
      vq = alphaq(1) + alphaq(2)*gamma**x14 + alphaq(3)*gamma**x24
     1    + alphaq(4)*gamma**x34 + alphaq(5) / gamma
c...........................................................
c set up the variables (sines and cosines) for the loops
c...........................................................
      su  = dsin(u)
      su2 = su * su 
      cu2 = 1. - su2
      cu  = dsqrt( cu2 )
      s2u = 2. * su * cu
      c2u = cu2 - su2
      s3u = su * ( 3. - 4. * su2 )
      c3u = cu * ( 4. * cu2 - 3. )
      c4u = -8. * cu2 * su2 + 1.
      s4u = su * ( (2.*c3u) + (2.*cu) )
      c5u = cu * ( (16.*cu2*cu2) - (20.*cu2) + 5. )
c...........................................................
c  Liquid contributions
c...........................................................
      asum = a(1) + a(2) * cu + a(3) * c2u
     1            + a(4) * c3u + a(5) * c4u + a(6) * c5u
      bsum = b(1) * su + b(2) * s2u + b(3) * s3u
     1       + b(4) * s4u + c*u + d
      esum = e(1) + e(2) * cu + e(3) * c2u
     1            + e(4) * c3u + e(5) * c4u + e(6) * c5u
      fsum = f(1) * su + f(2) * s2u + f(3) * s3u
     1       + f(4) * s4u + g*u + h
      isum = i(1) + i(2) * cu + i(3) * c2u
     1            + i(4) * c3u + i(5) * c4u + i(6) * c5u
      jsum = j(1) * su + j(2) * s2u + j(3) * s3u
     1       + j(4) * s4u + k*u + l
      psum = p(1) + p(2) * cu + p(3) * c2u
     1            + p(4) * c3u + p(5) * c4u + p(6) * c5u
      qsum = q(1) * su + q(2) * s2u + q(3) * s3u
     1       + q(4) * s4u + r*u + s
c      write(7,1002) asum,bsum,esum,fsum
c1002  format(10x,'Iron F factors, A, B, E, F = ',4(2x,1pe12.5))
c      write(7,1003) isum,jsum,psum,qsum
c1003  format(10x,'Iron G factors, I, J, P, Q = ',4(2x,1pe12.5))
c  Quantum correction contributions
      aqsum = aq(1) + aq(2) * cu + aq(3) * c2u
     1       + aq(4) * c3u + aq(5) * c4u + aq(6) * c5u
      bqsum = bq(1) * su + bq(2) * s2u + bq(3) * s3u
     1       + bq(4) * s4u + cq*u + dq
      eqsum = eq(1) + eq(2) * cu    + eq(3) * c2u
     1       + eq(4) * c3u + eq(5) * c4u + eq(6) * c5u
      fqsum = fq(1) * su + fq(2) * s2u + fq(3) * s3u
     1       + fq(4) * s4u + gq*u + hq
c      write(7,1004) aqsum,bqsum,eqsum,fqsum
c1004  format(10x,'Iron F Quan factors, A, B, E, F = ',4(2x,1pe12.5))
c...........................................................
c sum up Liquid contributions
c...........................................................
      fliq1    = asum + bsum
      fliq160  = esum + fsum
      gliq1    = isum + jsum
      gliq160  = psum + qsum

      fliq    = v * fliq1 + (1. - v) * fliq160
      gliq    = w * gliq1 + (1. - w) * gliq160
c...........................................................
c sum up Quantum correction contributions
c...........................................................
      rquan1 = aqsum + bqsum
      rquan160 = eqsum + fqsum

      delfof = (1. - vq)*rquan1 + vq * rquan160
      delgog = (1. - vq)*rquan1 + vq * rquan160
c      write(7,1100)
c1100  format(10x,'log T',7x,'log Rho',7x,'Gamma',9x,'F',
c     111x,'dF/F',10x,'G')
c      write(7,1005) t,dens,gamma,fliq,delfof,gliq
c1005  format('Iron',6(1x,1pe12.5))
c...........................................................
c sum up for total neutrino loss contributions in liquid phase
c...........................................................
      ftot = (1. - alfa * delfof) * fliq
      gtot = (1. - alfa * delgog) * gliq
c...........................................................
c compute neutrino loss rate
c...........................................................
      first = 0.5*(cv*cv + ca*ca + en*cvp*cvp + en*cap*cap)*ftot
      sec = 0.5*(cv*cv - ca*ca + en*cvp*cvp - en*cap*cap)*gtot
      qbremlf = 0.5738*(z*z/at)*(t8**6)*(first - sec)

      return
      end
c..........................................................................
c this subroutine returns the neutrino pair bremsstrahlung rxn rates
c for partially degenerate electrons for the liquid metal regime.
c From Munkata, Kohyama, & Itoh 1987, ApJ, 316, 708.
c Valid in the range:  Gamma 1 to 178. (DO NOT use below Gamma = 1)
c                      Temp  10**8 to 10**11
c                      Dens  1     to 10**12
c Z and AT are only places where composition dependence shows up.
c..........................................................................
      subroutine brempde(dens,t,gamma,qbremde)
      implicit double precision (a-z)
      common/comp/amhyhe,amheca,x(4)
      dimension a(6),b(5)
c...........................................................
      t8 = 10.**(t - 8.0)
      d = 10.**(dens)
      at = 1. /(x(2)*4. + x(3)*12. + x(4)*16.)
      z = (x(2)*2. + x(3)*6. + x(4)*8.)
c      at = 1. /(x(2)*4. + x(3)*12. + x(4)*16. + x(5)*56.)
c      z = (x(2)*2. + x(3)*6. + x(4)*8. + x(5)*26.)
      en = 2.
      s2theta = 0.217
      cv = 0.5 + 2.*s2theta
      ca = 0.5
      cvp = 1. - cv
      cap = 1. - ca
c...........................................................
c load coefficient arrays. NOTE: a(1) = a_0 etc.
c...........................................................
      a(1) = 23.5 
      a(2) = 6.83e+04
      a(3) = 7.81e+04
      a(4) = 230.
      a(5) = 6.70e+05
      a(6) = 7.66e+09
      b(1) = 1.47
      b(2) = 0.0329
      b(3) = (7.75e+05*t8**1.5) + (247.*t8**3.85)
      b(4) = 4.07 + (0.0240*t8**1.40)
      b(5) = 4.59e-05*t8**-0.110
c...........................................................
c crank it out.
c...........................................................
      xi = d / ((7.05e+06*t8**1.5) + (5.12e+04*t8**3))
      x2 = xi*xi
 
      f1 = 1./(a(1)+(a(2)/t8**2)+(a(3)/t8**5))
      f2 = 1.26*(1.+(1./xi))/(1.+(b(1)/xi)+(b(2)/x2))
      fsum = f1 + f2
      g1 = (1. + 1.e-9*d)*(a(4)+(a(5)/t8**2))+(a(6)/t8**5)
      g2 = (b(3)/d) + b(4) + b(5)*d**0.656
      gsum = (1./g1) + (1./g2)
 
      first = 0.5*(cv*cv + ca*ca + en*cvp*cvp + en*cap*cap)*fsum
      sec = 0.5*(cv*cv - ca*ca + en*cvp*cvp - en*cap*cap)*gsum
      qbremde = 0.5738*(z*z/at)*(t8**6)*(first - sec)

      return
      end 
c***************************************************************************
c This routine returns the bremsstrahlung neutrino rates computed for 
c Solid Carbon using the results of 
c Itoh, Kohyama, Matsumoto, & Seki 1984, ApJ, 285, 304 
c (+Errata 1987, ApJ, 322, 584)
c These formulae include lattice and phonon (lattice vibration) contributions
c***************************************************************************
      subroutine bremsolc(dens,t,gamma,qbremsc)
      implicit double precision (a-z)
c......................................................
c  declare variables (find the values in the block data piece of code).
c......................................................
      common/soldatc/ a(5), b(3), e(5), f(3), i(5), j(3), p(5), q(3),
     1          alpha(4), beta(4), c, d, g, h, k, l, r, s
      common/phondatc/ ap(5), bp(3), ip(5), jp(3),
     1          alphap(4), betap(4), cp, dp, kp, lp
c...........................................................
      at = 12.
      z = 6.
      twopi = 6.283185308
      pisqr = 9.869604404
      en = 2.
      s2theta = 0.217
      cv = 0.5 + 2.*s2theta
      ca = 0.5
      cvp = 1. - cv
      cap = 1. - ca
      x13 = -1./3.
      x23 = -2./3.
      t8 = 10.**t / 1.e+08
c...........................................................
c compute the interpolation coefficients v and w
c...........................................................
      u = twopi * ( dens - 3. ) / 9.
c lattice variables
c If gamma>5000., set to 5000. for the lattice results.
      if(gamma.gt.5000.)then
        gamma2 = 5000.
       else
        gamma2 = gamma
      endif
      v = alpha(1) + alpha(2) * gamma2**x13 + alpha(3) * gamma2**x23
     1    + alpha(4) / gamma2
      w = beta(1) + beta(2) * gamma2**x13 + beta(3) * gamma2**x23
     1    + beta(4) / gamma2
c phonon variables
      vp = alphap(1) + alphap(2)*gamma**x13 + alphap(3)*gamma**x23
     1    + alphap(4) / gamma
      wp = betap(1) + betap(2)*gamma**x13 + betap(3)*gamma**x23
     1    + betap(4) / gamma
c...........................................................
c set up the variables (sines and cosines) for the loops
c...........................................................
      su  = dsin(u)
      su2 = su * su 
      cu2 = 1. - su2
      cu  = dsqrt( cu2 )
      s2u = 2. * su * cu
      c2u = cu2 - su2
      s3u = su * ( 3. - 4. * su2 )
      c3u = cu * ( 4. * cu2 - 3. )
      c4u = -8. * cu2 * su2 + 1.
c...........................................................
c  Lattice contributions
c...........................................................
      asum = a(1) + a(2) * cu    + a(3) * c2u
     1            + a(4) * c3u + a(5) * c4u
      bsum = b(1) * su + b(2) * s2u + b(3) * s3u
     1       + c*u + d
      esum = e(1) + e(2) * cu    + e(3) * c2u
     1            + e(4) * c3u + e(5) * c4u
      fsum = f(1) * su + f(2) * s2u + f(3) * s3u
     1       + g*u + h
      isum = i(1) + i(2) * cu    + i(3) * c2u
     1            + i(4) * c3u + i(5) * c4u
      jsum = j(1) * su + j(2) * s2u + j(3) * s3u
     1       + k*u + l
      psum = p(1) + p(2) * cu    + p(3) * c2u
     1            + p(4) * c3u + p(5) * c4u
      qsum = q(1) * su + q(2) * s2u + q(3) * s3u
     1       + r*u + s
c  Phonon contributions
      apsum = ap(1) + ap(2)*cu + ap(3)*c2u
     1       + ap(4)*c3u + ap(5)*c4u
      bpsum = bp(1)*su + bp(2)*s2u + bp(3)*s3u
     1       + cp*u + dp
      ipsum = ip(1) + ip(2)*cu + ip(3)*c2u
     1       + ip(4)*c3u + ip(5)*c4u
      jpsum = jp(1)*su + jp(2)*s2u + jp(3)*s3u
     1       + kp*u + lp
c...........................................................
c sum up Lattice contributions
c...........................................................
      flat171 = asum + bsum
      flat5k  = esum + fsum
      glat171 = isum + jsum
      glat5k  = psum + qsum
      flat    = (1.-v) * flat171 + v * flat5k
      glat    = (1.-w) * glat171 + w * glat5k
c...........................................................
c sum up Phonon contributions
c We just extrapolate this is gamma > 5000.
c...........................................................
      fplat171 = vp*(apsum + bpsum)
      gplat171 = wp*(ipsum + jpsum)
c...........................................................
c sum up for total neutrino loss contributions in solid phase
c...........................................................
      ftot = flat + fplat171
      gtot = glat + gplat171
c...........................................................
c compute neutrino loss rate
c...........................................................
      first = 0.5*(cv*cv + ca*ca + en*cvp*cvp + en*cap*cap)*ftot
      sec = 0.5*(cv*cv - ca*ca + en*cvp*cvp - en*cap*cap)*gtot
      qbremsc = 0.5738*(z*z/at)*(t8**6)*(first - sec)

      return
      end 
c***************************************************************************
c This routine returns the bremsstrahlung neutrino rates computed for 
c Solid Helium using the results of 
c Itoh, Kohyama, Matsumoto, & Seki 1984, ApJ, 285, 304 
c (+Errata 1987, ApJ, 322, 584)
c These formulae include lattice and phonon (lattice vibration) contributions
c***************************************************************************
      subroutine bremsolh(dens,t,gamma,qbremsh)
      implicit double precision (a-z)
c......................................................
c  declare variables (find the values in the block data piece of code).
c......................................................
      common/soldath/ a(5), b(3), e(5), f(3), i(5), j(3), p(5), q(3),
     1          alpha(4), beta(4), c, d, g, h, k, l, r, s
      common/phondath/ ap(5), bp(3), ip(5), jp(3),
     1          alphap(4), betap(4), cp, dp, kp, lp
c...........................................................
      at = 4.
      z = 2.
      twopi = 6.283185308
      pisqr = 9.869604404
      en = 2.
      s2theta = 0.217
      cv = 0.5 + 2.*s2theta
      ca = 0.5
      cvp = 1. - cv
      cap = 1. - ca
      x13 = -1./3.
      x23 = -2./3.
      t8 = 10.**t / 1.e+08
c..............................................................
c compute the interpolation coefficients v and w
c..............................................................
      u = twopi * ( dens - 3. ) / 9.
c lattice variables
c Note: if gamma>5000, set to 5000 for the lattice phase part.
      if(gamma.gt.5000.)then
        gamma2 = 5000.
       else
        gamma2 = gamma
      endif
      v = alpha(1) + alpha(2) * gamma2**x13 + alpha(3) * gamma2**x23
     1    + alpha(4) / gamma2
      w = beta(1) + beta(2) * gamma2**x13 + beta(3) * gamma2**x23
     1    + beta(4) / gamma2
c phonon variables
      vp = alphap(1) + alphap(2)*gamma**x13 + alphap(3)*gamma**x23
     1    + alphap(4) / gamma
      wp = betap(1) + betap(2)*gamma**x13 + betap(3)*gamma**x23
     1    + betap(4) / gamma
c..............................................................
c set up the variables (sines and cosines) for the loops
c..............................................................
      su  = dsin(u)
      su2 = su * su 
      cu2 = 1. - su2
      cu  = dsqrt( cu2 )
      s2u = 2. * su * cu
      c2u = cu2 - su2
      s3u = su * ( 3. - 4. * su2 )
      c3u = cu * ( 4. * cu2 - 3. )
      c4u = -8. * cu2 * su2 + 1.
c..............................................................
c  Lattice contributions
c..............................................................
      asum = a(1) + a(2) * cu    + a(3) * c2u
     1            + a(4) * c3u + a(5) * c4u
      bsum = b(1) * su + b(2) * s2u + b(3) * s3u
     1       + c*u + d
      esum = e(1) + e(2) * cu    + e(3) * c2u
     1            + e(4) * c3u + e(5) * c4u
      fsum = f(1) * su + f(2) * s2u + f(3) * s3u
     1       + g*u + h
      isum = i(1) + i(2) * cu    + i(3) * c2u
     1            + i(4) * c3u + i(5) * c4u
      jsum = j(1) * su + j(2) * s2u + j(3) * s3u
     1       + k*u + l
      psum = p(1) + p(2) * cu    + p(3) * c2u
     1            + p(4) * c3u + p(5) * c4u
      qsum = q(1) * su + q(2) * s2u + q(3) * s3u
     1       + r*u + s
c  Phonon contributions
      apsum = ap(1) + ap(2)*cu + ap(3)*c2u
     1       + ap(4)*c3u + ap(5)*c4u
      bpsum = bp(1)*su + bp(2)*s2u + bp(3)*s3u
     1       + cp*u + dp
      ipsum = ip(1) + ip(2)*cu + ip(3)*c2u
     1       + ip(4)*c3u + ip(5)*c4u
      jpsum = jp(1)*su + jp(2)*s2u + jp(3)*s3u
     1       + kp*u + lp
c..............................................................
c sum up Lattice contributions
c..............................................................
      flat171 = asum + bsum
      flat5k  = esum + fsum
      glat171 = isum + jsum
      glat5k  = psum + qsum

      flat    = (1.-v) * flat171 + v * flat5k
      glat    = (1.-w) * glat171 + w * glat5k
c..............................................................
c sum up Phonon contributions
c We just extrapolate this is gamma > 5000.
c..............................................................
      fplat171 = vp*(apsum + bpsum)
      gplat171 = wp*(ipsum + jpsum)
c..............................................................
c sum up for total neutrino loss contributions in solid phase
c..............................................................
      ftot = flat + fplat171
      gtot = glat + gplat171
c..............................................................
c compute neutrino loss rate
c..............................................................
      first = 0.5*(cv*cv + ca*ca + en*cvp*cvp + en*cap*cap)*ftot
      sec = 0.5*(cv*cv - ca*ca + en*cvp*cvp - en*cap*cap)*gtot
      qbremsh = 0.5738*(z*z/at)*(t8**6)*(first - sec)

      return
      end 
c***************************************************************************
c This routine returns the bremsstrahlung neutrino rates computed for 
c Solid Oxygen using the results of 
c Itoh, Kohyama, Matsumoto, & Seki 1984, ApJ, 285, 304 
c (+Errata 1987, ApJ, 322, 584)
c Includes lattice and phonon (lattice vibrations) contributions.
c***************************************************************************
      subroutine bremsolo(dens,t,gamma,qbremso)
      implicit double precision (a-z)
c......................................................
c  declare variables (find the values in the block data piece of code).
c......................................................
      common/soldato/ a(5), b(3), e(5), f(3), i(5), j(3), p(5), q(3),
     1          alpha(4), beta(4), c, d, g, h, k, l, r, s
      common/phondato/ ap(5), bp(3), ip(5), jp(3),
     1          alphap(4), betap(4), cp, dp, kp, lp
c...........................................................
      at = 16.
      z = 8.
      twopi = 6.283185308
      pisqr = 9.869604404
      en = 2.
      s2theta = 0.217
      cv = 0.5 + 2.*s2theta
      ca = 0.5
      cvp = 1. - cv
      cap = 1. - ca
      x13 = -1./3.
      x23 = -2./3.
      t8 = 10.**t / 1.e+08
c...........................................................
c compute the interpolation coefficients v and w
c...........................................................
      u = twopi * ( dens - 3. ) / 9.
c lattice variables
c If gamma>5000., set to 5000. for lattice results.
      if(gamma.gt.5000.)then
        gamma2 = 5000.
       else
        gamma2 = gamma
      endif
      v = alpha(1) + alpha(2) * gamma2**x13 + alpha(3) * gamma2**x23
     1    + alpha(4) / gamma2
      w = beta(1) + beta(2) * gamma2**x13 + beta(3) * gamma2**x23
     1    + beta(4) / gamma2
c phonon variables
      vp = alphap(1) + alphap(2)*gamma**x13 + alphap(3)*gamma**x23
     1    + alphap(4) / gamma
      wp = betap(1) + betap(2)*gamma**x13 + betap(3)*gamma**x23
     1    + betap(4) / gamma
c...........................................................
c set up the variables (sines and cosines) for the loops
c...........................................................
      su  = dsin(u)
      su2 = su * su 
      cu2 = 1. - su2
      cu  = dsqrt( cu2 )
      s2u = 2. * su * cu
      c2u = cu2 - su2
      s3u = su * ( 3. - 4. * su2 )
      c3u = cu * ( 4. * cu2 - 3. )
      c4u = -8. * cu2 * su2 + 1.
c...........................................................
c  Lattice contributions
c...........................................................
      asum = a(1) + a(2) * cu    + a(3) * c2u
     1            + a(4) * c3u + a(5) * c4u
      bsum = b(1) * su + b(2) * s2u + b(3) * s3u
     1       + c*u + d
      esum = e(1) + e(2) * cu    + e(3) * c2u
     1            + e(4) * c3u + e(5) * c4u
      fsum = f(1) * su + f(2) * s2u + f(3) * s3u
     1       + g*u + h
      isum = i(1) + i(2) * cu    + i(3) * c2u
     1            + i(4) * c3u + i(5) * c4u
      jsum = j(1) * su + j(2) * s2u + j(3) * s3u
     1       + k*u + l
      psum = p(1) + p(2) * cu    + p(3) * c2u
     1            + p(4) * c3u + p(5) * c4u
      qsum = q(1) * su + q(2) * s2u + q(3) * s3u
     1       + r*u + s
c  Phonon contributions
      apsum = ap(1) + ap(2)*cu + ap(3)*c2u
     1       + ap(4)*c3u + ap(5)*c4u
      bpsum = bp(1)*su + bp(2)*s2u + bp(3)*s3u
     1       + cp*u + dp
      ipsum = ip(1) + ip(2)*cu + ip(3)*c2u
     1       + ip(4)*c3u + ip(5)*c4u
      jpsum = jp(1)*su + jp(2)*s2u + jp(3)*s3u
     1       + kp*u + lp
c...........................................................
c sum up Lattice contributions
c...........................................................
      flat171 = asum + bsum
      flat5k  = esum + fsum
      glat171 = isum + jsum
      glat5k  = psum + qsum
      flat    = (1.-v) * flat171 + v * flat5k
      glat    = (1.-w) * glat171 + w * glat5k
c...........................................................
c sum up Phonon contributions
c We just extrapolate this is gamma > 5000.
c...........................................................
      fplat171 = vp*(apsum + bpsum)
      gplat171 = wp*(ipsum + jpsum)
c...........................................................
c sum up for total neutrino loss contributions in solid phase
c...........................................................
      ftot = flat + fplat171
      gtot = glat + gplat171
c...........................................................
c compute neutrino loss rate
c...........................................................
      first = 0.5*(cv*cv + ca*ca + en*cvp*cvp + en*cap*cap)*ftot
      sec = 0.5*(cv*cv - ca*ca + en*cvp*cvp - en*cap*cap)*gtot
      qbremso = 0.5738*(z*z/at)*(t8**6)*(first - sec)

      return
      end 
c***************************************************************************
c This routine returns the bremsstrahlung neutrino rates computed for 
c Solid Iron using the results of 
c Itoh, Kohyama, Matsumoto, & Seki 1984, ApJ, 285, 304 
c (+Errata 1987, ApJ, 322, 584)
c Includes lattice and phonon (lattice vibrations) contributions.
c***************************************************************************
      subroutine bremsolf(dens,t,gamma,qbremsf)
      implicit double precision (a-z)
c......................................................
c  declare variables (find the values in the block data piece of code).
c......................................................
      common/soldatf/ a(5), b(3), e(5), f(3), i(5), j(3), p(5), q(3),
     1          alpha(4), beta(4), c, d, g, h, k, l, r, s
      common/phondatf/ ap(5), bp(3), ip(5), jp(3),
     1          alphap(4), betap(4), cp, dp, kp, lp
c...........................................................
      at = 56.
      z = 26.
      twopi = 6.283185308
      pisqr = 9.869604404
      en = 2.
      s2theta = 0.217
      cv = 0.5 + 2.*s2theta
      ca = 0.5
      cvp = 1. - cv
      cap = 1. - ca
      x13 = -1./3.
      x23 = -2./3.
      t8 = 10.**t / 1.e+08
c...........................................................
c compute the interpolation coefficients v and w
c...........................................................
      u = twopi * ( dens - 3. ) / 9.
c lattice variables
c If gamma>5000., set to 5000. for lattice results.
      if(gamma.gt.5000.)then
        gamma2 = 5000.
       else
        gamma2 = gamma
      endif
      v = alpha(1) + alpha(2) * gamma2**x13 + alpha(3) * gamma2**x23
     1    + alpha(4) / gamma2
      w = beta(1) + beta(2) * gamma2**x13 + beta(3) * gamma2**x23
     1    + beta(4) / gamma2
c phonon variables
      vp = alphap(1) + alphap(2)*gamma**x13 + alphap(3)*gamma**x23
     1    + alphap(4) / gamma
      wp = betap(1) + betap(2)*gamma**x13 + betap(3)*gamma**x23
     1    + betap(4) / gamma
c...........................................................
c set up the variables (sines and cosines) for the loops
c...........................................................
      su  = dsin(u)
      su2 = su * su 
      cu2 = 1. - su2
      cu  = dsqrt( cu2 )
      s2u = 2. * su * cu
      c2u = cu2 - su2
      s3u = su * ( 3. - 4. * su2 )
      c3u = cu * ( 4. * cu2 - 3. )
      c4u = -8. * cu2 * su2 + 1.
c...........................................................
c  Lattice contributions
c...........................................................
      asum = a(1) + a(2) * cu    + a(3) * c2u
     1            + a(4) * c3u + a(5) * c4u
      bsum = b(1) * su + b(2) * s2u + b(3) * s3u
     1       + c*u + d
      esum = e(1) + e(2) * cu    + e(3) * c2u
     1            + e(4) * c3u + e(5) * c4u
      fsum = f(1) * su + f(2) * s2u + f(3) * s3u
     1       + g*u + h
      isum = i(1) + i(2) * cu    + i(3) * c2u
     1            + i(4) * c3u + i(5) * c4u
      jsum = j(1) * su + j(2) * s2u + j(3) * s3u
     1       + k*u + l
      psum = p(1) + p(2) * cu    + p(3) * c2u
     1            + p(4) * c3u + p(5) * c4u
      qsum = q(1) * su + q(2) * s2u + q(3) * s3u
     1       + r*u + s
c  Phonon contributions
      apsum = ap(1) + ap(2)*cu + ap(3)*c2u
     1       + ap(4)*c3u + ap(5)*c4u
      bpsum = bp(1)*su + bp(2)*s2u + bp(3)*s3u
     1       + cp*u + dp
      ipsum = ip(1) + ip(2)*cu + ip(3)*c2u
     1       + ip(4)*c3u + ip(5)*c4u
      jpsum = jp(1)*su + jp(2)*s2u + jp(3)*s3u
     1       + kp*u + lp
c...........................................................
c sum up Lattice contributions
c...........................................................
      flat171 = asum + bsum
      flat5k  = esum + fsum
      glat171 = isum + jsum
      glat5k  = psum + qsum
      flat    = (1.-v) * flat171 + v * flat5k
      glat    = (1.-w) * glat171 + w * glat5k
c...........................................................
c sum up Phonon contributions
c We just extrapolate this is gamma > 5000.
c...........................................................
      fplat171 = vp*(apsum + bpsum)
      gplat171 = wp*(ipsum + jpsum)
c...........................................................
c sum up for total neutrino loss contributions in solid phase
c...........................................................
      ftot = flat + fplat171
      gtot = glat + gplat171
c...........................................................
c compute neutrino loss rate
c...........................................................
      first = 0.5*(cv*cv + ca*ca + en*cvp*cvp + en*cap*cap)*ftot
      sec = 0.5*(cv*cv - ca*ca + en*cvp*cvp - en*cap*cap)*gtot
      qbremsf = 0.5738*(z*z/at)*(t8**6)*(first - sec)

      return
      end 
c**************************************************************************
c get Hubbard-Lampe opacities for the appropriate composition
c**************************************************************************
      function oconhl(d,t)
      implicit double precision(a-h,o-z)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
c.........................................................................
      if ( je .eq. 3 ) then
         polytmp = hlcnc(d,t)
      elseif ( je .eq. 2 ) then
         polytmp = hlcnhe(d,t)
      elseif ( je .eq. 1 ) then
         polytmp = hlcnh(d,t)
      endif
c.........................................................................
      oconhl = polytmp
      return
      end 
c**************************************************************************
c  compute the conductive opacities using the method of itoh et al.
c  see top of program for references.
c**************************************************************************
      function oconit(d,t)
      implicit double precision(a-h,o-z)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      common/temp/s1,r1,s2,r2,b2,p2,t2,e2,xc2,xo2,fca2,f2,q2,w2,c
      data ac43, alac43 /3.025600e-04, -3.519188/ 
c.....................................................................
c  gamma constants
c.....................................................................
      data gamc3c, gamc3o, gamc2, gamc1 
     1     /3.5772e+06, 5.7782e+06, 573264., 2.275e+05/
c.....................................................................
c compute gamma, and call the appropriate routine (liquid metal or
c crystalline).  if gamma > 5000, then set gamma = 5000.
c This is the suggested proceedure from itoh et al (1984), since
c the results barely change when gamma > 5000.
c NOTE: there is no fit to xtal hydrogen.
c.....................................................................
      if ( je .eq. 3 ) then
         gammac = 10.**(d/3. - t) * gamc3c
         gammao = gammac / gamc3c * gamc3o
         gamma = xc2 * gammac + xo2 * gammao
c.........................................................................
c Carbon contribution
c.........................................................................
         if ( xc2 .gt. 0. ) then
            if ( gamma .lt. 171. ) then 
               condc = cndlc(d,t,gammac)
            elseif ( gamma .ge. 171 ) then
               if ( gammac .gt. 5000.) gammac = 5000.
               condc = cndsc(d,t,gammac)
            endif
         else
            condc = 0.
         endif
c.........................................................................
c Oxygen contribution
c.........................................................................
         if ( xo2 .gt. 0. ) then
            if ( gamma .lt. 171. ) then 
               condo = cndlo(d,t,gammao)
            elseif ( gamma .ge. 171 ) then
               if ( gammao .gt. 5000.) gammao = 5000.
               condo = cndso(d,t,gammao)
            endif
         else
            condo = 0.
         endif
         oconit = alac43 + (t*3. - d - (xc2*condc + xo2*condo) )
c.........................................................................
c Helium contribution
c.........................................................................
      elseif ( je .eq. 2 ) then
         gamma = gamc2 * 10.**(d/3. - t)
         if ( gamma .lt. 171. ) then
            conduct = cndlhe(d,t,gamma)
         elseif ( gamma .ge. 171 ) then 
            if ( gamma .gt. 5000.) gamma = 5000.
            conduct = cndshe(d,t,gamma)
         endif
         oconit = alac43 + (t*3. - d - conduct)
c.........................................................................
c Hydrogen contribution
c.........................................................................
      elseif ( je .eq. 1 ) then
         gamma = gamc1 * 10.**(d/3. - t)
         conduct = cndlhy(d,t,gamma)
         oconit = alac43 + (t*3. - d - conduct)
      endif
      return
      end 
c********************************************************************
c  compute the conductive opacity of a specific rho-t point. 
c  note:  in this version, the hubbard lampe opacities have been replaced
c  by the conductive opacities of itoh et al (see top of code for
c  references.)
c********************************************************************
      function ocon(d,t)
      implicit double precision(a-h,o-z)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
c..........................................................................
c  the itoh opacities are only good down to log(rho) = 1.6
c  therefore, maw put in this kludge where if the density is
c  less than 1.8 and greater than 1.5, the interpolate between
c  opacity tables.  if d < 1.5, then use hubbard-lampe conductive
c  opacities.
c..........................................................................
      if (d .ge. 1.8) then
         ocon = oconit(d,t)
      elseif ( d .ge. 1.5 ) then
         dfrac = (d - 1.5) / 0.3
         ocon = dfrac * oconit(d,t) + (1.-dfrac) * oconhl(d,t)
      else
         ocon = oconhl(d,t)
      endif
      if(dabs(ocon).gt.40.)then
        ocon=dsign(40.d0,ocon)
      endif
      return
      end 
c**************************************************************************
c  compute the total (rad. + cond.) opacity of a specific rho-t point. 
c  This function assumes a pure composition only. (See opac3 for mixtures.)
c  NOTE:  in this version, the Hubbard-Lampe opacities have been replaced
c  by the conductive opacities of itoh et al (see top of code for
c  references.)
c**************************************************************************
      function opac(d,t)
      implicit double precision(a-h,o-z)
      common/opacs/cso(3,29,8),csr(29),cst(29)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      dimension two(2)
      data jrow,krow/8,29/
c..........................................................................
c trap point on staggered rho,T grid
c..........................................................................
      j=1 
      k=krow
6     i=(j+k)/2
      if ( t - cst(i) ) 7,7,8 
7     k=i 
      go to 9
8     j=i 
9     if(iabs(j-k)-1) 10,10,6 
10    do 14 i=1,2
      l=k+i-2
      delta=d-csr(l)+1
      j=delta+1
      if(j-1) 11,11,12
11    j=2 
      go to 14
12    if(j-jrow) 14,14,13
13    j=jrow
14    two(i)=cso(je,l,j-1)+(cso(je,l,j)-cso(je,l,j-1))*(delta-j+1)
c..........................................................................
      orad=two(1)+(two(2)-two(1))*(t-cst(k-1))/(cst(k)-cst(k-1))
      if(dabs(orad).gt.40.) orad=dsign(40.d0,orad)
      ocond = ocon(d,t)
      opac=dlog10(1./(10.**(-orad)+10.**(-ocond)))
      return
      end 
c***************************************************************
c  opacities, but from Lagrange re-interpolator
c Call for each chemical species. If pure, use derivatives from
c within the routine. Otherwise, use 5 point differencing.
c See opac3 for further details.
c***************************************************************
      subroutine opacder(d,t,opcs,dlklr,dlklt)
      implicit double precision(a-h,o-z)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      common/opcswch/first
      logical first
      real*8 op(4),dkrx(4),ytb(4),xtb(4)
      real*8 ta(30),tb(30),ty(30,30,3) 
c***************************************************************
c read in the opacities if we haven't already
c***************************************************************
      if ( first ) then
        write(*,*) 'entering opac 1st time'
        first = .false.
        read(25,*) iaend,jbend
c        write(18,*) iaend,jbend
        read(25,705) (ta(li),li=1,iaend)
c        write(18,705) (ta(li), li=1,iaend)
        read(25,705) (tb(li),li=1,jbend)
c        write(18,705) (tb(li), li=1,jbend)
705     format(8f10.5)
710     format(10f8.4)
        do 1000 lk=1,3
        do 1000 li=1,iaend
        read(25,710) (ty(li,lj,lk),lj=1,jbend) 
c        write(18,710) (ty(li,lj,lk), lj=1,jbend)
1000    continue
        write(*,*) 'opacs read'
      endif
c locate our place on the grid.
      ider=1
      if(km.lt.1)then
        i = 9
        j = 7
        km = 0
      endif
65    continue
      km=km+1
c set temp = xa and dens = xb
      xa = t
      xb = d
c..........................................................................
c      write(17,*) 'T,rho,je,He,C',xa,xb,je,x(2),x(3)
      if( xa .lt. ta(1) ) go to 50
      if( xa .gt. ta(iaend)) go to 50
1     if ( xa - ta(i)) 4, 6, 2
2     if (xa - ta(i+1)) 6, 5, 3
3     i = i + 2
      go to 1
4     i = i - 1
      go to  1
5     i = i + 1
6     continue
c..........................................................................
      if( xb .lt. tb(1) ) go to 50
      if( xb .gt. tb(jbend)) go to 50
11    if (xb - tb(j)) 14, 16, 12
12    if(xb-tb(j+1)) 16, 15, 13
13    j = j + 2
      go to 11
14    j = j - 1
      go to 11
15    j = j + 1
16    continue
      itop=0
      it10=i-1
      if(it10 .ge. 1) go to 150
      it10=it10+1
      itop=1
150   continue
      if(ty(it10,j,je).gt.-7.99 .or. ty(it10,j+1,je).gt.-7.99) goto 160
      it10=it10+1
      itop=1
      if(it10 .gt.i) go to 400
      go to 150
160   it1f=it10+3
      if(it1f.le.iaend) goto 170
      if(itop .ne. 0) go to 400
      it10=it10-1
      goto 150
170   continue
      if(ty(it1f,j,je).gt.-7.99 .or. ty(it1f,j+1,je).gt.-7.99)goto 180
      if(itop .ne. 0) go to 400
      it10 = it10-1 
      if(it10.le. 0) go to 400
      if(it1f .le. i+1) go to 400
      go to 150
180   continue
c..........................................................................
      do 250 int=1,4
      it1=it10+int-1
      if(ty(it1,j,je).le.-7.99 .or. ty(it1,j+1,je).le.-7.99) go to 200
      js1=j-1
      if(js1 .le. 0) js1=js1+1
      if(ty(it1,js1,je) .le. -7.99) js1=js1+1
      if(js1+3 .gt. jbend) js1=js1-1
      if(ty(it1,js1+3,je) .le. -7.99) js1=js1-1
      do 190 k=1,4
      k1=js1+k-1
c add conductive opac. to rad. opac.
      ocond = ocon(tb(k1),ta(it1))
      ytb(k)=dlog10(1./(10.**(-ty(it1,k1,je))+10.**(-ocond)))
c      write(17,*) 'ytb',ytb(k),ty(it1,k1,je),ocond
190   xtb(k)=tb(k1) 
      call fh4(xb,ytb,xtb,op(int),dkrx(int),ider) 
c      write(17,*) 'op(int) go to 250',op(int)
      go to 250
c..........................................................................
200   if(ty(it1,j,je) .le. -7.99) js1=j+1
      if(ty(it1,j+1,je) .le. -7.99) js1=j-1
c add conductive opac. to rad. opac.
      ocond1 = ocon(tb(js1+1),ta(it1))
      ocond2 = ocon(tb(js1),ta(it1))
      op1=dlog10(1./(10.**(-ty(it1,js1+1,je))+10.**(-ocond1)))
      op2=dlog10(1./(10.**(-ty(it1,js1,je))+10.**(-ocond2)))
      dkrx(int)=(op1-op2)/(tb(js1+1)-tb(js1))
      op(int)=op1+dkrx(int)*(xb-tb(js1+1))
c      write(17,*) 'op bef 250',ocond1,op1,op2,op(int)
250   continue
      do 260 k=1,4
260   xtb(k)=ta(it10+k-1)
      call fh4(xa,op,xtb,opc,dlklt,ider)
      opcs=opc
      if(ider .ne. 1)then
        return
      endif
      call fh4(xa,dkrx,xtb,dlklr,dumfh,0)
      return
400   continue
c..........................................................................
      do 500 k=1,2
      it1=i+k-1
      if(ty(it1,j,je).le.-7.99 .and. ty(it1,j+1,je).le.-7.99) go to 450
      js1=j
      if(ty(it1,j,je).le.-7.99) js1=j+1
      if(ty(it1,j+1,je).le.-7.99) js1=j-1
      frxs=(xb-tb(js1+1))/(tb(js1+1)-tb(js1))
c add conductive opac. to rad. opac.
      ocond1 = ocon(tb(js1+1),ta(it1))
      ocond2 = ocon(tb(js1),ta(it1))
      op1=dlog10(1./(10.**(-ty(it1,js1+1,je))+10.**(-ocond1)))
      op2=dlog10(1./(10.**(-ty(it1,js1,je))+10.**(-ocond2)))
      op(k)=op1+(op1-op2)*frxs
c      write(17,*) 'op aft. 400',ty(it1,js1,je),ocond2
c      write(17,*) 'op aft. 400',t1,js1+1,op1,op2,op(k)
      if(ider .ne. 1) go to 500
c add conductive opac. to rad. opac.
      dop1=(op1-op2)/(tb(js1+1)-tb(js1))
      dop2=dop1
      if(js1-1 .le. 0) go to 420
      if(ty(it1,js1-1,je) .gt. -7.99) then
c add conductive opac. to rad. opac.
      ocond1 = ocon(tb(js1-1),ta(it1))
      ocond2 = ocon(tb(js1),ta(it1))
      op1=dlog10(1./(10.**(-ty(it1,js1-1,je))+10.**(-ocond1)))
      op2=dlog10(1./(10.**(-ty(it1,js1,je))+10.**(-ocond2)))
      dop1=0.5*(dop1+(op2-op1)/(tb(js1)-tb(js1-1)))
      endif
420   if(js1+2 .gt. jbend) go to 430
      if(ty(it1,js1+2,je).gt.-7.99) then
c add conductive opac. to rad. opac.
      ocond1 = ocon(tb(js1+1),ta(it1))
      ocond2 = ocon(tb(js1+2),ta(it1))
      op1=dlog10(1./(10.**(-ty(it1,js1+1,je))+10.**(-ocond1)))
      op2=dlog10(1./(10.**(-ty(it1,js1+2,je))+10.**(-ocond2)))
      dop2=0.5*(dop2+(op3-op1)/(tb(js1+2)-tb(js1+1)))
      endif
430   dkrx(k)=dop2+(dop2-dop1)/(tb(js1+1)-tb(js1))*(xb-tb(js1+1))
      go to 500
c..........................................................................
450   continue
      jc=1
      jco=0
      js1=j
      if(j.gt. 11) jc=-1
455   do 460 ljs=1,jbend
      js1=js1+jc
      if(js1.ge.jbend .or. js1.le.1) go to 465
      if(ty(it1,js1,je) .gt. -7.99) go to 470
460   continue
465   if(jco .ne. 0) go to 50 
      jco=jco+1
      jc=-jc
      go to 455
470   if(jc .eq. -1) js1=js1-1
c add conductive opac. to rad. opac.
      ocond1 = ocon(tb(js1+1),ta(it1))
      ocond2 = ocon(tb(js1),ta(it1))
      op1=dlog10(1./(10.**(-ty(it1,js1+1,je))+10.**(-ocond1)))
      op2=dlog10(1./(10.**(-ty(it1,js1,je))+10.**(-ocond2)))
      dkrx(k)=(op1-op2)/(tb(js1+1)-tb(js1))
      op(k)=op1+dkrx(k)*(xb-tb(js1+1))
c      write(17,*) 'op bef 500',ty(it1,js1+1,je),ocond1
c      write(17,*) 'op bef 500',it1,js1+1,op1,op2,op(k)
c..........................................................................
500   continue
      dlklt=(op(2)-op(1))/(ta(i+1)-ta(i))
      opcs=op(2)+dlklt*(xa-ta(i+1))
      if(ider .eq. 0) return
      dlklr=dkrx(2)+(dkrx(2)-dkrx(1))*(xa-ta(i+1))/(ta(i+1)-ta(i))
      return
c..........................................................................
50    continue
       write(6,620) xa, xb
620    format(1h ,52hargument of opacity table is range out for log(t) =
     1  ,f8.5, 16h, and log(rho) = , f8.4 )
      stop
      end 
c***************************************************************
c Lagrange interpolating polynomial
c***************************************************************
      subroutine fh4(x,yt,xt,y,dydx,indh)
      implicit double precision(a-h,o-z)
      real*8 yt(4),xt(4),xmt(4)
      a1=yt(1)/((xt(1)-xt(2))*(xt(1)-xt(3))*(xt(1)-xt(4)))
      a2=yt(2)/((xt(2)-xt(1))*(xt(2)-xt(3))*(xt(2)-xt(4)))
      a3=yt(3)/((xt(3)-xt(1))*(xt(3)-xt(2))*(xt(3)-xt(4)))
      a4=yt(4)/((xt(4)-xt(1))*(xt(4)-xt(2))*(xt(4)-xt(3)))
      do 100 k=1,4
100   xmt(k)=x-xt(k)
      y=a1*xmt(2)*xmt(3)*xmt(4)+a2*xmt(1)*xmt(3)*xmt(4) +
     1 a3*xmt(1)*xmt(2)*xmt(4)+a4*xmt(1)*xmt(2)*xmt(3)
      if( indh .eq. 0) return 
      fr=xmt(2)/(xt(3)-xt(2)) 
      if(xmt(4)*xmt(3) .lt. 0.) go to 150
      dydx1=(xt(1)-xt(4))*a1*(xmt(2)+xmt(3))
     1  +(xt(2)-xt(4))*a2*(xmt(1)+xmt(3))
     2 +(xt(3)-xt(4))*a3*(xmt(2)+xmt(1))
      go to 200
150   dydx1=(yt(4)-yt(3))/(xt(4)-xt(3)) 
      fr=-xmt(4)/(xt(4)-xt(3))
200   if(xmt(1)*xmt(2) .lt. 0.) go to 250
      dydx2 = (xt(2)-xt(1))*a2*(xmt(3)+xmt(4)) +
     1  (xt(3)-xt(1))*a3*(xmt(2)+xmt(4)) +
     2  (xt(4)-xt(1))*a4*(xmt(3)+xmt(2))
      go to 300
250   dydx2=(yt(2)-yt(1))/(xt(2)-xt(1)) 
      fr=-xmt(2)/(xt(2)-xt(1))
300   dydx=dydx2*fr+dydx1*(1.-fr)
      return 
      end
c***************************************************************
c  this guy acts like opac, but interpolates between opacities.
c  the vector x(1-3) must be set for this to work!
c  13 Nov 89 MAW
c  Added in calls to opacder under certain conditions. 4-91 PAB
c***************************************************************
      function opac3(rhol,tl)
      implicit double precision(a-h,o-z)
      common/opacfin/ifinal
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      common/icomps/icomp
      real*8 z(3),az(3)
c.............................
c initialize constants
c.............................
c      az(1)=1.0
      if (icomp.eq.1) az(1)=1.0
      if (icomp.eq.2) az(1)=3.0
      if (icomp.eq.3) az(1)=4.0
      az(2)=4.0
      az(3)=12.0
c      z(1)=1.0
      if (icomp.eq.1) z(1)=1.0
      if (icomp.eq.2) z(1)=2.0
      if (icomp.eq.3) z(1)=2.0
      z(2)=2.0
      z(3)=6.0
      opl = 0.
      jesav = je
c......................................
c pure composition (comp. > 0.999) 
c......................................
      do 50 i = 1,3
        xx = x(i)
        if(xx .gt. 0.999)then
          je = i
          if(ifinal.eq.0)then
 	    opac3 = opac(rhol,tl)
           else
            call opacder(rhol,tl,opcs,dlklr,dlklt)
            opac3 = opcs
          endif
          je = jesav
          return
        endif
50	continue
c        write(17,*) 'after first loop, je=',je,x(3)
c...........................................................
c mixtures
c...........................................................
	do 100 i = 1,3
	   xje = x(i)
	   if ( xje.gt. 0.001) then
	     je = i
c             if(ifinal.eq.0)then
                opl = opl + xje * 10.**opac(rhol,tl)
c              else
c               call opacder(rhol,tl,opcs,dlklr,dlklt)
c               opl = opl + xje * 10.**opcs
c                write(17,11) i,xje,rhol,tl,opcs,dlklr,dlklt
c             endif
	   endif
100	continue
c11      format(1x,'opacs',i3,6(1pe12.5,1x))
	je = jesav
        opac3   = dlog10(opl)
999     return
	end
c**************************************************************************
c  is it liquid metal, or is it xtal lattice?
c  ac(6) contains the melting coefficients for carbon.
c**************************************************************************
      function phasec(t)
      implicit double precision(a-h,o-z)
      common/ac/ac(6)
c...........................................................
      phasec = ac(6)
      do 1 i=1,5
         phasec = phasec * t + ac(6-i)
1     continue
      return
      end 
c**************************************************************************
c  is it liquid metal, or is it xtal lattice?
c  ao(6) contains the melting coefficients for oxygen.
c**************************************************************************
      function phaseo(t)
      implicit double precision(a-h,o-z)
      common/ao/ao(6)
c...........................................................
      phaseo=ao(6)
      do 1 i=1,5
         phaseo = phaseo * t + ao(6-i)
1     continue
      return
      end 
c**************************************************************************
c Numerical integration via Simpson's rule. (From Numerical Recipies)
c**************************************************************************
      subroutine qsimp(func,a,b,s)
      implicit double precision(a-h,o-z)
      external func
c...........................................................
      parameter(eps = 1.d-6, jmax=25) 
c...........................................................
      ost = -1.e-30
      os =  -1.e-30
      do 11 j = 1,jmax
        call trapzd(func,a,b,st,j)
        s = (4.*st-ost)/3. 
        if ( dabs(s-os) .lt. eps*dabs(os)) return
        os = s
        ost = st 
11    continue
      write(*,*) 'qsimp: too many steps'
      write(*,*) 'exiting.'
      stop
      end
c**************************************************************************
c  read in the Heubner opacities and the envelope eos.
c  then compute the optical depths.
c**************************************************************************
      subroutine read1
      implicit double precision(a-h,o-z)
      dimension dummy(7)
      parameter (nd=38)
c      common/tenv/rhok(3,60,35),eta(3,60,35),pp(3,60,35),datg(3,60,35),
c     1 od(3,18,35),tk(3,60),uu(3,60,35),xtt(3,60,35),xrt(3,60,35)
      common/tenv/rhok(3,60,nd),eta(3,60,nd),pp(3,60,nd),datg(3,60,nd),
     1 od(3,18,nd),tk(3,60),uu(3,60,nd),xtt(3,60,nd),xrt(3,60,nd)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
c      common/sl/sl(11)
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1 ,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
      common/d/ ml(3,60),nmax,kind,lmax,nsuff,nlim,nlim1,j,iter,lmam, 
     1 nlum,ko,nhp
      common/opacs/cso(3,29,8),csr(29),cst(29)
      common/dirac/ psi(141),f12(141),f32(141)
      common/ac/ac(6)
      common/ao/ao(6)
      common/alpha/alph(4),qmix1,qmix2,zone(2,2)
      common/opacfin/ifinal
      common/opcswch/first
      common/icomps/icomp
      logical first
c..................................................
c  melting coefficients for carbon.
c..................................................
      data ac/1.24605188e3,-9.32465995e2,2.776543312e2,-4.080201741e1,
     1       2.979129728e0,-8.665569421e-2/
c..................................................
c  melting coefficients for oxygen from a fit to the output of
c  xtal for temps in the range 5.4 < log(t) < 8.0.
c..................................................
      data ao/-2513.772484, 1823.347553, -528.588773, 76.834498,
     1       -5.580515, 0.161850/
c..................................................
c format statements 
c..................................................
2     format(i5,i3,f7.4,2i3)
31    format(1x,f5.1,2(1x,f10.6),40x)
1004  format(f16.6,8f8.2)
1005  format(8f10.4)
1006  format(1p,2e12.2,0p,4f8.4,' < amhyhe, amheca, alph(1-4)')
c..................................................
c     read the mass boundaries and alpha's for diffusion calculation
c (changed, now read from input (first line))
c..................................................
      read *, amhyhe,amheca, alph(1),alph(2),alph(3),alph(4)
      write(1,1006) amhyhe,amheca, alph(1),alph(2),alph(3),alph(4)
      write(9,1006) amhyhe,amheca, alph(1),alph(2),alph(3),alph(4)
      write(17,1006) amhyhe,amheca, alph(1),alph(2),alph(3),alph(4)
      write(20,1006) amhyhe,amheca, alph(1),alph(2),alph(3),alph(4)
      write(26,1006) amhyhe,amheca, alph(1),alph(2),alph(3),alph(4)
c..................................................
c     read control parameters and the atomic number z and mass az
c     for hydrogen, helium, and carbon. 
c..................................................
      read(15,2) nmax,lmax,err,lmam,nlum 
      first = .true.
      ifinal = 0
c      az(1)=1.0
      if (icomp.eq.1) az(1)=1.0
      if (icomp.eq.2) az(1)=3.0
      if (icomp.eq.3) az(1)=4.0
      az(2)=4.0
      az(3)=12.0
c      z(1)=1.0
      if (icomp.eq.1) z(1)=1.0
      if (icomp.eq.2) z(1)=2.0
      if (icomp.eq.3) z(1)=2.0
      z(2)=2.0
      z(3)=6.0
c..................................................
c     define the eos grid
c..................................................
      do 200 nel = 1,3
        lunit = nel + 1
          read ( lunit, * ) dum
        do 175 l = 1, lmax
          read (lunit,*) ml(nel,l)
          kmax = ml(nel,l)
          do 150 k = 1,kmax 
            read (lunit,*) tk(nel,l),rhok(nel,l,k),pp(nel,l,k),
     1        uu(nel,l,k),xtt(nel,l,k),xrt(nel,l,k),
     2        datg(nel,l,k),eta(nel,l,k) 
150       continue
c      write(*,*) 'nel,l,k',nel,l,k
175     continue
        rewind lunit
200   continue
c..................................................
c read the Heubner radiative opacities cso on the density-
c temperature grid csr-cst
c Used to be Cox-Stewart opacities.
c..................................................
      read(15,1004) (cst(l),csr(l),dummy,l=1,29)
      read(15,1005) (((cso(jm,l,k),k=1,8),l=1,29),jm=1,3)
c..................................................
c compute optical depths(log) od for first lmam isotherms
c..................................................
      gs=1.e+08
      do 23 jm=1,3
        je=jm
        do 22 l=1,lmam
          kmax=ml(jm,l)
          tau=10.**(opac(rhok(jm,l,1),tk(jm,l))+pp(jm,l,1))/gs*0.5
          od(jm,l,1)=dlog10(tau)
          do 21 k=2,kmax
            tau=tau+0.5/gs*(10.**pp(jm,l,k)-10.**pp(jm,l,k-1))
     1            *(10.**opac(rhok(jm,l,k),tk(jm,l))
     2            +10.**opac(rhok(jm,l,k-1),tk(jm,l)))
            od(jm,l,k)=dlog10(tau)
21        continue
22      continue
23    continue
c..................................................
c     read fermi-dirac f12,f32 functions in the range of degeneracy
c     parameter -4 < psi < 10 
c..................................................
      do 30 k=1,141 
        read(15,31) psi(k),f32(k),f12(k)
30    continue
c..................................................
c message to user
c..................................................
      print 321
321   format(' eos tables and tape5 read')
      return
      end 
c**********************************************************************
c  read model and some more control parameters
c  also contains write1, write2, and write3.
c**********************************************************************
      subroutine read2
      implicit double precision(a-h,o-z)
      common/shells/ sa(400),ra(400),ba(400),pa(400),ta(400),
     1 ea(400),xca(400),fca(400),s(400),r(400),b(400),
     2 p(400),t(400),e(400),xc(400),sk(400),
     3 rk(400),bk(400),pk(400),tk(400)
      common/thermo/u2,up2,ut2,e2,ep2,et2,psi2,pg,o2,op2,ot2,fp,ft,
     1 en2(5),fn,fcc,ce,ci,cif,w,nu
      common/surf/u(2,3),v(2,3),ww(2,3),rm,bm,is,ks,ls,ms,kstart,npass
      common/temp/s1,r1,s2,r2,b2,p2,t2,ea2,xc2,xo2,fca2,f2,q2,w2,c
      common/contrl/ds,g,sm,wc,it,nite,ja,jb,j,k,l
      common/xx/sin,sout,smid,grid1,grid2,ucent,cste,kon,kom
      common/times/tmax1,tmax2,tmax3,time,time1,f,dg,dg1,g1,ip5,ip6,ip8
      common/terp/stpms,rat1
      common/vca/modnr
      common/dvca/sgtmp
      common/xbnd/amxc,amxo,gold
      common/ixswch/ixswch
      common/corats/ams(10),delmass(10),corat(10),dcorat(10),
     1              ncore, irdold, alph 
      common/fin/ifin
      common/dfin/rfin,blfin
      common/tfit/tfit(100),told,ifit,itfit,nite1
      common/gne/gnew
c......................................................... 
      dimension bn(5)
      dimension x(400,7),y(400,7),z(400,5)
      dimension cp(400), cv(400)
c......................................................... 
      equivalence (sa(1),x(1,1)),(s(1),y(1,1)),(sk(1),z(1,1))
c......................................................... 
c Format Statements for READ2
c......................................................... 
1     format(12i4)
2     format(f6.3,f10.6,e14.7,e10.3,f5.2,2e9.2,i4,f9.6)
3     format(2e16.8,4f8.5,i4,f5.2)
4     format(7f7.5) 
7     format(i10/(e15.8,f9.6,1pe11.4,0pf10.6,f9.6,e14.7,f8.6))
71    format(i10/(e15.8,f9.6,1pe11.4,0pf10.6,f9.6,e14.7,f8.6,f8.6))
8     format(4i10/2f10.6/(6d13.7))
20    format(e15.8,f9.6,e15.8,f9.6,2e15.8)
21    format(2f9.3,f10.6)
103   format(///20h    **** model nr.  ,i4,7h  ***  ,f6.3,
     1  43h solar masses (h - he - c/o ))  ***  age = ,1pd14.8,
     2  11h years ****/)
104   format(//6h  ip5=i4,6h  ip6=i4,6h  ip7=i4,7h  nmod=i4,8h  nite1=
     1i4,4h  m=i4,5h  md=i4,5h  nn=i4,6h  ip8=i4,6h    w=f8.3)
114   format(60x,4hsin=f7.4,6h sout=f7.4,7h  smid=f7.4,7h grid1=f7.4, 
     1 7h grid2=f7.4/)
105   format(4h dg=f10.6,4h  g=f10.6,4h sg=e16.8,7h stpms=e11.4,
     1 4h  f=f10.6,8h  tmax1=e10.4,8h  tmax2=e10.4//)
106   format  (5h  sm=e16.8,6h rat1=e16.8,5h   c=f6.3,6h   xh=f9.5,
     1 6h   yh=f9.5,6h  yyh=f9.5,6h   ja=i4//)
153   format(e15.8,f9.6,1pe11.4,0pf10.6,f9.6,e14.7,2f8.6)
160   format(f6.3,f10.6,e14.7,e10.3,f5.2,2e9.2,i4,f9.6)
c162   format(1h ,i2,f6.2,e13.6,6f8.3,7x,32hnn,sm,sg,p2,t2,ucent,rm,bl,
c     1tel)
162   format(1h ,i2,f6.2,e13.6,6f8.3,7x,30hnn,sm,sg,p2,t2,ucent,rm,bl,tel)
163   format(1h ,'amx=',f6.3,'  amxo=',f6.3)
c......................................................... 
c Format statements for WRITE2
c......................................................... 
309   format(42h         **** convection in mass-shells j=i3,
     1  7h, s(j)=, 1e16.8,12h, through j=i3,7h, s(j)=e16.8)
310   format(26h              containing  f11.5,
     1  23h solar masses with x12=f8.6,'  x16=',f8.6) 
319   format(/' s2=',1pe16.8,'  r2=',0pf12.7,'  b2=', 1pe12.4,
     1 '  p2=', 0pf11.6,'  t2=',f11.6,'  e2=',1pe14.7,
     2 '  xc2=',0pf8.6,'  xo2=',f8.6)
321   format(' q2=', 1pe12.4, '  f2=', e12.4, '  w2=', e12.4,'  ea2=',
     1 e12.4,'  psi=',e12.4,'  cp= ',e12.4,'  cv = ',e12.4) 
323   format(4h u2= e12.4,5h  o2= e12.4,4h  b=f11.6,5h  wc=f11.6,
     1 5hvesc=e12.4,9h  eratio=e12.4)
325   format(5h fte=e12.4,5h fcc=e12.4,5h  fn=e12.4,23x,2hr=e12.6,
     15h   m=f9.6)
326   format(1h ,6h bnpl=f8.4,6h bnpa=f8.4,6h bnph=f8.4,6h bnrc=f8.4, 
     16h bnbr=f8.4) 
327   format(5h epl=1pe12.4,5h eph=e12.4,5h epa=e12.4,5h erc=e12.4,
     1 5h ebr=e12.4,6h etot=e12.4)
c......................................................... 
c Format statements for WRITE3
c......................................................... 
711   format(/7h   fcc=e12.4,6h,  s2=f8.3 ,7h, xdel=f12.6,
     1 26h  max.c12 rate, x12-change)
712   format(7h    fn=e12.4,6h,  s2=f8.3 ,23h      max.neutrino rate) 
713   format(7h    bl=f12.4,6h, bcc=f12.4,6h,  bn=f12.4,7h,bgrav=f12.4,
     1 8h  egrav=e14.5/)
724   format(2h  f7.3,e14.5,3e14.4,23h  m,r,v,a,shock at peak)
725   format(9h         e14.5,2e14.4,35h                r,v,a at last sh
     1ell/)
333   format(8(1x,f8.4))
      gnew=0.0
c.......................................
c subroutine TAPE reads in carbon and oxygen interior eos table
c.......................................
      call tape
c..........................................................................
c  read in the model, and set some variables
c  note that the model is now tagged on the end of the input file
c..........................................................................
      read *,ip5,ip6,ip7,nmod,nite1,m,md,nu,ip8,ip1,irdold,ip40
      print *,    '             nmod:',nmod
      print *,    '             ip40:',ip40
      if(nu .le. 0)then
        print *,    'neutrino-less models'
      elseif(nu .eq. 1)then
        print *,    'old BPS neutrino rates'
      elseif(nu .eq. 2)then
        print *,    'Munkata etal neutrino rates'
      elseif(nu .eq. 3)then
        print *,    'Itoh etal neutrino rates'
      endif
      write(20,*)    '             nmod:',nmod
      write(20,*)    '             ip40:',ip40
      if ( irdold .eq. 1 .or. irdold .eq. 0) then
         iii = 0
 888     iii = iii + 1
         read *, ams(iii), corat(iii)
         if ( iii .gt. 1 ) then
            dcorat(iii) = corat(iii) - corat(iii - 1)
            delmass(iii) = ams(iii) - ams(iii-1)
         endif
         if ( ams(iii) .lt. 1.0 ) goto 888
         ncore = iii
      elseif ( irdold .eq. 2 ) then
         read *, alph
      endif
      read(*,3) sm,rat1,c,xh,yh,yyh,khomo,time
      read(*,4) ce,cif,sin,sout,smid,grid1,grid2
      read(*,2) dg,g,sg,stpms,f,tmax1,tmax2,modnr,g3 
      tmax3=10.*tmax2
      gold=g
      read(*,20) speak,rpeak1,vpeak1,rlast,vexp1,egrav1
      read(*,21) bgrav,w,g1
      read(*,8) ks,ls,ms,kstart,rm,bm,u,v,ww
c..........................................................................
c  here is where we read in the model.  if we are reading in
c  an old model, then we also have to specify the run of the
c  desired c/o profile.  (corat = 1 ==> pure c).
c  if irdold = 1, then do linear interpolation between the specified
c      mass points
c  if irdold = 2, then use the relation given in Barrat, Hansen,
c      & Mochkovitch 1988, A+A, 199, L15
c..........................................................................
      if ( irdold .eq. 1 ) then
         read(*, 7) jb,((y(j,i),i=1,7),j=1,jb)
         do 1000 i = 1,jb
            amr  = 10.**y(i,1) 
            iii = 2 
 999        if ( amr .gt. ams(iii) ) then
                 iii = iii + 1
                 goto 999
            endif
            xc2 = dcorat(iii) / delmass(iii) * (amr - ams(iii-1))
     1              + corat(iii-1)
            xo2 = 1 - xc2
            xc(i) = xc2
            if ( xc2 .gt. .00001 .and. xc2 .lt. .99999)then
               call istatco(p(i),t(i),0,.false.)
            elseif ( xc2 .ge. .999999) then
               call istat1(p(i),t(i),0,12,.false.)
            elseif ( xc2 .le. .000001) then
               call istat1(p(i),t(i),0,16,.false.)
            endif
            e(i) = e2
 1000    continue
         write(6,*) 'model read in and rescaled to c/o model'
         write(20,*) 'model read in and rescaled to c/o model'
c..........................................................................
      elseif ( irdold .eq. 2 ) then
         read(*, 7) jb,((y(j,i),i=1,7),j=1,jb)
         do 1001 i = 1,jb
            amr  = 10.**y(i,1) 
            if ( i .ne. jb ) then
               xo2 = .5 * ( 1. + alph ) * ( 1. - amr )**alph
            else
               xo2 = 0.
            endif
            xc2 = 1 - xo2
            xc(i) = xc2
            if ( xc2 .gt. .000001 .and. xc2 .lt. .999999) then
               call istatco(p(i),t(i),0,.false.)
            elseif ( xc2 .ge. .999999) then
               call istat1(p(i),t(i),0,12,.false.)
            elseif ( xc2 .le. .000001) then
               call istat1(p(i),t(i),0,16,.false.)
            endif
            e(i) = e2
 1001    continue
         write(6,*) 'model read in and rescaled to bhm c/o model'
         write(20,*) 'model read in and rescaled to bhm c/o model'
         corat(1) = xc(1)
         corat(2) = xc(jb)
         ncore = 2
      else
         read(*, 7) jb,((y(j,i),i=1,7),j=1,jb)
c         read(*, 71) jb,((y(j,i),i=1,7),j=1,jb)
c         write(*, 7) jb,((y(j,i),i=1,7),j=1,jb)
         do j=1,jb
           xc(j)=y(j,7)
         enddo
c         corat(1) = xc(1)
c         corat(2) = xc(jb)
c         ncore = 2
      endif
c..........................................................................
c  write header line to tape1 and tape9 
c..........................................................................
      smass=10.**(sm-33.298635)
      write(9,6) smass, dg, stpms, corat(1), corat(ncore)
      write(26,6) smass, dg, stpms, corat(1), corat(ncore)
      write(1,6) smass, dg, stpms, corat(1), corat(ncore)
    6 format('*** h/he/c-o ',f4.2,'msun wd: dg=',f6.3,
     1  ' stpms='e9.3,' co1=',f3.1,' co2=',f3.1,' ***')
      call armove(x,y,jb,7)
c..........................................................................
c perform a homology transform if requested, store new model in name.out
c..........................................................................
      if(khomo.gt.0) then
        call homo(xh,yh,yyh,jb)
      endif
      ja=jb
      time1=time
      dg1=dg
      if ( w.lt.1. ) ip8=-1
      npass=0
c..........................................................................
c finally, write header line to tape40. 
c..........................................................................
      write(40,*) '10**s2     r2     t2      p2    e2    b2      fn'
      write(40,*) '   cp(k)   cv(k)  o2  wc   w2   theta    psi2  rho'
      return
c************************************************************
c write a (header) summary of a model, then set some
c variables to start a new sequence.
c************************************************************
      entry write1
      nite=nite1
      modnr=modnr+1 
c      sg=sg+10.**(g-7.499095) 
      smass=10.**(sm-33.298635)
      write(17, 103)modnr,smass,sg
      write(17, 104) ip5,ip6,ip7,nmod,nite1,m,md,nu,ip8,w
      write(17, 114) sin,sout,smid,grid1,grid2
      write(17, 105) dg,g,sg,stpms,f,tmax1,tmax2
      write(17, 106) sm,rat1,c,xh,yh,yyh,ja
      fcc1=0.
      xdel=0.
      sc1=0.
      fn1=0.
      sn1=0.
      k1=0
      kon=0
      kom=0
      total=0.
      tmass=0.
      egrav=0.
      bcc=1e-10
      do 207 i=1,5
         bn(i)=1e-10
207   continue
      bnt=1e-10
      cste=c
      g2=dg-g3
c............................................................ 
      do 208 j=1,jb 
         xc2 = xc(j)
         xo2 = 1 - xc2
c         write(*,*) j,xc2,xo2,p(j),t(j)
         if ( xc2 .gt. .000001 .and. xc2 .lt. .999999) then 
            call istatco(p(j),t(j),0,.false.)
         elseif ( xc2 .ge. .999999) then
            call istat1(p(j),t(j),0,12,.false.)
         elseif ( xc2 .le. .000001) then
            call istat1(p(j),t(j),0,16,.false.)
         endif
         fca(j)=fcc/((1.-2.2892*fcc*10.**(g-18.)/0.999999)**2)
208   continue
c............................................................ 
c fca is fractional C abundance at time t + delta t
c See eqn 3 of Kutter & Savedoff 1969, ApJ, 157, 1021.
c............................................................ 
      return
c***************************************************************
c  called by end, write2 writes a shell-by-shell summary of 
c  the computed model
c***************************************************************
      entry write2
c..............................................................
c  compute specific heats cp and cv, and store in 
c  cp() and cv().  write out both here and along with
c  the model (to tape9).
c..............................................................
      cp(k) = et2
      cv(k) = et2 - ep2 * ut2 / up2
      xmass=10.**s2-10.**s1
      smr=10.**(sm-33.591065)
      xr=xmass*smr
      do 330 i=1,5
        bn(i)=bn(i)+en2(i)*xr
330   continue
      bnt=bnt+fn*xr 
      egrav=egrav-xmass*10.**(2.*sm-7.176004)*(10.**s2+10.**s1)/
     1 (10.**r2+10.**r1)
c..............................................................
c compute radius, velocity, acceleration, and shock at location of max.
c expansion. (At least that's what it used to do.)
c Now it computes this for m/M* = 0.1 (fixed) 
c..............................................................
      if ( speak .le. s2 .and. s1 .le. speak ) then
         rpeak=r1+(r2-r1)*(speak-s1)/(s2-s1)
         vpeak=(10.**rpeak-10.**rpeak1)*10.**(-g) 
         apeak=2.*(vpeak-vpeak1)/(10.**g+10.**g1) 
         shock=apeak*10.**(rpeak*2.-speak-sm+7.176004)
      endif
c..............................................................
c mess with core convection zone, if one is present.
c..............................................................
      if(xc2.ne.0.) xctmp=xc2/(1.+2.2892*fcc*10.**(g2-18.)/xc2) 
      if ( xc(k) .lt. 0.000001 ) xc(k)=0.000001
      if ( kon .gt. 0 ) then
         k1=k
         tmass=tmass+xmass
         total=total+xctmp*xmass
      elseif ( k1 .gt. 0 ) then
         do 308 k3=kom,k1
            xctmp=total/tmass
308      continue
         xotmp = xo2
         tmass=tmass*10.**(sm-33.2986)
         write(17, 309)kom,s(kom),k1,s(k1)
         write(17, 310) tmass,xctmp,xotmp  
         write(20,309) kom, s(kom), k1, s(k1)
         write(20,310) tmass, xctmp, xotmp
         k1=0
         kom=0
         total=0.
         tmass=0.
      endif
c..............................................................
c carbon burning stuff
c..............................................................
      if ( fcc .gt. fcc1 ) then
         fcc1=fcc
         xdel=xc(k)-xc2
         sc1=s2
      endif
 
      if ( fn .gt. fn1 ) then 
         fn1=fn
         sn1=s2
      endif
      if(it.le.0 .and. mod(modnr,ip1).ne.0 
     1 .and. mod(k,10).ne.1)then
         if ( ip40 .eq. 1 ) then
            goto 1980
         else
            return
         endif
       endif
 
      pz=10.**(pg-p2)
      eratio=10.**(u2+s2+sm-p2-r2-7.352095)/(2.-pz)
      vesc=10.**(.5*(s2+sm-r2-6.874974))
      fte=(e2-ea2)*10.**(t2-g)
      r3=10.**r2
      s3=10.**s2
      write(17,319) s2,r2,b2,p2,t2,e2,xc2,xo2
      write(17,321) q2,f2,w2,ea2,psi2,cp(k),cv(k)
      write(17,323) u2,o2,pz,wc,vesc,eratio 
      write(17,325) fte,fcc,fn,r3,s3
      write(17,327) (en2(i),i=1,5),fn
      theta = 7832.3 * dsqrt( 10.**u2 ) / 10.**t2
      gammac = 10.**(u2/3. -t2) * 3.5772e6
      gammao = gammac / 3.5772e6 * 5.7782e6
      gamma = xc2 * gammac + xo2 * gammao
      write(17, 332) theta, gamma
332   format(' theta= ',1pe12.5,' gamma= ',e12.5/)
c..........................................
c before exiting, write to tape40 if ip40=1
c..........................................
1980  if ( ip40 .eq. 0 ) return
 
      bl=dlog10(b(jb))
      tel=bl/4.-rm/2.+9.18458 
      if ( k .eq. 1 ) then
         write(40,*) '---------------------------------------------'
         write(40,1999) modnr,sg,10.**rm,10.**tel,bl
         write(40,700) jb
      endif
      write(40,2001) 10.**s2,r2,t2,p2,e2,b2,fn,
     1    cp(k),cv(k),o2,wc,w2,theta,psi2,u2
      if ( k .eq. jb ) then
         write(40,1999) modnr,sg,10.**rm,10.**tel,bl,dlog10(bnt)
      endif
1999  format(i4,1pe13.5,1pe13.5,0p,f9.0,f9.4,f9.4)
2001  format(f9.7,3f9.5,1p,3e13.5,/,3x,3e11.3,0p,2f9.5,1p,2e11.3,
     1       0pf9.4)
      return
c*************************************************************
c write header and model to tape9.
c*************************************************************
      entry write3
c stuff to do with correctly computing ages
        sg=sg+10.**(gnew-7.499095) 
c        sg=sg+10.**(g-7.499095) 
      sn1=10.**sn1
      sc1=10.**sc1
      bl=dlog10(b(jb))
      blfin=bl
      rfin=rm
      bcc=dlog10(bcc)
      do 331 i=1,5
         bn(i)=dlog10(bn(i))
331   continue
      bnt=dlog10(bnt)
c grav. potential luminosity
      bgrav=dlog10(dabs(egrav-egrav1))-g-33.591065 
c old expansion quantities. velocity and acceleration
      vexp=(10.**r(jb)-10.**rlast)*10.**(-g)
      aexp=2.*(vexp-vexp1)/(10.**g+10.**g1)
      spek=10.**speak
      rpek=10.**rpeak
      rlast=r(jb)
      rlas=10.**rlast
      write(17, 711) fcc1,sc1,xdel 
      write(17, 712) fn1,sn1
      write(17, 326) (bn(i),i=1,5)
      write(17, 713) bl,bcc,bnt,bgrav,egrav 
      write(17, 724) spek,rpek,vpeak,apeak,shock
      write(17, 725) rlas,vexp,aexp
c dump out neutrino rates to separate file
      write(51, 333) (bn(i),i=1,5),bnt,bgrav,bl
c rename variables to those of previous model.
      egrav1=egrav
      vexp1=vexp
      vpeak1=vpeak
      rpeak1=rpeak
      g1=g
      g=g2
      g3=.5*dmax1(bl,bcc)
      p2=p(1)
      t2=t(1)
      tel=bl/4.-rm/2.+9.18458 
c..................................................................... 
      if ( ixswch .eq. 1 ) then
         pfreeze = phasec(t2) 
      elseif ( ixswch .eq. 2 ) then
         pfreeze = phaseo(t2) 
      elseif ( ixswch .eq. 3 ) then
         pfreeze = xc(1) * phasec(t2) + (1.-xc(1)) * phaseo(t2)
      elseif ( ixswch .eq. 4 ) then
         pfreeze = dmin1( phasec(t2), phaseo(t2) )
      else
         write(*,*) 'read2: ixswch not set. exiting.'
         stop
      endif
      xtest = y(1,4) - pfreeze
      amxc = -10.
      amxo = -10.
      if ( xtest .ge. 0. ) then
         call xbndry
      endif
c      told=t2
      gold=g
      if ( ixswch .eq. 4 ) then
         write(1,161) modnr,sg,p2,t2,ucent,rm,tel,bl,bnt,
     1      10.**amxc,10.**amxo 
         write(9,161) modnr,sg,p2,t2,ucent,rm,tel,bl,bnt,
     1      10.**amxc,10.**amxo 
      else
         write(1,161) modnr,sg,p2,t2,ucent,rm,tel,bl,bnt,10.**amxc
         write(9,161) modnr,sg,p2,t2,ucent,rm,tel,bl,bnt,10.**amxc
         write(20,161) modnr,sg,p2,t2,ucent,rm,tel,bl,bnt,10.**amxc
         write(50,161) modnr,sg,p2,t2,ucent,rm,tel,bl,bnt,10.**amxc
         smx=10.**(sm-33.298635)
c comment out the next line for montreal model looking files
c         write(50,261) smx,modnr,0,sg,10.**bl,10.**rm,10.**tel,-4.,-17.
      endif
      write(9,160) dg,g,sg,stpms,f,tmax1,tmax2,modnr,g3
161   format(i4,1p,e12.4,0p,f7.3,2f6.3,f7.3,f9.6,2f9.4,2f7.3)
261   format(f4.1,2i5,1p,3e9.2,0p,f8.0,f9.3,f8.3)
      write(9,20) speak,rpeak1,vpeak1,rlast,vexp1,egrav1
      write(9,21) bgrav,w,g1
      write(9,8) ks,ls,ms,kstart,rm,bm,u,v,ww
      write(17, 162)nu,sm,sg,p2,t2,ucent,rm,bl,tel
      write(17, 163)10.**amxc,10.**amxo
c................................................................ 
      if ( ip7 .gt. 0 ) then
         if(mod(m,md).eq.0) then
            write(6,876) modnr,sg,10.**tel,bm,t2 
            write(26,876) modnr,sg,10.**tel,bm,t2 
            write(20,876) modnr,sg,10.**tel,bm,t2 
876         format('mod',i4,'  age:',1pe12.2,'  te:',0pf8.0, 
     1             '  l:',f6.2,'  tc:',f6.2,'  written')
            write(9,700) jb
700         format(i10)
            do 154 j=1,jb
               write(9,153) (y(j,i),i=1,7),1.-y(j,7)
154         continue
         else
            write(6,877) modnr,sg,10.**tel,bm,t2 
            write(17,877) modnr,sg,10.**tel,bm,t2 
            write(20,877) modnr,sg,10.**tel,bm,t2 
877         format('mod',i4,'  age:',1pe12.2,'  te:',0pf8.0, 
     1             '  l:',f6.2,'  tc:',f6.2,'  not written')
         endif
         m=m-1
      endif
c................................................................ 
      nmod=nmod-1
c note: before 8 23 91 this was le 0.
      if ( nmod.le.0 )then
        nshint = jb
        call eprep(nshint)
        stop
      endif
      return
      end 
c**************************************************************************
c  find the 'coordinates' in the P, T grid of four points that enclose 
c  a specified pressure and temperature.
c  NOTE: if you try to go off the grid, failsafe is visited.
c  do it too many times, and the program quits.
c**************************************************************************
      subroutine select(x,z,dz,xorder,iorder,jmin,jmax,converg)
      implicit double precision(a-h,o-z)
      common/iii/ iii
      common /ifail/ifail
      logical converg
c      real*8 xorder(1)
c      integer iorder(1)
      real*8 xorder(*)
      integer iorder(*)
c................................................................ 
1001  format(' *** failsafe visited 50 times. stop. ***')
1000  format(i4,' ieos: ',f5.2,' !c (',f5.2,1x,f5.2,')')
c................................................................ 
      itest=50
      j=(x-z)/dz+1.5e0
      zmax = jmax * dz + z
      if ( j .le. jmin ) then 
         j=jmin
         iii=iii+1
         if ( converg .and. iii .ge. itest ) then 
            write(26,1001)
            write(*,1001)
            stop
         endif
         ifail = ifail + 1
         if ( converg .and. iii .eq. 1 ) then
             write(*,1000) iii, x, z, zmax
             write(17,1000) iii, x, z, zmax
             write(20,1000) iii, x, z, zmax
c	     write(*,*) 'Off IEOS.  Stop.'
c	     stop
         endif
      elseif ( j .gt. jmax ) then
         j=jmax
         iii=iii+1
         if ( iii .ge. itest ) then
            write(26,1001)
	    write(*,*) 'x z =',x,z
            write(*,1001)
            stop
         endif
         ifail = ifail + 1
         if ( converg .and. iii .eq. 1 ) then
             write(17,1000) iii, x, z, zmax
             write(20,1000) iii, x, z, zmax
             write(6,1000) iii, x, z, zmax
c	     write(*,*) 'Off IEOS.  Stop.'
c	     stop
         endif
      endif
c................................................................ 
      ii=j
      jl=0
      jr=0
      do 16 i=1,4
         xorder(i)=z+(ii-1)*dz
         iorder(i)=ii
         if(j+jr-jmax) 12,15,12
12       if(j-jl-jmin) 13,14,13
13       if((xorder(i)-x)*dz) 14,15,15
14       jr=jr+1
         ii=j+jr
         go to 16
15       jl=jl+1
         ii=j-jl
16    continue
      return
      end 
c**************************************************************************
c  this routine is called once.  after reading in the desired
c  alph() constants (diffusion exponents), compute the
c  integrals 1 thru 4 to calibrate qmix1 and qmix2 and the upper
c  and lower boundaries of both transition zones.  integration
c  is by the trapazoidal rule.  the integration subroutines
c  are by press et al.
c       qmix1, qmix2: mass where concentrations of two species are equal
c       zone(1,1):    upper h transition zone boundary
c       zone(1,2):    lower h transition zone boundary
c       zone(2,1):    upper he transition zone boundary
c       zone(2,2):    lower he transition zone boundary
c  nominal values derived using 'trace' approximation and assuming
c  diffusive equilibrium:
c       alph(1): 5
c       alph(1): -5/4
c       alph(1): 2
c       alph(1): -2/3
c  MAW 2/14/88
c**************************************************************************
      subroutine setprof
      implicit double precision(a-h,o-z)
      common/alpha/alph(4),qmix1,qmix2,zone(2,2)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      external func1,func2,func3,func4
c........................................................... 
      zero = 0.
      cent = 100.
      one  = 1.
      write(*,*) 'alpha:', alph
      call qsimp(func1,zero,   one, aint1)
      call qsimp(func2, one,  cent, aint2)
      call qsimp(func3,zero,   one, aint3)
      call qsimp(func4, one,  cent, aint4)
c........................................................... 
      write(6,*)
      write(6,10) 1,aint1
      write(6,10) 2,aint2
      write(6,10) 3,aint3
      write(6,10) 4,aint4
      write(17,*)
      write(17,10) 1,aint1
      write(17,10) 2,aint2
      write(17,10) 3,aint3
      write(17,10) 4,aint4
10    format('Integral of Func',i1.1,' = ',f10.5)
c........................................................... 
      qmix1 = amhyhe / ( aint1 + aint2 )
      qmix2 = amheca / ( aint3 + aint4 )
c........................................................... 
      write(17,30) 1,qmix1
      write(17,30) 2,qmix2
30    format('Qmix',i1.1,' = ',e12.5)
c........................................................... 
      zone(1,1) =   0.01 * qmix1
      zone(1,2) = 100.   * qmix1
      zone(2,1) =   0.01 * qmix2
      zone(2,2) = 400.   * qmix2
c........................................................... 
      write(17,20) 1,1,zone(1,1)
      write(17,20) 1,2,zone(1,2)
      write(17,20) 2,1,zone(2,1)
      write(17,20) 2,2,zone(2,2)
20    format('Zone',i1.1,3x,i1.1,' = ',e12.5)
      return
      end
c*************************************************************************
c Given arrays X & Y (Y=f(X)) of length n containing a tabulated
c function, and given the values yp1 and ypn of the first 
c derivatives at the endpoints, this routine returns an array Y2
c of length n containing the second derivatives ofthe interpolating
c function at the tabulated points of X.
c If yp1 and/or ypn are 10^{30} or larger, the routine sets the
c BC's for a natural spline, with zero second derivatives at that
c boundary.
c Routine is from Numerical Recipies
c*************************************************************************
      subroutine spline(x,y,n,yp1,ypn,y2)
      implicit double precision(a-h,o-z)
      parameter (nmax=10000)
      dimension x(n),y(n),y2(n),u(nmax)
      if(yp1.gt.0.99e30) then
        y2(1)=0.
        u(1)=0.
      else
        y2(1)=-0.5
        u(1)=(3./(x(2)-x(1)))*((y(2)-y(1))/(x(2)-x(1))-yp1)
      endif
      do 11 i=2,n-1
        sig=(x(i)-x(i-1))/(x(i+1)-x(i-1))
        p=sig*y2(i-1)+2.
        y2(i)=(sig-1.)/p
        u(i)=(6.*((y(i+1)-y(i))/(x(i+1)-x(i))-(y(i)-y(i-1))
     1      /(x(i)-x(i-1)))/(x(i+1)-x(i-1))-sig*u(i-1))/p
11    continue
      if(ypn.gt.0.99e30) then
        qn=0.
        un=0.
      else
        qn=0.5
        un=(3./(x(n)-x(n-1)))*(ypn-(y(n)-y(n-1))/(x(n)-x(n-1)))
      endif
      y2(n)=(un-qn*u(n-1))/(qn*y2(n-1)+1.)
      do 12 k=n-1,1,-1
        y2(k)=y2(k)*y2(k+1)+u(k)
12    continue
      return
      end
c*************************************************************************
c given the input arrays xa & ya (y=f(x)) of length n, tabulating
c a function, and given the spline coefficients y2a (from spline)
c and given an evaluation point x, this routine returns a cubic-spline
c interpolated value y.
c Routine is from Numerical Recipies
c*************************************************************************
      subroutine splint(xa,ya,y2a,n,x,y)
      implicit double precision(a-h,o-z)
      dimension xa(n),ya(n),y2a(n)
c find place in table
      call hunt(xa,n,x,klo)
      khi = klo + 1
      h=xa(khi)-xa(klo)
      if (h.eq.0.)then
c xa points are not distinct, DUMMY!
        write(*,*)  'bad xa input.'
        stop
      endif
      a=(xa(khi)-x)/h
      b=(x-xa(klo))/h
c evaluate spline
      y=a*ya(klo)+b*ya(khi)+
     *      ((a**3-a)*y2a(klo)+(b**3-b)*y2a(khi))*(h**2)/6.
      return
      end
c*************************************************************************
c  stitch together the envelope and the interior. 
c*************************************************************************
      subroutine stitch(hbp,hbt,hba,hrp,hrt,hra,dp2,dt2)
      implicit double precision(a-h,o-z)
      common/tablec/tablec(6,39,87)
      common/tableo/tableo(6,36,87)
      common/shells/shells(8000)
      common/temp/s1,r1,s2,r2,b2,p2,t2,e2,xc2,xo2,fca2,f2,q2,w2,c
      common/surf/u(2,3),v(2,3),w(2,3),r,b,i,k,l,m,kstart,npass
      dimension w1(2,2),ww2(2,2),w3(2,2),w4(2,2),h(2,2),del(2),kk(3)
      data fnat/0.4342945/
c..........................................................................
c initialize variables
c..........................................................................
      b=dlog10(b2)
      do 1 i=1,3
        kk(i)=0
1     continue
c..........................................................................
c  start loop where we try to trap luminosity and radius.
c  2 is the top of the loop.
c..........................................................................
2     g1=k*((w(1,2)-w(1,3))*(r-w(2,2))+(w(2,3)-w(2,2))*(b-w(1,2)))
      if ( g1 .lt. 0. ) then
        do 4 i=1,2 
          w(i,1)=w(i,3)+w(i,2)-w(i,1) 
4       continue
        k=-k
        m=m-3*l
        l=-l
        kk(1)=1
        go to 2
      endif
      g2=k*((w(1,3)-w(1,1))*(r-w(2,3))+(w(2,1)-w(2,3))*(b-w(1,3)))
      if ( g2 .lt. 0. ) then
         do 7 i=1,2 
            w(i,2)=2*w(i,1)-w(i,2)
7        continue
         k=-k
         kk(2)=1
         go to 2
      endif
      g3=k*((w(1,1)-w(1,2))*(r-w(2,1))+(w(2,2)-w(2,1))*(b-w(1,1)))
      if ( g3 .lt. 0. ) then
         do 10 i=1,2
            w(i,3)=2*w(i,1)-w(i,3)
10       continue
         k=-k
         kk(3)=1
         go to 2
      endif
c..........................................................................
c this is the bottom of the loop
c now that we've trapped the values, let's get on with it
c..........................................................................
      if ( kstart .eq. 1 ) then
         kk(1)=1
         kk(2)=1
         kk(3)=1
         kstart=0
      endif
      if ( kk(1)+kk(2)+kk(3) .gt. 0 ) then
         write(11) shells
         rewind 11
         do 13 i=1,3
            if (kk(i) .gt. 0) call env
13       continue
         read(11) shells
         rewind 11
      elseif ( npass .gt. 0 ) then
         go to 17
      endif
c..........................................................................
c goes through here just after starting up the sequence. 
c..........................................................................
      do 15 i=1,2
         do 25 j=1,2
            n=m-l*j 
            cd=w(j,n)-w(j,1)
            w1(i,j)=(u(i,n)-u(i,1))/cd
            ww2(i,j)=(v(i,n)-v(i,1))/cd 
25       continue
15    continue
c..........................................................................
      call solve(w3(1,1),w3(2,1),w1,-1.0d0,0.0d0)
      call solve(w3(1,2),w3(2,2),w1,0.0d0,-1.0d0)
c..........................................................................
      do 16 i=1,2
         do 26 j=1,2
            w4(i,j)=ww2(i,1)*w3(1,j)+ww2(i,2)*w3(2,j)
26       continue
16    continue
17    del(1)=p2-u(1,1)
      del(2)=t2-u(2,1)
      r=w(2,1)
      h1=v(1,1)
      h2=v(2,1)
      do 18 i=1,2
         r=r+w3(2,i)*del(i)
         h1=h1+w4(1,i)*del(i) 
         h2=h2+w4(2,i)*del(i) 
18    continue
      h1=hba+b2-10.**h1
      h2=hra+r2-h2
      h(1,1)=hbp-w4(1,1)*b2/fnat
      h(1,2)=hbt-w4(1,2)*b2/fnat
      h(2,1)=hrp-w4(2,1)
      h(2,2)=hrt-w4(2,2)
      call solve(dp2,dt2,h,h1,h2)
      npass=npass+1 
      return
      end 
c*************************************************************************
c  called from within stitch
c*************************************************************************
      subroutine solve(u1,u2,t,c1,c2)
      implicit double precision(a-h,o-z)
      dimension t(2,2)
 
      d=t(1,1)*t(2,2)-t(2,1)*t(1,2)
      u1=(t(1,2)*c2-t(2,2)*c1)/d
      u2=(t(2,1)*c1-t(1,1)*c2)/d
      return
      end 
c*************************************************************************
c read in the specification for the interior eos, and the eos itself. 
c new version here reads c eos from tape13 and o eos from tape14,
c while the control parameters are still read from tape5 (MAW)
c*************************************************************************
      subroutine tape
      implicit double precision(a-h,o-z)
      common/tablec/tablec(6,39,87)
      common/tableo/tableo(6,36,87)
      common/sizec/tminc,tmaxc,deltc,pminc,pmaxc,delpc,pmc(50),ipc(50),
     1 iphasec(50),ioverc,itc 
      common/sizeo/tmino,tmaxo,delto,pmino,pmaxo,delpo,pmo(50),ipo(50),
     1 iphaseo(50),iovero,ito 
c.........................................................................
 1000 format(8f10.2)
 1001 format(8i10)
c.........................................................................
c read c interior eos
C.........................................................................
      read(15,1001) itc,ioverc 
      read(15,1000) (pmc(i),i=1,itc)
      read(15,1001) (ipc(i),i=1,itc)
      read(15,1001) (iphasec(i),i=1,itc) 
      read(15,1000) pminc,pmaxc,delpc
      read(15,1000) tminc,tmaxc,deltc
      read(13) tablec
      rewind 13
      close(13)
      write(6,100) 
      write(20,100) 
100   format(' carbon interior eos table read')
c.........................................................................
c read o interior eos
c.........................................................................
      read(15,1001) ito,iovero 
      read(15,1000) (pmo(i),i=1,ito)
      read(15,1001) (ipo(i),i=1,ito)
      read(15,1001) (iphaseo(i),i=1,ito) 
      read(15,1000) pmino,pmaxo,delpo
      read(15,1000) tmino,tmaxo,delto
      read(14) tableo
      rewind 14
      close(14)
      write(6,200) 
      write(20,200) 
200   format(' oxygen interior eos table read')
      return
      end
c*************************************************************************
c Numerical integration by Trapezoid rule. (Called by QSIMP)
c From Numerical Recipies
c*************************************************************************
      subroutine trapzd(func,a,b,s,n) 
      implicit double precision(a-h,o-z)
      external func
      logical debug
      debug = .false.
c.........................................................................
      if (n.eq.1) then
         s = 0.5*(b-a)*(func(a)+func(b))
         it = 1
       else
         tnm = it 
         del = (b-a)/tnm
         x = a + 0.5 * del
         sum = 0. 
         do 11 j = 1,it
            dum = func(x)
            sum = sum + dum 
            x = x + del
31          format(1x,2f10.5)
11       continue 
         s = 0.5 * (s + (b-a) * sum/tnm )
         it = 2 * it
         if (debug) write(6,*) 'trapzd: best guess = ',s 
        endif
        return
        end
c*******************************************************************************
c     print the structure of envelopes and convective transfer properties
c     both scaled down,  also interpolate between shells to find exact
c     physical properties when eta=0,20, at the bottom of the convection
c     zone and when the mass of the envelope is=1e-6* stellar mass
c*******************************************************************************
      subroutine write4
      implicit double precision(a-h,o-z)
      double precision m
      logical writeit
c.........................................................................
      common/shells/r(800),m(800),p(800),t(800),el1(800),ol1(800),
     1 rl1(800),atg(800),rtg(800),phs(800)
      parameter (nd=38)
c      common/tenv/rhok(3,60,35),eta(3,60,35),pp(3,60,35),datg(3,60,35),
c     1 od(3,18,35),tk(3,60),uu(3,60,35),xtt(3,60,35),xrt(3,60,35)
      common/tenv/rhok(3,60,nd),eta(3,60,nd),pp(3,60,nd),datg(3,60,nd),
     1 od(3,18,nd),tk(3,60),uu(3,60,nd),xtt(3,60,nd),xrt(3,60,nd)
      common/je/je
      common/comp/amhyhe,amheca,x(4)
      common/xcomp/xcomp(400,3)
      common/slr/sl(11)
      common/c/az(3),z(3),taux,ts,ps,sm,rs,err,te,gs,rho,ol1x,rl1x,el1x
     1 ,atgx,rtgx,tl,pl,xmass,acu,am,bk,an0,pi
      common/d/ ml(3,60),nmax,kind,lmax,nsuff,nlim,nlim1,j,iter,lmam, 
     1 nlum,ko,nhp
      common/dirac/ psi(141),f12(141),f32(141)
      common/grad/ drdtp(400),cp(400),ttg(400),drdtpx,cpx,pecx,ttgx,
     1 deptx,dpdr,dtdr,dmdr,dr,xr,xt,cv,drdptx
      common/surf/u(2,3),v(2,3),w(2,3),rm,bm,is,ks,ls,ms,kstart,npass 
      common/terp/stpms,rat1
      common/prop/ bn2(400),ac2(400)
      common/writeit/writeit
      common/oldval/bold,teold 
      common/mixl/aml,mls
      dimension s(400),d(400),nbn2(400),amfrac(400)
c.........................................................................
c Format statements
c.........................................................................
12    format(/13h log(m/msun)=,f6.3,2x,13h log(r/rsun)=,f6.3,2x,
     1  11h log(grav)=,f6.3,2x,11h log(teff)=,f6.3,2x,13h log(l/lsun)=,
     2  f6.3,2x,12h log(l*/m*)=,f6.3// 12h iterations=,i2,
     3  59h  nc=-1 if convv + nondeg, 0 if convv + deg, +1 if nonconvv/)
33    format(' too few shells')
37    format(4x,6hradius,5x,11hlog(1-r/rs),2x,11hlog(1-m/ms),2x,4hlogp,
     1 4x,4hlogt,4x,6hlogrho,4x,7hopacity,5x,3heta,5x,3httg,8x,3hatg,8x,
     2 3hrtg,6x,2hnc,3x,4hne/z)
900   format(/,4x,6hradius,5x,11hlog(1-r/rs),2x,11hlog(1-m/ms),10x,3hbn2
     1,11x,3hac2,8x,2hje,4x,4hnbn2)
38    format(2x,1pe11.5,3x,0pf8.4,5x,f8.4,1x,2(1x,f7.3),2x,f7.3,3x,
     1 1pe9.3,2x,0pf6.2,3(2x,e9.3),2x,i2,3x,f5.3) 
902   format(2x,1pe11.5,3x,0pf8.4,5x,f8.4,2(2x,1pe16.9),0p,3f12.9,i5)
101   format(/1x,8henvelope,2x,9hstructure,16x,6hscale=,i2,5x,
     1 ' total shells: ',i5)
103   format('log(1-r/rs)=',f8.4,2x,'log(1-m/ms)=',f8.4,2x,'logp=',
     1 f7.3,2x,'logt=',f6.3,2x,'logrho=',f6.3,2x,'logkappa=',f6.3,2x,
     2 'eta=',f6.2)
104   format(//
     1  57h physical conditions at the bottom of the convection zone/)
106   format(///////31h convective transfer properties,16x,6hscale=i2)
107   format(/4x,6hz=rs-r,5x,11hlog(1-r/rs),2x,11hlog(1-m/ms),3x,
     1    7hlocal g,6x,        6hpeclet,6x,3httg,6x,5hsuper,5x,
     1  10hmix length,2x,5hfc/ft,3x,9hconv velo,3x,5hv/vth)
108   format(2x,1pe11.5,3x,0pf8.4,5x,f8.4,4x,1pe9.3,4x,e9.3,3x,0pf5.3,
     1 3x,1pe9.3,3x,e9.3,3x,0pf5.3,3x,1pe9.3,3x,0pf5.3)
109   format(//' physical conditions when m(env)/m(star)=',1pd14.6/)
c..................................................
c     envelope structure
c..................................................
      rads=r(1)
c*      nmod=(nlim/100)+1
      nmod=1
      if ( writeit ) write(26,101) nmod,nlim
      alc4pig = -6.07666 - 2. * dlog10(gs)
      amass = 10.**sm * 1.989e33
      do 4 n=1,nlim 
987     format('m(n): ',f20.17)
        if( 1.-r(n)/rads .ge. 1.d-18 ) then
           d(n) = dlog10( 1.-r(n)/rads )
        else
           d(n) = -10000.
        endif
        amfrac(n)=10.**m(n)/sm
        ame1=1.d0-amfrac(n)
        if (ame1 .ge. 1.d-18) then
           s(n)=dlog10(ame1)
        else
            s(n) = -10000.
        endif
4     continue
c..................................................................
      xs=dlog10(sl(j))
      xr=dlog10(rs) 
      xg=dlog10(gs) 
      xt=dlog10(te) 
      xm=dlog10(sm) 
      ratio=xs-xm
      if ( writeit ) write(26,12) xm,xr,xg,xt,xs,ratio,iter
      if ( nsuff .le. 0 ) then
         write(26,33)
         write(6,33)
         write(20,33)
      endif
      if ( writeit ) write(26,37)
c..................................................................
      ion=-1
      do 11 n=1,nlim,nmod
         if ( atg(n) .gt. rtg(n) ) then 
            nc=1
         else
            if ( el1(n) .lt. 0. ) then
               nc=-1
            else
               nc=0 
            endif
         endif
         opy=10.**ol1(n)
c....................................................
c     compute degree of ionization xion 
c....................................................
         if ( ion .gt. 0. ) then
            xion=1.0
         else
            zzz = xcomp(n,1)*z(1) + xcomp(n,2)*z(2) + xcomp(n,3)*z(3) 
c            xion=10.**(1.5*t(n)-rl1(n))*am/bk/zzz/1.329153
c     1             *f12(el1(n))
            if ( xion .gt. 1.0) then
               xion=1.0
               ion=1
            endif
         endif
         if ( writeit ) write(26,38) r(n),d(n),s(n),p(n),t(n),rl1(n),
     1         opy,el1(n),ttg(n),atg(n),rtg(n),nc,xion
11    continue
c....................................................
c end of loop
c....................................................
c     physical properties at places of interest
c....................................................
      it=1
      stop=dlog10(1.-10.**stpms)
      do 90 n=1,nlim
         l=n
         if ( s(n) .gt. stop) goto 91
90    continue
91    slope=(s(l-1)-stop)/(s(l-1)-s(l)) 
c.......................................
c returns to 22 from below
c.......................................
22    dfin=d(l-1)+(d(l)-d(l-1))*slope
      sfin=s(l-1)+(s(l)-s(l-1))*slope
      pfin=p(l-1)+(p(l)-p(l-1))*slope
      tfin=t(l-1)+(t(l)-t(l-1))*slope
      rfin=rl1(l-1)+(rl1(l)-rl1(l-1))*slope
      ofin=ol1(l-1)+(ol1(l)-ol1(l-1))*slope
      efin=el1(l-1)+(el1(l)-el1(l-1))*slope
      if ( it .le. 1 ) then
         stp = 10.**stop
         if ( writeit ) write(26,109) stp
         if ( writeit ) write(26,103) dfin,sfin,pfin,tfin,rfin,ofin,efin
         slum=sl(1) 
         u(1,is)=pfin
         u(2,is)=tfin
         v(1,is)=dlog10(slum) 
         v(2,is)=dlog10(1-10.**dfin)+w(2,is)
         it=it+1
c..............................................................
c  skip over the next section with the following statement. 
c  the next section computes the convective properties.
c  it's caused some problems in the hot models
c..............................................................
         if ( nlim1 .le. 0 ) go to 30
         do 25 n=kind,nlim
            l=n
            if ( atg(n) .gt. rtg(n) ) goto 26
25       continue
26       slope=(atg(l-1)-rtg(l-1))/(atg(l-1)-rtg(l-1)-atg(l)+rtg(l))
         go to 22
      endif
c..............................................................
c bottom of loop
c..............................................................
      if ( writeit ) write(26,104)
      if ( writeit ) write(26,103) dfin,sfin,pfin,tfin,rfin,ofin,efin
c..............................................................
c convection zone
c..............................................................
      nmod1=((nlim1-kind)/100)+1
      if ( writeit ) write(26,106) nmod1 
      if ( writeit ) write(26,107)
c..............................................................
c compute geometrical depth zc, local gravity glocl, superadiabaticity
c super, ratio of convective to total flux fcf, convective velocity
c conv, ratio of convective to thermal velocities vcvt and peclet
c number
c..............................................................
      do 52 n=kind,nlim1,nmod1
         zc=rads-r(n)
         glocl=27398.*10.**m(n)*(6.96e+10/r(n))**2
         super=ttg(n)-atg(n)
         fcf=1.-ttg(n)/rtg(n) 
         if ( fcf .le. 0.) then
            fcf=0.
            conv=0. 
         else
            conv=27398.*(te/5800.)**4*10.**(m(n)-t(n)-rl1(n))
     1           *3.90e+33*phs(n)
     1           *drdtp(n)*fcf/(16.*pi*cp(n)*r(n)**2)
            conv=conv*aml
c..............................................................
c choose mixing-length version here
c (mls) =0 Bohm-Vitense or (mls=1) Bohm & Cassinelli
c default is Bohm-Vitense version
c..............................................................
            if(mls.eq.1)then
c ml2 or ml3 version
c              conv=2.*1.414213562*conv
              conv=2.*dsqrt(2.d0)*conv
            endif
            if ( conv .gt. 0.0 ) conv = conv**0.333333
         endif
         azave = xcomp(n,1)*az(1)+xcomp(n,2)*az(2)+xcomp(n,3)*az(3) 
         vcvt=conv/(3.*bk*an0/azave*10.**t(n))**0.5
c also need to adjust the peclet #. default is Bohm-Vitense.
         peclet=cp(n)*phs(n)*conv/4.535688e-04*10.**(2.*rl1(n)+ol1(n)-
     1         3.*t(n))
         peclet=peclet*aml*aml
c Bohm & Cassinelli version 
         if(mls.eq.1)then
c           peclet=peclet*3.024272/4.535688
           peclet=peclet/dsqrt(2.d0)
         endif
         if ( writeit ) write(26,108) zc,d(n),s(n),glocl,peclet,ttg(n),
     1       super,phs(n),fcf,conv,vcvt 
52    continue
c..............................................................
30    continue
      if ( writeit ) write(26,900)
      do 901 n=1,nlim,nmod
         nbn2(n)=bn2(n)/dabs(bn2(n))
         if (bn2(n) .ge. 0.) then
            bn2(n)=dlog10(bn2(n))
         else
            bn2(n) = -7.
         endif
         if (ac2(n) .ge. 0.) then
            ac2(n)=dlog10(ac2(n))
         else
            ac2(n) = -7.
         endif
         if ( writeit ) write(26,902) r(n),d(n),s(n),bn2(n), 
     1         ac2(n),xcomp(n,1),xcomp(n,2),xcomp(n,3),nbn2(n)
c         write(1,1901) amfrac(n),xcomp(n,1),xcomp(n,2),
c     1        xcomp(n,3),0.d0
901   continue
1901  format(5(2x,f15.10))
      return
      end 
c********************************************************************
c find the xtal boundary and print it's mass fraction
c********************************************************************
      subroutine xbndry
      implicit double precision(a-h,o-z)
      common/ac/ac(6)
      common/ao/ao(6)
      common/shells/ sa(400),ra(400),ba(400),pa(400),ta(400),
     1 ea(400),xca(400),fca(400),s(400),r(400),b(400),
     2 p(400),t(400),e(400),xc(400),sk(400),
     3 rk(400),bk(400),pk(400),tk(400)
      common/temp/s1,r1,s2,r2,b2,p2,t2,e2,xc2,xo2,fca2,f2,q2,w2,c
      common/contrl/ds,g,sm,wc,it,nite,ja,jb,j,k,l
      common/xbnd/amxc,amxo,gold
      common/ixswch/ixswch
      common/kill/ killit, nxmods, ikill
      logical killit
      logical oncec, onceo
      dimension y(400,7)
      equivalence (s(1),y(1,1))
c.................................................
c.. locate xtal boundary between two shells   ....
c.................................................
      jx = jb+2
      jxc = jx
      jxo = jx
      oncec = .true.
      onceo = .true.
      do 2 i=1,jb
         temper  = y(i,5)
         xc2 = xc(i)
         xo2 = 1.0-xc2
         if ( ixswch .eq. 1 ) then
            pfreeze = phasec(temper)
         elseif ( ixswch .eq. 2 ) then
            pfreeze = phaseo(temper)
         elseif ( ixswch .eq. 3 ) then
            pfrzc = phasec(temper)
            pfrzo = phaseo(temper)
            pfreeze = xc2 * pfrzc + xo2 * pfrzo
         elseif ( ixswch .eq. 4 ) then
            pfrzc = phasec(temper)
            pfrzo = phaseo(temper)
            pfreeze = dmax1( pfrzc, pfrzo )
         else
            write(20,*) 'ixswch not set. exiting.'
            write(6,*) 'ixswch not set. exiting.'
            stop
         endif
         press = y(i,4)
 876     format('xbndry: i, xc, xo, t(i), p, px: ',i3,5f9.3)
         if ( ixswch .ne. 4 ) then
            if (press .le. pfreeze) then
               jx=i-1
               goto 3
            endif
         else
            if (press .le. pfrzc .and. oncec) then
               jxc = i-1
               oncec = .false.
            endif
            if (press .le. pfrzo  .and. onceo) then
               jxo = i-1
               onceo = .false.
            endif
            if (press .le. pfrzc .and. press .le. pfrzo ) goto 3
         endif
2     continue
c.................................................................. 
3     continue
      if ( ixswch .ne. 4 ) then
         write(17, 91)pfreeze,jx,gold
         jxold=jx
         kx = jx + 1
         temp1 = y(jx,5)
         temp2 = y(kx,5)
         pres1 = y(jx,4)
         pres2 = y(kx,4)
         if ( ixswch .eq. 1 ) then
            px1 = phasec(temp1)
            px2 = phasec(temp2)
         elseif ( ixswch .eq. 2 ) then
            px1 = phaseo(temp1)
            px2 = phaseo(temp2)
         elseif ( ixswch .eq. 3 ) then
            pxc = phasec(temp1)
            pxo = phaseo(temp1)
            px1 = xc(jx) * pxc + (1.-xc(jx)) * pxo
            pxc = phasec(temp2)
            pxo = phaseo(temp2)
            px2 = xc(kx) * pxc + (1.-xc(kx)) * pxo
         endif
         sfrac = (px1 - pres1) / (pres2 - px2 + px1 - pres1 )
         am1 = 10.**s(jx)
         am2 = 10.**s(kx)
         amxc = dlog10( am1 + sfrac*(am2 - am1) ) 
         write(17, 991)jx,pres1,pres2,px1,px2,amxc
991      format('xbndry: jx, pres1, pres2, px1, px2, amxc:',i3,5f7.2) 
         write(17,987) 10.**amxc, jx, 10.**y(jx,1), kx, 10.**y(kx,1)
         write(6,987) 10.**amxc, jx, 10.**y(jx,1), kx, 10.**y(kx,1)
987      format(' amx: ',f7.3,2('  shell',i3,': ',f7.3))
      else
         if ( jxc .ne. jb + 2 .and. jxc .ne. 0 ) then
            jxcold = jxc
            pxc=phasec(y(jxc,5))
            write(17, 91)pxc,jxc,g
            kx=jxc+1
            dmdp=(y(kx,1)-y(jxc,1))/(y(kx,4)-y(jxc,4))
            amxc= y(jxc,1)+dmdp*(pxc-y(jxc,4))
            write(17,988) 10.**amxc, jxc, 10.**y(jxc,1), kx,10.**y(kx,1)
            write(6,988) 10.**amxc, jxc, 10.**y(jxc,1), kx, 10.**y(kx,1)
988         format(' amxc: ',f7.3,2('  shell',i3,': ',f7.3))
         endif
         if ( jxo .ne. jb + 2 .and. jxo .ne. 0 ) then
            jxoold = jxo
            pxo=phaseo(y(jxo,5))
            write(17, 91)pxo,jxo,g
            kx=jxo+1
            dmdp=(y(kx,1)-y(jxo,1))/(y(kx,4)-y(jxo,4))
            amxo= y(jxo,1)+dmdp*(pxo-y(jxo,4))
            write(6,989) 10.**amxo, jxo, 10.**y(jxo,1), kx, 10.**y(kx,1)
            write(17,989) 10.**amxo, jxo, 10.**y(jxo,1), kx,10.**y(kx,1)
989         format(' amxo: ',f7.3,2('  shell',i3,': ',f7.3))
         endif
      endif
c.................................................
c.. find mx/mstar for this model              ....
c.................................................
91    format(1h ,'px,jx,g  ',e18.8,i5,e18.8)
      if ( jx .eq. jb-1 ) then
	 ikill = ikill+1
         if ( ikill .gt. nxmods ) killit = .true.
      endif
      return
      end 
c**********************************************************************
c  Read file names for tape1, tape6, tape9, and output from stdinput
c**********************************************************************
      subroutine openem
      implicit double precision(a-h,o-z)
      common/names/t1,t6,t9,t50
      common/icomps/icomp
      character*30 t1,t6,t9,t40,t50,out,log
c.................................................
10    format(a30)
c.................................................
      read(*,10) t1
      read(*,10) t6
      read(*,10) t9
      read(*,10) out
      read(*,10) log
      read(*,10) t40
      read(*,10) t50
c.................................................
      open(1,file=t1,status='unknown')
      if (icomp.eq.1) open(2,file='EEOSH',status='old')
      if (icomp.eq.2) open(2,file='EEOSHE3',status='old')
      if (icomp.eq.3) open(2,file='EEOSHE.nor',status='old')
      open(3,file='EEOSHE',status='old')
      open(4,file='EEOSC',status='old')
      open(9,file=t9,status='unknown')
      open(11,
     1     status='scratch',form='unformatted')
c      open(12,file='T12.tmp',
c     1     status='scratch',form='unformatted')
      open(13,file='IEOSC',
     1     status='old',form='unformatted')
      open(14,file='IEOSO',
     1     status='old',form='unformatted')
      if (icomp.eq.1) open(15,file='AUXIN5',status='old') 
      if (icomp.eq.2 .or. icomp.eq.3) open(15,file='AUXIN5DB',status='old') 
      open(17,file=out,status='unknown') 
      open(20,file=log,status='unknown') 
      if (icomp.eq.1) open(25,file='sqopac',status='old')
      if (icomp.eq.2 .or. icomp.eq.3) open(25,file='sqopacdb',status='old')
      open(26,file=t6,status='unknown') 
      open(40,file=t40,status='unknown') 
      open(50,file=t50,status='unknown') 
      open(51,file='neut.out',status='unknown') 
c.................................................
      rewind(1)
      rewind(9)
      rewind(11)
      rewind(12)
      rewind(17)
      rewind(20)
      rewind(26)
      return
      end
c***************************************************************
c                    subroutine smooth
c this subroutine is a quick and dirty way to fix the jump in
c in derivative quants. at the core-envelope boundary.
c If the jump is greater than a pre-set limit (see eprep),
c we come here and replace the jump by a linear interpolation
c of delad between the last core point and some predetermined
c point out in the envelope.
c This is not a terribly physical subroutine, but adjusting the
c stop mass (where it's possible) show this assumption isn't
c too awful.
c***************************************************************
      subroutine smooth(nl,mver)
      implicit double precision(a-h,o-z)
      common/dprep/aa(20,500), ecv(400), ext(400), exr(400)
c set upper limit to smoothing function at 10 points (for now)
      nup = nl + 20
c.....................................................................
c determine quant. to be smoothed and mass 
c at top and bottom of smoothing interval
c.....................................................................
      if(mver.eq.1)then
        delbot = aa(16,nl)
        deltop = aa(16,nup)
      endif
      if(mver.eq.2)then
        delbot = aa(15,nl)
        deltop = aa(15,nup)
      endif
      if(mver.eq.3)then
        delbot = aa(10,nl)
        deltop = aa(10,nup)
      endif
      if(mver.eq.4)then
        delbot = aa(9,nl)
        deltop = aa(9,nup)
      endif
      botmass = aa(2,nl)
      topmass = aa(2,nup)
c determine size of mass and delad difference
      rmass = topmass - botmass
      deladr = dabs(delbot - deltop)
      do 10 n=nl+1,nup
c special case, deltop = delbot
        if(deltop.eq.delbot)then
          if(mver.eq.1)then
            aa(16,n) = delbot
          endif
          if(mver.eq.2)then
            aa(15,n) = delbot
          endif
          if(mver.eq.3)then
            aa(10,n) = delbot
          endif
          if(mver.eq.4)then
            aa(9,n) = delbot
          endif
        endif
c.....................................................................
c determine step size in mass
        frac = (aa(2,n) - botmass) / rmass
c determine stepsize in delad
        delfrac = deladr * frac
c compute new value of delad
c delad(top) < delad(bot)
        if(deltop.lt.delbot)then
          if(mver.eq.1)then
            aa(16,n) = delbot - delfrac
          endif
          if(mver.eq.2)then
            aa(15,n) = delbot - delfrac
          endif
          if(mver.eq.3)then
            aa(10,n) = delbot - delfrac
          endif
          if(mver.eq.4)then
            aa(9,n) = delbot - delfrac
          endif
        endif
c delad(top) > delad(bot)
        if(deltop.gt.delbot)then
          if(mver.eq.1)then
            aa(16,n) = delbot + delfrac
          endif
          if(mver.eq.2)then
            aa(15,n) = delbot + delfrac
          endif
          if(mver.eq.3)then
            aa(10,n) = delbot + delfrac
          endif
          if(mver.eq.4)then
            aa(9,n) = delbot + delfrac
          endif
        endif
10    continue
      return
      end
